@@ -13,6 +13,8 @@
 #
 # Copyright Buildbot Team Members
 
+from __future__ import with_statement
+
 # strategy:
 #
 # if there is a VERSION file, use its contents. otherwise, call git to
@@ -24,16 +26,18 @@ version = "latest"
 
 try:
     fn = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'VERSION')
-    version = open(fn).read().strip()
+    with open(fn) as f:
+        version = f.read().strip()
 
 except IOError:
-    from subprocess import Popen, PIPE, STDOUT
+    from subprocess import Popen, PIPE
     import re
 
     VERSION_MATCH = re.compile(r'\d+\.\d+\.\d+(\w|-)*')
 
     try:
-        p = Popen(['git', 'describe', '--tags', '--always'], stdout=PIPE, stderr=STDOUT)
+        dir = os.path.dirname(os.path.abspath(__file__))
+        p = Popen(['git', 'describe', '--tags', '--always'], cwd=dir, stdout=PIPE, stderr=PIPE)
         out = p.communicate()[0]
 
         if (not p.returncode) and out:
