@@ -27,10 +27,11 @@ assert(gtk.Window) # in gtk1 it's gtk.GtkWindow
 from twisted.spread import pb
 
 #from buildbot.clients.base import Builder, Client
-from buildbot.clients.base import TextClient
+from buildbot.clients.base import StatusClient
+from buildbot.clients.text import TextClient
 from buildbot.util import now
 
-from buildbot.status.builder import SUCCESS, WARNINGS, FAILURE, EXCEPTION
+from buildbot.status.results import SUCCESS, WARNINGS, FAILURE, EXCEPTION
 
 '''
 class Pane:
@@ -497,8 +498,11 @@ class ThreeRowClient(pb.Referenceable):
 class GtkClient(TextClient):
     ClientClass = ThreeRowClient
 
-    def __init__(self, master):
+    def __init__(self, master, events="steps", username="statusClient", passwd="clientpw"):
         self.master = master
+        self.username = username
+        self.passwd = passwd
+        self.listener = StatusClient(events)
 
         w = gtk.Window()
         self.w = w
