@@ -57,11 +57,11 @@ class Model(base.DBConnectorComponent):
             nullable=False),
         sa.Column('buildername', sa.String(length=256), nullable=False),
         sa.Column('priority', sa.Integer, nullable=False,
-            server_default=sa.DefaultClause("0")), # TODO: used?
+            server_default=sa.DefaultClause("0")),
 
         # if this is zero, then the build is still pending
         sa.Column('complete', sa.Integer,
-            server_default=sa.DefaultClause("0")), # TODO: boolean
+            server_default=sa.DefaultClause("0")),
 
         # results is only valid when complete == 1; 0 = SUCCESS, 1 = WARNINGS,
         # etc - see master/buildbot/status/builder.py
@@ -90,12 +90,9 @@ class Model(base.DBConnectorComponent):
     # data about a build is still stored in on-disk pickles.
     builds = sa.Table('builds', metadata,
         sa.Column('id', sa.Integer,  primary_key=True),
-
-        # XXX
-        # the build number is local to the builder and (maybe?) the buildmaster
         sa.Column('number', sa.Integer, nullable=False),
-
-        sa.Column('brid', sa.Integer, sa.ForeignKey('buildrequests.id'), nullable=False),
+        sa.Column('brid', sa.Integer, sa.ForeignKey('buildrequests.id'),
+            nullable=False),
         sa.Column('start_time', sa.Integer, nullable=False),
         sa.Column('finish_time', sa.Integer),
     )
@@ -104,10 +101,11 @@ class Model(base.DBConnectorComponent):
 
     # This table contains input properties for buildsets
     buildset_properties = sa.Table('buildset_properties', metadata,
-        sa.Column('buildsetid', sa.Integer, sa.ForeignKey('buildsets.id'), nullable=False),
+        sa.Column('buildsetid', sa.Integer, sa.ForeignKey('buildsets.id'),
+            nullable=False),
         sa.Column('property_name', sa.String(256), nullable=False),
         # JSON-encoded tuple of (value, source)
-        sa.Column('property_value', sa.String(1024), nullable=False), # TODO: too short?
+        sa.Column('property_value', sa.String(1024), nullable=False),
     )
 
     # This table represents Buildsets - sets of BuildRequests that share the
@@ -120,56 +118,62 @@ class Model(base.DBConnectorComponent):
         sa.Column('external_idstring', sa.String(256)),
 
         # a short string giving the reason the buildset was created
-        sa.Column('reason', sa.String(256)), # TODO: sa.Text
-        sa.Column('submitted_at', sa.Integer, nullable=False), # TODO: redundant
+        sa.Column('reason', sa.String(256)),
+        sa.Column('submitted_at', sa.Integer, nullable=False),
 
         # if this is zero, then the build set is still pending
-        sa.Column('complete', sa.SmallInteger, nullable=False, server_default=sa.DefaultClause("0")), # TODO: redundant
-        sa.Column('complete_at', sa.Integer), # TODO: redundant
+        sa.Column('complete', sa.SmallInteger, nullable=False,
+            server_default=sa.DefaultClause("0")),
+        sa.Column('complete_at', sa.Integer),
 
         # results is only valid when complete == 1; 0 = SUCCESS, 1 = WARNINGS,
         # etc - see master/buildbot/status/builder.py
-        sa.Column('results', sa.SmallInteger), # TODO: synthesize from buildrequests
+        sa.Column('results', sa.SmallInteger),
 
         # buildset belongs to all sourcestamps with setid
-        sa.Column('sourcestampsetid', sa.Integer, sa.ForeignKey('sourcestampsets.id')),
+        sa.Column('sourcestampsetid', sa.Integer,
+            sa.ForeignKey('sourcestampsets.id')),
     )
 
     # changes
 
     # Files touched in changes
     change_files = sa.Table('change_files', metadata,
-        sa.Column('changeid', sa.Integer, sa.ForeignKey('changes.changeid'), nullable=False),
-        sa.Column('filename', sa.String(1024), nullable=False), # TODO: sa.Text
+        sa.Column('changeid', sa.Integer, sa.ForeignKey('changes.changeid'),
+            nullable=False),
+        sa.Column('filename', sa.String(1024), nullable=False),
     )
 
     # Properties for changes
     change_properties = sa.Table('change_properties', metadata,
-        sa.Column('changeid', sa.Integer, sa.ForeignKey('changes.changeid'), nullable=False),
+        sa.Column('changeid', sa.Integer, sa.ForeignKey('changes.changeid'),
+            nullable=False),
         sa.Column('property_name', sa.String(256), nullable=False),
         # JSON-encoded tuple of (value, source)
-        sa.Column('property_value', sa.String(1024), nullable=False), # TODO: too short?
+        sa.Column('property_value', sa.String(1024), nullable=False),
     )
 
     # users associated with this change; this allows multiple users for
     # situations where a version-control system can represent both an author
     # and committer, for example.
     change_users = sa.Table("change_users", metadata,
-        sa.Column("changeid", sa.Integer, sa.ForeignKey('changes.changeid'), nullable=False),
+        sa.Column("changeid", sa.Integer, sa.ForeignKey('changes.changeid'),
+            nullable=False),
         # uid for the author of the change with the given changeid
-        sa.Column("uid", sa.Integer, sa.ForeignKey('users.uid'), nullable=False)
+        sa.Column("uid", sa.Integer, sa.ForeignKey('users.uid'),
+            nullable=False)
     )
 
     # Changes to the source code, produced by ChangeSources
     changes = sa.Table('changes', metadata,
         # changeid also serves as 'change number'
-        sa.Column('changeid', sa.Integer,  primary_key=True), # TODO: rename to 'id'
+        sa.Column('changeid', sa.Integer,  primary_key=True),
 
         # author's name (usually an email address)
         sa.Column('author', sa.String(256), nullable=False),
 
         # commit comment
-        sa.Column('comments', sa.String(1024), nullable=False), # TODO: too short?
+        sa.Column('comments', sa.String(1024), nullable=False),
 
         # old, CVS-related boolean
         sa.Column('is_dir', sa.SmallInteger, nullable=False), # old, for CVS
@@ -181,7 +185,6 @@ class Model(base.DBConnectorComponent):
         # revision identifier for this change
         sa.Column('revision', sa.String(256)), # CVS uses NULL
 
-        # ?? (TODO)
         sa.Column('revlink', sa.String(256)),
 
         # this is the timestamp of the change - it is usually copied from the
@@ -194,11 +197,17 @@ class Model(base.DBConnectorComponent):
 
         # repository specifies, along with revision and branch, the
         # source tree in which this change was detected.
-        sa.Column('repository', sa.String(length=512), nullable=False, server_default=''),
+        sa.Column('repository', sa.String(length=512), nullable=False,
+            server_default=''),
+
+        # codebase is a logical name to specify what is in the repository
+        sa.Column('codebase', sa.String(256), nullable=False,
+            server_default=sa.DefaultClause("")),
 
         # project names the project this source code represents.  It is used
         # later to filter changes
-        sa.Column('project', sa.String(length=512), nullable=False, server_default=''),
+        sa.Column('project', sa.String(length=512), nullable=False,
+            server_default=''),
     )
 
     # sourcestamps
@@ -212,10 +221,10 @@ class Model(base.DBConnectorComponent):
 
         # base64-encoded version of the patch file
         sa.Column('patch_base64', sa.Text, nullable=False),
-        
+
         # patch author, if known
         sa.Column('patch_author', sa.Text, nullable=False),
-        
+
         # patch comment
         sa.Column('patch_comment', sa.Text, nullable=False),
 
@@ -225,12 +234,14 @@ class Model(base.DBConnectorComponent):
 
     # The changes that led up to a particular source stamp.
     sourcestamp_changes = sa.Table('sourcestamp_changes', metadata,
-        sa.Column('sourcestampid', sa.Integer, sa.ForeignKey('sourcestamps.id'), nullable=False),
-        sa.Column('changeid', sa.Integer, sa.ForeignKey('changes.changeid'), nullable=False),
+        sa.Column('sourcestampid', sa.Integer,
+            sa.ForeignKey('sourcestamps.id'), nullable=False),
+        sa.Column('changeid', sa.Integer, sa.ForeignKey('changes.changeid'),
+            nullable=False),
     )
 
-    # A sourcestampset identifies a set of sourcestamps
-    # A sourcestamp belongs to a particular set if the sourcestamp has the same setid
+    # A sourcestampset identifies a set of sourcestamps. A sourcestamp belongs
+    # to a particular set if the sourcestamp has the same setid
     sourcestampsets = sa.Table('sourcestampsets', metadata,
         sa.Column('id', sa.Integer,  primary_key=True),
     )
@@ -253,13 +264,20 @@ class Model(base.DBConnectorComponent):
         sa.Column('patchid', sa.Integer, sa.ForeignKey('patches.id')),
 
         # the repository from which this source should be checked out
-        sa.Column('repository', sa.String(length=512), nullable=False, server_default=''),
+        sa.Column('repository', sa.String(length=512), nullable=False,
+            server_default=''),
+
+        # codebase is a logical name to specify what is in the repository
+        sa.Column('codebase', sa.String(256), nullable=False,
+            server_default=sa.DefaultClause("")),
 
         # the project this source code represents
-        sa.Column('project', sa.String(length=512), nullable=False, server_default=''),
+        sa.Column('project', sa.String(length=512), nullable=False,
+            server_default=''),
 
         # each sourcestamp belongs to a set of sourcestamps
-        sa.Column('sourcestampsetid', sa.Integer, sa.ForeignKey('sourcestampsets.id')),
+        sa.Column('sourcestampsetid', sa.Integer,
+            sa.ForeignKey('sourcestampsets.id')),
     )
 
     # schedulers
@@ -343,7 +361,8 @@ class Model(base.DBConnectorComponent):
     sa.Index('builds_brid', builds.c.brid)
     sa.Index('buildsets_complete', buildsets.c.complete)
     sa.Index('buildsets_submitted_at', buildsets.c.submitted_at)
-    sa.Index('buildset_properties_buildsetid', buildset_properties.c.buildsetid)
+    sa.Index('buildset_properties_buildsetid',
+            buildset_properties.c.buildsetid)
     sa.Index('changes_branch', changes.c.branch)
     sa.Index('changes_revision', changes.c.revision)
     sa.Index('changes_author', changes.c.author)
@@ -354,9 +373,11 @@ class Model(base.DBConnectorComponent):
     sa.Index('scheduler_changes_objectid', scheduler_changes.c.objectid)
     sa.Index('scheduler_changes_changeid', scheduler_changes.c.changeid)
     sa.Index('scheduler_changes_unique', scheduler_changes.c.objectid,
-                    scheduler_changes.c.changeid, unique=True)
-    sa.Index('sourcestamp_changes_sourcestampid', sourcestamp_changes.c.sourcestampid)
-    sa.Index('sourcestamps_sourcestampsetid', sourcestamps.c.sourcestampsetid, unique=False)
+            scheduler_changes.c.changeid, unique=True)
+    sa.Index('sourcestamp_changes_sourcestampid',
+            sourcestamp_changes.c.sourcestampid)
+    sa.Index('sourcestamps_sourcestampsetid', sourcestamps.c.sourcestampsetid,
+            unique=False)
     sa.Index('users_identifier', users.c.identifier, unique=True)
     sa.Index('users_info_uid', users_info.c.uid)
     sa.Index('users_info_uid_attr_type', users_info.c.uid,
@@ -406,7 +427,8 @@ class Model(base.DBConnectorComponent):
             repo_version = repo.latest
             try:
                 # migrate.api doesn't let us hand in an engine
-                schema = migrate.versioning.schema.ControlledSchema(engine, self.repo_path)
+                schema = migrate.versioning.schema.ControlledSchema(engine,
+                                                                self.repo_path)
                 db_version = schema.version
             except exceptions.DatabaseNotControlledError:
                 return False
@@ -430,11 +452,13 @@ class Model(base.DBConnectorComponent):
             except:
                 return False
 
-        # due to http://code.google.com/p/sqlalchemy-migrate/issues/detail?id=100, we cannot
-        # use the migrate.versioning.api module.  So these methods perform similar wrapping
-        # functions to what is done by the API functions, but without disposing of the engine.
+        # http://code.google.com/p/sqlalchemy-migrate/issues/detail?id=100
+        # means  we cannot use the migrate.versioning.api module.  So these
+        # methods perform similar wrapping functions to what is done by the API
+        # functions, but without disposing of the engine.
         def upgrade(engine):
-            schema = migrate.versioning.schema.ControlledSchema(engine, self.repo_path)
+            schema = migrate.versioning.schema.ControlledSchema(engine,
+                    self.repo_path)
             changeset = schema.changeset(None)
             for version, change in changeset:
                 log.msg('migrating schema version %s -> %d'
@@ -462,7 +486,8 @@ class Model(base.DBConnectorComponent):
                     "The minimum version is 0.6.1." % (version,))
 
         def version_control(engine, version=None):
-            migrate.versioning.schema.ControlledSchema.create(engine, self.repo_path, version)
+            migrate.versioning.schema.ControlledSchema.create(engine,
+                    self.repo_path, version)
 
         # the upgrade process must run in a db thread
         def thd(engine):
