@@ -14,7 +14,7 @@
 # Copyright Buildbot Team Members
 
 
-import random, weakref
+import weakref
 from zope.interface import implements
 from twisted.python import log, failure
 from twisted.spread import pb
@@ -27,8 +27,16 @@ from buildbot.status.builder import RETRY
 from buildbot.status.buildrequest import BuildRequestStatus
 from buildbot.process.properties import Properties
 from buildbot.process import buildrequest, slavebuilder
+from buildbot.process.build import Build
 from buildbot.process.slavebuilder import BUILDING
-from buildbot.db import buildrequests
+
+def enforceChosenSlave(bldr, slavebuilder, breq):
+    if 'slavename' in breq.properties:
+        slavename = breq.properties['slavename']
+        if isinstance(slavename, basestring):
+            return slavename==slavebuilder.slave.slavename
+
+    return True
 
 class Builder(config.ReconfigurableServiceMixin,
               pb.Referenceable,
@@ -37,7 +45,7 @@ class Builder(config.ReconfigurableServiceMixin,
     # reconfigure builders before slaves
     reconfig_priority = 196
 
-    def __init__(self, name):
+    def __init__(self, name, _addServices=True):
         service.MultiService.__init__(self)
         self.name = name
 
@@ -61,8 +69,15 @@ class Builder(config.ReconfigurableServiceMixin,
         self.config = None
         self.builder_status = None
 
-        self.reclaim_svc = internet.TimerService(10*60, self.reclaimAllBuilds)
-        self.reclaim_svc.setServiceParent(self)
+        if _addServices:
+            self.reclaim_svc = internet.TimerService(10*60,
+                                            self.reclaimAllBuilds)
+            self.reclaim_svc.setServiceParent(self)
+
+            # update big status every 30 minutes, working around #1980
+            self.updateStatusService = internet.TimerService(30*60,
+                                            self.updateBigStatus)
+            self.updateStatusService.setServiceParent(self)
 
     def reconfigService(self, new_config):
         # find this builder in the config
@@ -77,30 +92,27 @@ class Builder(config.ReconfigurableServiceMixin,
             self.builder_status = self.master.status.builderAdded(
                     builder_config.name,
                     builder_config.builddir,
-                    builder_config.category)
+                    builder_config.category,
+                    builder_config.description)
 
         self.config = builder_config
 
+        self.builder_status.setDescription(builder_config.description)
+        self.builder_status.setCategory(builder_config.category)
         self.builder_status.setSlavenames(self.config.slavenames)
+        self.builder_status.setCacheSize(new_config.caches['Builds'])
 
         return defer.succeed(None)
 
     def stopService(self):
         d = defer.maybeDeferred(lambda :
                 service.MultiService.stopService(self))
-        def flushMaybeStartBuilds(_):
-            # at this point, self.running = False, so another maybeStartBuild
-            # invocation won't hurt anything, but it also will not complete
-            # until any currently-running invocations are done, so we know that
-            # the builder is quiescent at that time.
-            return self.maybeStartBuild()
-        d.addCallback(flushMaybeStartBuilds)
         return d
 
     def __repr__(self):
         return "<Builder '%r' at %d>" % (self.name, id(self))
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def getOldestRequestTime(self):
 
         """Returns the submitted_at of the oldest unclaimed build request for
@@ -108,18 +120,15 @@ class Builder(config.ReconfigurableServiceMixin,
 
         @returns: datetime instance or None, via Deferred
         """
-        wfd = defer.waitForDeferred(
-            self.master.db.buildrequests.getBuildRequests(
-                        buildername=self.name, claimed=False))
-        yield wfd
-        unclaimed = wfd.getResult()
+        unclaimed = yield self.master.db.buildrequests.getBuildRequests(
+                        buildername=self.name, claimed=False)
 
         if unclaimed:
             unclaimed = [ brd['submitted_at'] for brd in unclaimed ]
             unclaimed.sort()
-            yield unclaimed[0]
+            defer.returnValue(unclaimed[0])
         else:
-            yield None
+            defer.returnValue(None)
 
     def reclaimAllBuilds(self):
         brids = set()
@@ -240,14 +249,33 @@ class Builder(config.ReconfigurableServiceMixin,
         self.updateBigStatus()
 
     def updateBigStatus(self):
-        if not self.slaves:
-            self.builder_status.setBigState("offline")
-        elif self.building or self.old_building:
-            self.builder_status.setBigState("building")
-        else:
-            self.builder_status.setBigState("idle")
+        try:
+            # Catch exceptions here, since this is called in a LoopingCall.
+            if not self.builder_status:
+                return
+            if not self.slaves:
+                self.builder_status.setBigState("offline")
+            elif self.building or self.old_building:
+                self.builder_status.setBigState("building")
+            else:
+                self.builder_status.setBigState("idle")
+        except Exception:
+            log.err(None, "while trying to update status of builder '%s'" % (self.name,))
 
-    @defer.deferredGenerator
+    def getAvailableSlaves(self):
+        return [ sb for sb in self.slaves if sb.isAvailable() ]
+
+    def canStartWithSlavebuilder(self, slavebuilder):
+        locks = [(self.botmaster.getLockFromLockAccess(access), access)
+                    for access in self.config.locks ]
+        return Build.canStartWithSlavebuilder(locks, slavebuilder)
+
+    def canStartBuild(self, slavebuilder, breq):
+        if callable(self.config.canStartBuild):
+            return defer.maybeDeferred(self.config.canStartBuild, self, slavebuilder, breq)
+        return defer.succeed(True)
+
+    @defer.inlineCallbacks
     def _startBuildFor(self, slavebuilder, buildrequests):
         """Start a build on the given slave.
         @param build: the L{base.Build} to start
@@ -262,9 +290,12 @@ class Builder(config.ReconfigurableServiceMixin,
         # into a list so that, at any point, we can abort this operation.
         cleanups = []
         def run_cleanups():
-            while cleanups:
-                fn = cleanups.pop()
-                fn()
+            try:
+                while cleanups:
+                    fn = cleanups.pop()
+                    fn()
+            except:
+                log.err(failure.Failure(), "while running %r" % (run_cleanups,))
 
         # the last cleanup we want to perform is to update the big
         # status based on any other cleanup
@@ -289,10 +320,7 @@ class Builder(config.ReconfigurableServiceMixin,
         self.updateBigStatus()
 
         try:
-            wfd = defer.waitForDeferred(
-                    slavebuilder.prepare(self.builder_status, build))
-            yield wfd
-            ready = wfd.getResult()
+            ready = yield slavebuilder.prepare(self.builder_status, build)
         except:
             log.err(failure.Failure(), 'while preparing slavebuilder:')
             ready = False
@@ -303,7 +331,7 @@ class Builder(config.ReconfigurableServiceMixin,
             log.msg("slave %s can't build %s after all; re-queueing the "
                     "request" % (build, slavebuilder))
             run_cleanups()
-            yield False
+            defer.returnValue(False)
             return
 
         # ping the slave to make sure they're still there. If they've
@@ -317,10 +345,7 @@ class Builder(config.ReconfigurableServiceMixin,
         log.msg("starting build %s.. pinging the slave %s"
                 % (build, slavebuilder))
         try:
-            wfd = defer.waitForDeferred(
-                    slavebuilder.ping())
-            yield wfd
-            ping_success = wfd.getResult()
+            ping_success = yield slavebuilder.ping()
         except:
             log.err(failure.Failure(), 'while pinging slave before build:')
             ping_success = False
@@ -328,7 +353,7 @@ class Builder(config.ReconfigurableServiceMixin,
         if not ping_success:
             log.msg("slave ping failed; re-queueing the request")
             run_cleanups()
-            yield False
+            defer.returnValue(False)
             return
 
         # The buildslave is ready to go. slavebuilder.buildStarted() sets its
@@ -339,14 +364,11 @@ class Builder(config.ReconfigurableServiceMixin,
 
         # tell the remote that it's starting a build, too
         try:
-            wfd = defer.waitForDeferred(
-                    slavebuilder.remote.callRemote("startBuild"))
-            yield wfd
-            wfd.getResult()
+            yield slavebuilder.remote.callRemote("startBuild")
         except:
             log.err(failure.Failure(), 'while calling remote startBuild:')
             run_cleanups()
-            yield False
+            defer.returnValue(False)
             return
 
         # create the BuildStatus object that goes with the Build
@@ -356,14 +378,24 @@ class Builder(config.ReconfigurableServiceMixin,
         try:
             bids = []
             for req in build.requests:
-                wfd = defer.waitForDeferred(
-                    self.master.db.builds.addBuild(req.id, bs.number))
-                yield wfd
-                bids.append(wfd.getResult())
+                bid = yield self.master.db.builds.addBuild(req.id, bs.number)
+                bids.append(bid)
         except:
             log.err(failure.Failure(), 'while adding rows to build table:')
             run_cleanups()
-            yield False
+            defer.returnValue(False)
+            return
+
+        # IMPORTANT: no yielding is allowed from here to the startBuild call!
+
+        # it's possible that we lost the slave remote between the ping above
+        # and now.  If so, bail out.  The build.startBuild call below transfers
+        # responsibility for monitoring this connection to the Build instance,
+        # so this check ensures we hand off a working connection.
+        if not slavebuilder.remote:
+            log.msg("slave disappeared before build could start")
+            run_cleanups()
+            defer.returnValue(False)
             return
 
         # let status know
@@ -374,16 +406,20 @@ class Builder(config.ReconfigurableServiceMixin,
         # (through our BuilderStatus object, which is its parent).  Finally it
         # will start the actual build process.  This is done with a fresh
         # Deferred since _startBuildFor should not wait until the build is
-        # finished.
-        d = build.startBuild(bs, self.expectations, slavebuilder)
+        # finished.  This uses `maybeDeferred` to ensure that any exceptions
+        # raised by startBuild are treated as deferred errbacks (see
+        # http://trac.buildbot.net/ticket/2428).
+        d = defer.maybeDeferred(build.startBuild,
+                bs, self.expectations, slavebuilder)
         d.addCallback(self.buildFinished, slavebuilder, bids)
         # this shouldn't happen. if it does, the slave will be wedged
-        d.addErrback(log.err)
+        d.addErrback(log.err, 'from a running build; this is a '
+            'serious error - please file a bug at http://buildbot.net')
 
         # make sure the builder's status is represented correctly
         self.updateBigStatus()
 
-        yield True
+        defer.returnValue(True)
 
     def setupProperties(self, props):
         props.setProperty("buildername", self.name, "Builder")
@@ -426,14 +462,11 @@ class Builder(config.ReconfigurableServiceMixin,
 
         self.updateBigStatus()
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def _maybeBuildsetsComplete(self, requests):
         # inform the master that we may have completed a number of buildsets
         for br in requests:
-            wfd = defer.waitForDeferred(
-                self.master.maybeBuildsetComplete(br.bsid))
-            yield wfd
-            wfd.getResult()
+            yield self.master.maybeBuildsetComplete(br.bsid)
 
     def _resubmit_buildreqs(self, build):
         brids = [br.id for br in build.requests]
@@ -452,189 +485,35 @@ class Builder(config.ReconfigurableServiceMixin,
             # the first time we get a good build, create our Expectations
             # based upon its results
             self.expectations = Expectations(progress)
-        log.msg("new expectations: %s seconds" % \
+        log.msg("new expectations: %s seconds" %
                 self.expectations.expectedBuildTime())
 
     # Build Creation
 
-    @defer.deferredGenerator
-    def maybeStartBuild(self):
+    @defer.inlineCallbacks
+    def maybeStartBuild(self, slavebuilder, breqs):
         # This method is called by the botmaster whenever this builder should
-        # check for and potentially start new builds.  Do not call this method
-        # directly - use master.botmaster.maybeStartBuildsForBuilder, or one
-        # of the other similar methods if more appropriate
+        # start a set of buildrequests on a slave. Do not call this method
+        # directly - use master.botmaster.maybeStartBuildsForBuilder, or one of
+        # the other similar methods if more appropriate
 
         # first, if we're not running, then don't start builds; stopService
         # uses this to ensure that any ongoing maybeStartBuild invocations
         # are complete before it stops.
         if not self.running:
+            defer.returnValue(False)
             return
 
-        # Check for available slaves.  If there are no available slaves, then
-        # there is no sense continuing
-        available_slavebuilders = [ sb for sb in self.slaves
-                                    if sb.isAvailable() ]
-        if not available_slavebuilders:
-            self.updateBigStatus()
-            return
+        # If the build fails from here on out (e.g., because a slave has failed),
+        # it will be handled outside of this function. TODO: test that!
 
-        # now, get the available build requests
-        wfd = defer.waitForDeferred(
-                self.master.db.buildrequests.getBuildRequests(
-                        buildername=self.name, claimed=False))
-        yield wfd
-        unclaimed_requests = wfd.getResult()
-
-        if not unclaimed_requests:
-            self.updateBigStatus()
-            return
-
-        # sort by submitted_at, so the first is the oldest
-        unclaimed_requests.sort(key=lambda brd : brd['submitted_at'])
-
-        # get the mergeRequests function for later
-        mergeRequests_fn = self._getMergeRequestsFn()
-
-        # match them up until we're out of options
-        while available_slavebuilders and unclaimed_requests:
-            # first, choose a slave (using nextSlave)
-            wfd = defer.waitForDeferred(
-                self._chooseSlave(available_slavebuilders))
-            yield wfd
-            slavebuilder = wfd.getResult()
-
-            if not slavebuilder:
-                break
-
-            if slavebuilder not in available_slavebuilders:
-                log.msg(("nextSlave chose a nonexistent slave for builder "
-                         "'%s'; cannot start build") % self.name)
-                break
-
-            # then choose a request (using nextBuild)
-            wfd = defer.waitForDeferred(
-                self._chooseBuild(unclaimed_requests))
-            yield wfd
-            brdict = wfd.getResult()
-
-            if not brdict:
-                break
-
-            if brdict not in unclaimed_requests:
-                log.msg(("nextBuild chose a nonexistent request for builder "
-                         "'%s'; cannot start build") % self.name)
-                break
-
-            # merge the chosen request with any compatible requests in the
-            # queue
-            wfd = defer.waitForDeferred(
-                self._mergeRequests(brdict, unclaimed_requests,
-                                    mergeRequests_fn))
-            yield wfd
-            brdicts = wfd.getResult()
-
-            # try to claim the build requests
-            brids = [ brdict['brid'] for brdict in brdicts ]
-            try:
-                wfd = defer.waitForDeferred(
-                        self.master.db.buildrequests.claimBuildRequests(brids))
-                yield wfd
-                wfd.getResult()
-            except buildrequests.AlreadyClaimedError:
-                # one or more of the build requests was already claimed;
-                # re-fetch the now-partially-claimed build requests and keep
-                # trying to match them
-                self._breakBrdictRefloops(unclaimed_requests)
-                wfd = defer.waitForDeferred(
-                        self.master.db.buildrequests.getBuildRequests(
-                                buildername=self.name, claimed=False))
-                yield wfd
-                unclaimed_requests = wfd.getResult()
-
-                # go around the loop again
-                continue
-
-            # claim was successful, so initiate a build for this set of
-            # requests.  Note that if the build fails from here on out (e.g.,
-            # because a slave has failed), it will be handled outside of this
-            # loop. TODO: test that!
-
-            # _startBuildFor expects BuildRequest objects, so cook some up
-            wfd = defer.waitForDeferred(
-                    defer.gatherResults([ self._brdictToBuildRequest(brdict)
-                                          for brdict in brdicts ]))
-            yield wfd
-            breqs = wfd.getResult()
-
-            wfd = defer.waitForDeferred(
-                    self._startBuildFor(slavebuilder, breqs))
-            yield wfd
-            build_started = wfd.getResult()
-
-            if not build_started:
-                # build was not started, so unclaim the build requests
-                wfd = defer.waitForDeferred(
-                    self.master.db.buildrequests.unclaimBuildRequests(brids))
-                yield wfd
-                wfd.getResult()
-
-                # and try starting builds again.  If we still have a working slave,
-                # then this may re-claim the same buildrequests
-                self.botmaster.maybeStartBuildsForBuilder(self.name)
-
-            # finally, remove the buildrequests and slavebuilder from the
-            # respective queues
-            self._breakBrdictRefloops(brdicts)
-            for brdict in brdicts:
-                unclaimed_requests.remove(brdict)
-            available_slavebuilders.remove(slavebuilder)
-
-        self._breakBrdictRefloops(unclaimed_requests)
-        self.updateBigStatus()
-        return
+        build_started = yield self._startBuildFor(slavebuilder, breqs)
+        defer.returnValue(build_started)
 
     # a few utility functions to make the maybeStartBuild a bit shorter and
     # easier to read
 
-    def _chooseSlave(self, available_slavebuilders):
-        """
-        Choose the next slave, using the C{nextSlave} configuration if
-        available, and falling back to C{random.choice} otherwise.
-
-        @param available_slavebuilders: list of slavebuilders to choose from
-        @returns: SlaveBuilder or None via Deferred
-        """
-        if self.config.nextSlave:
-            return defer.maybeDeferred(lambda :
-                    self.config.nextSlave(self, available_slavebuilders))
-        else:
-            return defer.succeed(random.choice(available_slavebuilders))
-
-    def _chooseBuild(self, buildrequests):
-        """
-        Choose the next build from the given set of build requests (represented
-        as dictionaries).  Defaults to returning the first request (earliest
-        submitted).
-
-        @param buildrequests: sorted list of build request dictionaries
-        @returns: a build request dictionary or None via Deferred
-        """
-        if self.config.nextBuild:
-            # nextBuild expects BuildRequest objects, so instantiate them here
-            # and cache them in the dictionaries
-            d = defer.gatherResults([ self._brdictToBuildRequest(brdict)
-                                      for brdict in buildrequests ])
-            d.addCallback(lambda requestobjects :
-                    self.config.nextBuild(self, requestobjects))
-            def to_brdict(brobj):
-                # get the brdict for this object back
-                return brobj.brdict
-            d.addCallback(to_brdict)
-            return d
-        else:
-            return defer.succeed(buildrequests[0])
-
-    def _getMergeRequestsFn(self):
+    def getMergeRequestsFn(self):
         """Helper function to determine which mergeRequests function to use
         from L{_mergeRequests}, or None for no merging"""
         # first, seek through builder, global, and the default
@@ -655,94 +534,30 @@ class Builder(config.ReconfigurableServiceMixin,
     def _defaultMergeRequestFn(self, req1, req2):
         return req1.canBeMergedWith(req2)
 
-    @defer.deferredGenerator
-    def _mergeRequests(self, breq, unclaimed_requests, mergeRequests_fn):
-        """Use C{mergeRequests_fn} to merge C{breq} against
-        C{unclaimed_requests}, where both are build request dictionaries"""
-        # short circuit if there is no merging to do
-        if not mergeRequests_fn or len(unclaimed_requests) == 1:
-            yield [ breq ]
-            return
-
-        # we'll need BuildRequest objects, so get those first
-        wfd = defer.waitForDeferred(
-            defer.gatherResults(
-                [ self._brdictToBuildRequest(brdict)
-                  for brdict in unclaimed_requests ]))
-        yield wfd
-        unclaimed_request_objects = wfd.getResult()
-        breq_object = unclaimed_request_objects.pop(
-                unclaimed_requests.index(breq))
-
-        # gather the mergeable requests
-        merged_request_objects = [breq_object]
-        for other_breq_object in unclaimed_request_objects:
-            wfd = defer.waitForDeferred(
-                defer.maybeDeferred(lambda :
-                    mergeRequests_fn(self, breq_object, other_breq_object)))
-            yield wfd
-            if wfd.getResult():
-                merged_request_objects.append(other_breq_object)
-
-        # convert them back to brdicts and return
-        merged_requests = [ br.brdict for br in merged_request_objects ]
-        yield merged_requests
-
-    def _brdictToBuildRequest(self, brdict):
-        """
-        Convert a build request dictionary to a L{buildrequest.BuildRequest}
-        object, caching the result in the dictionary itself.  The resulting
-        buildrequest will have a C{brdict} attribute pointing back to this
-        dictionary.
-
-        Note that this does not perform any locking - be careful that it is
-        only called once at a time for each build request dictionary.
-
-        @param brdict: dictionary to convert
-
-        @returns: L{buildrequest.BuildRequest} via Deferred
-        """
-        if 'brobj' in brdict:
-            return defer.succeed(brdict['brobj'])
-        d = buildrequest.BuildRequest.fromBrdict(self.master, brdict)
-        def keep(buildrequest):
-            brdict['brobj'] = buildrequest
-            buildrequest.brdict = brdict
-            return buildrequest
-        d.addCallback(keep)
-        return d
-
-    def _breakBrdictRefloops(self, requests):
-        """Break the reference loops created by L{_brdictToBuildRequest}"""
-        for brdict in requests:
-            try:
-                del brdict['brobj'].brdict
-            except KeyError:
-                pass
-
 
 class BuilderControl:
     implements(interfaces.IBuilderControl)
 
-    def __init__(self, builder, master):
+    def __init__(self, builder, control):
         self.original = builder
-        self.master = master
+        self.control = control
 
     def submitBuildRequest(self, ss, reason, props=None):
-        d = ss.getSourceStampSetId(self.master.master)
+        d = ss.getSourceStampSetId(self.control.master)
         def add_buildset(sourcestampsetid):
-            return self.master.master.addBuildset(
+            return self.control.master.addBuildset(
                     builderNames=[self.original.name],
                     sourcestampsetid=sourcestampsetid, reason=reason, properties=props)
         d.addCallback(add_buildset)
         def get_brs((bsid,brids)):
             brs = BuildRequestStatus(self.original.name,
                                      brids[self.original.name],
-                                     self.master.master.status)
+                                     self.control.master.status)
             return brs
         d.addCallback(get_brs)
         return d
 
+    @defer.inlineCallbacks
     def rebuildBuild(self, bs, reason="<rebuild, no reason given>", extraProperties=None):
         if not bs.isFinished():
             return
@@ -755,37 +570,43 @@ class BuilderControl:
             properties.updateFromProperties(extraProperties)
 
         properties_dict = dict((k,(v,s)) for (k,v,s) in properties.asList())
-        ss = bs.getSourceStamp(absolute=True)
-        d = ss.getSourceStampSetId(self.master.master)
-        def add_buildset(sourcestampsetid):
-            return self.master.master.addBuildset(
-                    builderNames=[self.original.name],
-                    sourcestampsetid=sourcestampsetid, reason=reason, properties=properties_dict)
-        d.addCallback(add_buildset)
-        return d
+        ssList = bs.getSourceStamps(absolute=True)
+        
+        if ssList:
+            sourcestampsetid = yield  ssList[0].getSourceStampSetId(self.control.master)
+            dl = []
+            for ss in ssList[1:]:
+                # add defered to the list
+                dl.append(ss.addSourceStampToDatabase(self.control.master, sourcestampsetid))
+            yield defer.gatherResults(dl)
 
-    @defer.deferredGenerator
+            bsid, brids = yield self.control.master.addBuildset(
+                    builderNames=[self.original.name],
+                    sourcestampsetid=sourcestampsetid, 
+                    reason=reason, 
+                    properties=properties_dict)
+            defer.returnValue((bsid, brids))
+        else:
+            log.msg('Cannot start rebuild, rebuild has no sourcestamps for a new build')
+            defer.returnValue(None)
+
+    @defer.inlineCallbacks
     def getPendingBuildRequestControls(self):
         master = self.original.master
-        wfd = defer.waitForDeferred(
-            master.db.buildrequests.getBuildRequests(
+        brdicts = yield master.db.buildrequests.getBuildRequests(
                 buildername=self.original.name,
-                claimed=False))
-        yield wfd
-        brdicts = wfd.getResult()
+                claimed=False)
 
         # convert those into BuildRequest objects
         buildrequests = [ ]
         for brdict in brdicts:
-            wfd = defer.waitForDeferred(
-                buildrequest.BuildRequest.fromBrdict(self.master.master,
-                                                     brdict))
-            yield wfd
-            buildrequests.append(wfd.getResult())
+            br = yield buildrequest.BuildRequest.fromBrdict(
+                    self.control.master, brdict)
+            buildrequests.append(br)
 
         # and return the corresponding control objects
-        yield [ buildrequest.BuildRequestControl(self.original, r)
-                 for r in buildrequests ]
+        defer.returnValue([ buildrequest.BuildRequestControl(self.original, r)
+                            for r in buildrequests ])
 
     def getBuild(self, number):
         return self.original.getBuild(number)
@@ -806,4 +627,3 @@ class BuilderControl:
             if not success:
                 return False
         return True
-
