@@ -16,7 +16,7 @@
 import re
 
 from zope.interface import implements
-from twisted.internet import reactor, defer, error
+from twisted.internet import defer, error
 from twisted.protocols import basic
 from twisted.spread import pb
 from twisted.python import log, components
@@ -24,11 +24,12 @@ from twisted.python.failure import Failure
 from twisted.web.util import formatFailure
 from twisted.python.reflect import accumulateClassList
 
-from buildbot import interfaces, locks, util, config
+from buildbot import interfaces, util, config
 from buildbot.status import progress
 from buildbot.status.results import SUCCESS, WARNINGS, FAILURE, SKIPPED, \
      EXCEPTION, RETRY, worst_status
 from buildbot.process import metrics, properties
+from buildbot.util.eventual import eventually
 
 class BuildStepFailed(Exception):
     pass
@@ -42,18 +43,22 @@ class RemoteCommand(pb.Referenceable):
     rc = None
     debug = False
 
-    def __init__(self, remote_command, args, ignore_updates=False, collectStdout=False):
+    def __init__(self, remote_command, args, ignore_updates=False,
+            collectStdout=False, collectStderr=False, decodeRC={0:SUCCESS}):
         self.logs = {}
         self.delayedLogs = {}
         self._closeWhenFinished = {}
         self.collectStdout = collectStdout
+        self.collectStderr = collectStderr
         self.stdout = ''
+        self.stderr = ''
 
         self._startTime = None
         self._remoteElapsed = None
         self.remote_command = remote_command
         self.args = args
         self.ignore_updates = ignore_updates
+        self.decodeRC = decodeRC
 
     def __repr__(self):
         return "<RemoteCommand '%s' at %d>" % (self.remote_command, id(self))
@@ -85,13 +90,13 @@ class RemoteCommand(pb.Referenceable):
         # when our parent Step calls our .lostRemote() method.
         return self.deferred
 
-    def useLog(self, loog, closeWhenFinished=False, logfileName=None):
-        assert interfaces.ILogFile.providedBy(loog)
+    def useLog(self, log, closeWhenFinished=False, logfileName=None):
+        assert interfaces.ILogFile.providedBy(log)
         if not logfileName:
-            logfileName = loog.getName()
+            logfileName = log.getName()
         assert logfileName not in self.logs
         assert logfileName not in self.delayedLogs
-        self.logs[logfileName] = loog
+        self.logs[logfileName] = log
         self._closeWhenFinished[logfileName] = closeWhenFinished
 
     def useLogDelayed(self, logfileName, activateCallBack, closeWhenFinished=False):
@@ -192,7 +197,7 @@ class RemoteCommand(pb.Referenceable):
         # call the real remoteComplete a moment later, but first return an
         # acknowledgement so the slave can retire the completion message.
         if self.active:
-            reactor.callLater(0, self._finished, failure)
+            eventually(self._finished, failure)
         return None
 
     def addStdout(self, data):
@@ -204,6 +209,8 @@ class RemoteCommand(pb.Referenceable):
     def addStderr(self, data):
         if 'stdio' in self.logs:
             self.logs['stdio'].addStderr(data)
+        if self.collectStderr:
+            self.stderr += data
 
     def addHeader(self, data):
         if 'stdio' in self.logs:
@@ -268,6 +275,14 @@ class RemoteCommand(pb.Referenceable):
                     log.msg("closing log %s" % loog)
                 loog.finish()
         return maybeFailure
+
+    def results(self):
+        if self.rc in self.decodeRC:
+            return self.decodeRC[self.rc]
+        return FAILURE
+
+    def didFail(self):
+        return self.results() == FAILURE
 LoggedRemoteCommand = RemoteCommand
 
 
@@ -344,7 +359,9 @@ class RemoteShellCommand(RemoteCommand):
                  want_stdout=1, want_stderr=1,
                  timeout=20*60, maxTime=None, logfiles={},
                  usePTY="slave-config", logEnviron=True,
-                 collectStdout=False, interruptSignal=None):
+                 collectStdout=False,collectStderr=False,
+                 interruptSignal=None,
+                 initialStdin=None, decodeRC={0:SUCCESS}):
 
         self.command = command # stash .command, set it later
         if env is not None:
@@ -361,10 +378,13 @@ class RemoteShellCommand(RemoteCommand):
                 'maxTime': maxTime,
                 'usePTY': usePTY,
                 'logEnviron': logEnviron,
+                'initial_stdin': initialStdin
                 }
         if interruptSignal is not None:
             args['interruptSignal'] = interruptSignal
-        RemoteCommand.__init__(self, "shell", args, collectStdout=collectStdout)
+        RemoteCommand.__init__(self, "shell", args, collectStdout=collectStdout,
+                               collectStderr=collectStderr,
+                               decodeRC=decodeRC)
 
     def _start(self):
         self.args['command'] = self.command
@@ -381,7 +401,29 @@ class RemoteShellCommand(RemoteCommand):
     def __repr__(self):
         return "<RemoteShellCommand '%s'>" % repr(self.command)
 
-class BuildStep(properties.PropertiesMixin):
+class _BuildStepFactory(util.ComparableMixin):
+    """
+    This is a wrapper to record the arguments passed to as BuildStep subclass.
+    We use an instance of this class, rather than a closure mostly to make it
+    easier to test that the right factories are getting created.
+    """
+    compare_attrs = ['factory', 'args', 'kwargs' ]
+    implements(interfaces.IBuildStepFactory)
+
+    def __init__(self, factory, *args, **kwargs):
+        self.factory = factory
+        self.args = args
+        self.kwargs = kwargs
+
+    def buildStep(self):
+        try:
+            return self.factory(*self.args, **self.kwargs)
+        except:
+            log.msg("error while creating step, factory=%s, args=%s, kwargs=%s"
+                    % (self.factory, self.args, self.kwargs))
+            raise
+
+class BuildStep(object, properties.PropertiesMixin):
 
     haltOnFailure = False
     flunkOnWarnings = False
@@ -424,20 +466,26 @@ class BuildStep(properties.PropertiesMixin):
     progress = None
 
     def __init__(self, **kwargs):
-        self.factory = (self.__class__, dict(kwargs))
         for p in self.__class__.parms:
             if kwargs.has_key(p):
                 setattr(self, p, kwargs[p])
                 del kwargs[p]
         if kwargs:
-            why = "%s.__init__ got unexpected keyword argument(s) %s" \
-                  % (self, kwargs.keys())
-            raise TypeError(why)
+            config.error("%s.__init__ got unexpected keyword argument(s) %s" \
+                  % (self.__class__, kwargs.keys()))
         self._pendingLogObservers = []
 
+        if not isinstance(self.name, str):
+            config.error("BuildStep name must be a string: %r" % (self.name,))
+
         self._acquiringLock = None
         self.stopped = False
 
+    def __new__(klass, *args, **kwargs):
+        self = object.__new__(klass)
+        self._factory = _BuildStepFactory(klass, *args, **kwargs)
+        return self
+
     def describe(self, done=False):
         return [self.name]
 
@@ -451,10 +499,11 @@ class BuildStep(properties.PropertiesMixin):
         pass
 
     def addFactoryArguments(self, **kwargs):
-        self.factory[1].update(kwargs)
+        # this is here for backwards compatability
+        pass
 
-    def getStepFactory(self):
-        return self.factory
+    def _getStepFactory(self):
+        return self._factory
 
     def setStepStatus(self, step_status):
         self.step_status = step_status
@@ -475,17 +524,13 @@ class BuildStep(properties.PropertiesMixin):
         self.remote = remote
         self.deferred = defer.Deferred()
         # convert all locks into their real form
-        lock_list = []
-        for access in self.locks:
-            if not isinstance(access, locks.LockAccess):
-                # Buildbot 0.7.7 compability: user did not specify access
-                access = access.defaultAccess()
-            lock = self.build.builder.botmaster.getLockByID(access.lockid)
-            lock_list.append((lock, access))
-        self.locks = lock_list
+        self.locks = [(self.build.builder.botmaster.getLockByID(access.lockid), access)
+                        for access in self.locks ]
         # then narrow SlaveLocks down to the slave that this build is being
         # run on
-        self.locks = [(l.getLock(self.build.slavebuilder), la) for l, la in self.locks]
+        self.locks = [(l.getLock(self.build.slavebuilder.slave), la)
+                        for l, la in self.locks ]
+
         for l, la in self.locks:
             if l in self.build.locks:
                 log.msg("Hey, lock %s is claimed by both a Step (%s) and the"
@@ -510,7 +555,7 @@ class BuildStep(properties.PropertiesMixin):
             return defer.succeed(None)
         log.msg("acquireLocks(step %s, locks %s)" % (self, self.locks))
         for lock, access in self.locks:
-            if not lock.isAvailable(access):
+            if not lock.isAvailable(self, access):
                 self.step_status.setWaitingForLocks(True)
                 log.msg("step %s waiting for lock %s" % (self, lock))
                 d = lock.waitUntilMaybeAvailable(self, access)
@@ -539,16 +584,26 @@ class BuildStep(properties.PropertiesMixin):
         renderables = []
         accumulateClassList(self.__class__, 'renderables', renderables)
 
+        def setRenderable(res, attr):
+            setattr(self, attr, res)
+
+        dl = [ doStep ]
         for renderable in renderables:
-            setattr(self, renderable, self.build.render(getattr(self, renderable)))
+            d = self.build.render(getattr(self, renderable))
+            d.addCallback(setRenderable, renderable)
+            dl.append(d)
+        dl = defer.gatherResults(dl)
 
-        doStep.addCallback(self._startStep_3)
-        return doStep
+        dl.addCallback(self._startStep_3)
+        return dl
 
+    @defer.inlineCallbacks
     def _startStep_3(self, doStep):
+        doStep = doStep[0]
         try:
             if doStep:
-                if self.start() == SKIPPED:
+                result = yield defer.maybeDeferred(self.start)
+                if result == SKIPPED:
                     doStep = False
         except:
             log.msg("BuildStep.startStep exception in .start")
@@ -561,7 +616,7 @@ class BuildStep(properties.PropertiesMixin):
             # the step immediately; we skip calling finished() as
             # subclasses may have overridden that an expect it to be called
             # after start() (bug #837)
-            reactor.callLater(0, self._finishFinished, SKIPPED)
+            eventually(self._finishFinished, SKIPPED)
 
     def start(self):
         raise NotImplementedError("your subclass must implement this method")
@@ -601,11 +656,19 @@ class BuildStep(properties.PropertiesMixin):
         # from finished() so that subclasses can override finished()
         if self.progress:
             self.progress.finish()
+
+        try:
+            hidden = self._maybeEvaluate(self.hideStepIf, results, self)
+        except Exception:
+            why = Failure()
+            self.addHTMLLog("err.html", formatFailure(why))
+            self.addCompleteLog("err.text", why.getTraceback())
+            results = EXCEPTION
+            hidden = False
+
         self.step_status.stepFinished(results)
-        
-        hidden = self._maybeEvaluate(self.hideStepIf, results, self)
         self.step_status.setHidden(hidden)
-        
+
         self.releaseLocks()
         self.deferred.callback(results)
 
@@ -621,8 +684,12 @@ class BuildStep(properties.PropertiesMixin):
         try:
             if self.progress:
                 self.progress.finish()
-            self.addHTMLLog("err.html", formatFailure(why))
-            self.addCompleteLog("err.text", why.getTraceback())
+            try:
+                self.addCompleteLog("err.text", why.getTraceback())
+                self.addHTMLLog("err.html", formatFailure(why))
+            except Exception:
+                log.err(Failure(), "error while formatting exceptions")
+
             # could use why.getDetailedTraceback() for more information
             self.step_status.setText([self.name, "exception"])
             self.step_status.setText2([self.name])
@@ -630,17 +697,16 @@ class BuildStep(properties.PropertiesMixin):
 
             hidden = self._maybeEvaluate(self.hideStepIf, EXCEPTION, self)
             self.step_status.setHidden(hidden)
-        except:
-            log.msg("exception during failure processing")
-            log.err()
+        except Exception:
+            log.err(Failure(), "exception during failure processing")
             # the progress stuff may still be whacked (the StepStatus may
             # think that it is still running), but the build overall will now
             # finish
+
         try:
             self.releaseLocks()
-        except:
-            log.msg("exception while releasing locks")
-            log.err()
+        except Exception:
+            log.err(Failure(), "exception while releasing locks")
 
         log.msg("BuildStep.failed now firing callback")
         self.deferred.callback(EXCEPTION)
@@ -719,6 +785,9 @@ class BuildStep(properties.PropertiesMixin):
             value = value(*args, **kwargs)
         return value
 
+components.registerAdapter(
+        BuildStep._getStepFactory,
+        BuildStep, interfaces.IBuildStepFactory)
 components.registerAdapter(
         lambda step : interfaces.IProperties(step.build),
         BuildStep, interfaces.IProperties)
@@ -747,9 +816,6 @@ class LoggingBuildStep(BuildStep):
     def __init__(self, logfiles={}, lazylogfiles=False, log_eval_func=None,
                  *args, **kwargs):
         BuildStep.__init__(self, *args, **kwargs)
-        self.addFactoryArguments(logfiles=logfiles,
-                                 lazylogfiles=lazylogfiles,
-                                 log_eval_func=log_eval_func)
 
         if logfiles and not isinstance(logfiles, dict):
             config.error(
@@ -854,9 +920,7 @@ class LoggingBuildStep(BuildStep):
     def evaluateCommand(self, cmd):
         if self.log_eval_func:
             return self.log_eval_func(cmd, self.step_status)
-        if cmd.rc != 0:
-            return FAILURE
-        return SUCCESS
+        return cmd.results()
 
     def getText(self, cmd, results):
         if results == SUCCESS:
@@ -901,9 +965,7 @@ class LoggingBuildStep(BuildStep):
 #   log_eval_func=lambda c,s: regex_log_evaluator(c, s, regexs)
 # )
 def regex_log_evaluator(cmd, step_status, regexes):
-    worst = SUCCESS
-    if cmd.rc != 0:
-        worst = FAILURE
+    worst = cmd.results()
     for err, possible_status in regexes:
         # worst_status returns the worse of the two status' passed to it.
         # we won't be changing "worst" unless possible_status is worse than it,
