@@ -1,34 +1,39 @@
-# -*- test-case-name: buildbot.test.test_step -*-
+# This file is part of Buildbot.  Buildbot is free software: you can
+# redistribute it and/or modify it under the terms of the GNU General Public
+# License as published by the Free Software Foundation, version 2.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
+#
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, write to the Free Software Foundation, Inc., 51
+# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# Copyright Buildbot Team Members
 
-from buildbot import util
-from buildbot.process.base import Build
+import warnings
+from twisted.python import deprecate, versions
+
+from buildbot import interfaces, util
+from buildbot.process.build import Build
 from buildbot.process.buildstep import BuildStep
 from buildbot.steps.source import CVS, SVN
 from buildbot.steps.shell import Configure, Compile, Test, PerlModuleTest
 
 # deprecated, use BuildFactory.addStep
+@deprecate.deprecated(versions.Version("buildbot", 0, 8, 6))
 def s(steptype, **kwargs):
     # convenience function for master.cfg files, to create step
     # specification tuples
-    return (steptype, kwargs)
+    return interfaces.IBuildStepFactory(steptype(**kwargs))
 
-class ArgumentsInTheWrongPlace(Exception):
-    """When calling BuildFactory.addStep(stepinstance), addStep() only takes
-    one argument. You passed extra arguments to addStep(), which you probably
-    intended to pass to your BuildStep constructor instead. For example, you
-    should do::
-
-     f.addStep(ShellCommand(command=['echo','stuff'], haltOnFailure=True))
-
-    instead of::
-
-     f.addStep(ShellCommand(command=['echo','stuff']), haltOnFailure=True)
-    """
 
 class BuildFactory(util.ComparableMixin):
     """
     @cvar  buildClass: class to use when creating builds
-    @type  buildClass: L{buildbot.process.base.Build}
+    @type  buildClass: L{buildbot.process.build.Build}
     """
     buildClass = Build
     useProgress = 1
@@ -36,36 +41,30 @@ class BuildFactory(util.ComparableMixin):
     compare_attrs = ['buildClass', 'steps', 'useProgress', 'workdir']
 
     def __init__(self, steps=None):
-        if steps is None:
-            steps = []
-        self.steps = [self._makeStepFactory(s) for s in steps]
+        self.steps = []
+        if steps:
+            self.addSteps(steps)
 
-    def _makeStepFactory(self, step_or_factory):
-        if isinstance(step_or_factory, BuildStep):
-            return step_or_factory.getStepFactory()
-        return step_or_factory
-
-    def newBuild(self, request):
+    def newBuild(self, requests):
         """Create a new Build instance.
-        @param request: a L{base.BuildRequest} describing what is to be built
+
+        @param requests: a list of buildrequest dictionaries describing what is
+        to be built
         """
-        b = self.buildClass(request)
+        b = self.buildClass(requests)
         b.useProgress = self.useProgress
         b.workdir = self.workdir
         b.setStepFactories(self.steps)
         return b
 
-    def addStep(self, step_or_factory, **kwargs):
-        if isinstance(step_or_factory, BuildStep):
-            if kwargs:
-                raise ArgumentsInTheWrongPlace()
-            s = step_or_factory.getStepFactory()
-        elif type(step_or_factory) == type(BuildStep) and \
-                issubclass(step_or_factory, BuildStep):
-            s = (step_or_factory, dict(kwargs))
-        else:
-            raise ValueError('%r is not a BuildStep nor BuildStep subclass' % step_or_factory)
-        self.steps.append(s)
+    def addStep(self, step, **kwargs):
+        if kwargs or (type(step) == type(BuildStep) and issubclass(step, BuildStep)):
+            warnings.warn(
+                    "Passing a BuildStep subclass to factory.addStep is "
+                    "deprecated. Please pass a BuildStep instance instead.",
+                    DeprecationWarning, stacklevel=2)
+            step = step(**kwargs)
+        self.steps.append(interfaces.IBuildStepFactory(step))
 
     def addSteps(self, steps):
         for s in steps:
@@ -93,25 +92,25 @@ class GNUAutoconf(BuildFactory):
             else:
                 assert isinstance(configure, (list, tuple))
                 command = configure + configureFlags
-            self.addStep(Configure, command=command, env=configureEnv)
+            self.addStep(Configure(command=command, env=configureEnv))
         if compile is not None:
-            self.addStep(Compile, command=compile)
+            self.addStep(Compile(command=compile))
         if test is not None:
-            self.addStep(Test, command=test)
+            self.addStep(Test(command=test))
 
 class CPAN(BuildFactory):
     def __init__(self, source, perl="perl"):
         BuildFactory.__init__(self, [source])
-        self.addStep(Configure, command=[perl, "Makefile.PL"])
-        self.addStep(Compile, command=["make"])
-        self.addStep(PerlModuleTest, command=["make", "test"])
+        self.addStep(Configure(command=[perl, "Makefile.PL"]))
+        self.addStep(Compile(command=["make"]))
+        self.addStep(PerlModuleTest(command=["make", "test"]))
 
 class Distutils(BuildFactory):
     def __init__(self, source, python="python", test=None):
         BuildFactory.__init__(self, [source])
-        self.addStep(Compile, command=[python, "./setup.py", "build"])
+        self.addStep(Compile(command=[python, "./setup.py", "build"]))
         if test is not None:
-            self.addStep(Test, command=test)
+            self.addStep(Test(command=test))
 
 class Trial(BuildFactory):
     """Build a python module that uses distutils and trial. Set 'tests' to
@@ -143,15 +142,15 @@ class Trial(BuildFactory):
 
         from buildbot.steps.python_twisted import Trial
         buildcommand = buildpython + ["./setup.py", "build"]
-        self.addStep(Compile, command=buildcommand, env=env)
-        self.addStep(Trial,
+        self.addStep(Compile(command=buildcommand, env=env))
+        self.addStep(Trial(
                      python=trialpython, trial=self.trial,
                      testpath=testpath,
                      tests=tests, testChanges=useTestCaseNames,
                      randomly=self.randomly,
                      recurse=self.recurse,
                      env=env,
-                     )
+                     ))
 
 
 # compatibility classes, will go away. Note that these only offer
@@ -170,7 +169,7 @@ class BasicBuildFactory(GNUAutoconf):
         mode = "clobber"
         if cvsCopy:
             mode = "copy"
-        source = s(CVS, cvsroot=cvsroot, cvsmodule=cvsmodule, mode=mode)
+        source = CVS(cvsroot=cvsroot, cvsmodule=cvsmodule, mode=mode)
         GNUAutoconf.__init__(self, source,
                              configure=configure, configureEnv=configureEnv,
                              compile=compile,
@@ -184,7 +183,7 @@ class QuickBuildFactory(BasicBuildFactory):
                  compile="make all",
                  test="make check", cvsCopy=False):
         mode = "update"
-        source = s(CVS, cvsroot=cvsroot, cvsmodule=cvsmodule, mode=mode)
+        source = CVS(cvsroot=cvsroot, cvsmodule=cvsmodule, mode=mode)
         GNUAutoconf.__init__(self, source,
                              configure=configure, configureEnv=configureEnv,
                              compile=compile,
@@ -196,7 +195,7 @@ class BasicSVN(GNUAutoconf):
                  configure=None, configureEnv={},
                  compile="make all",
                  test="make check"):
-        source = s(SVN, svnurl=svnurl, mode="update")
+        source = SVN(svnurl=svnurl, mode="update")
         GNUAutoconf.__init__(self, source,
                              configure=configure, configureEnv=configureEnv,
                              compile=compile,
