@@ -12,6 +12,8 @@
 # Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 # Copyright Buildbot Team Members
+
+from __future__ import with_statement
 """
 Buildbot metrics module
 
@@ -38,7 +40,7 @@ from twisted.internet.task import LoopingCall
 from twisted.internet import reactor
 from twisted.application import service
 from buildbot import util, config
-from buildbot.util.bbcollections import defaultdict
+from collections import defaultdict
 
 import gc, os, sys
 # Make use of the resource module if we can
@@ -323,43 +325,47 @@ class AttachedSlavesWatcher(object):
 def _get_rss():
     if sys.platform == 'linux2':
         try:
-            return int(open("/proc/%i/statm" % os.getpid()).read().split()[1])
+            with open("/proc/%i/statm" % os.getpid()) as f:
+                return int(f.read().split()[1])
         except:
             return 0
     return 0
 
 def periodicCheck(_reactor=reactor):
-    # Measure how much garbage we have
-    garbage_count = len(gc.garbage)
-    MetricCountEvent.log('gc.garbage', garbage_count, absolute=True)
-    if garbage_count == 0:
-        level = ALARM_OK
-    else:
-        level = ALARM_WARN
-    MetricAlarmEvent.log('gc.garbage', level=level)
+    try:
+        # Measure how much garbage we have
+        garbage_count = len(gc.garbage)
+        MetricCountEvent.log('gc.garbage', garbage_count, absolute=True)
+        if garbage_count == 0:
+            level = ALARM_OK
+        else:
+            level = ALARM_WARN
+        MetricAlarmEvent.log('gc.garbage', level=level)
 
-    if resource:
-        r = resource.getrusage(resource.RUSAGE_SELF)
-        attrs = ['ru_utime', 'ru_stime', 'ru_maxrss', 'ru_ixrss', 'ru_idrss',
-                'ru_isrss', 'ru_minflt', 'ru_majflt', 'ru_nswap',
-                'ru_inblock', 'ru_oublock', 'ru_msgsnd', 'ru_msgrcv',
-                'ru_nsignals', 'ru_nvcsw', 'ru_nivcsw']
-        for i,a in enumerate(attrs):
-            # Linux versions prior to 2.6.32 didn't report this value, but we
-            # can calculate it from /proc/<pid>/statm
-            v = r[i]
-            if a == 'ru_maxrss' and v == 0:
-                v = _get_rss() * resource.getpagesize() / 1024
-            MetricCountEvent.log('resource.%s' % a, v, absolute=True)
-        MetricCountEvent.log('resource.pagesize', resource.getpagesize(), absolute=True)
-    # Measure the reactor delay
-    then = util.now(_reactor)
-    dt = 0.1
-    def cb():
-        now = util.now(_reactor)
-        delay = (now - then) - dt
-        MetricTimeEvent.log("reactorDelay", delay)
-    _reactor.callLater(dt, cb)
+        if resource:
+            r = resource.getrusage(resource.RUSAGE_SELF)
+            attrs = ['ru_utime', 'ru_stime', 'ru_maxrss', 'ru_ixrss', 'ru_idrss',
+                    'ru_isrss', 'ru_minflt', 'ru_majflt', 'ru_nswap',
+                    'ru_inblock', 'ru_oublock', 'ru_msgsnd', 'ru_msgrcv',
+                    'ru_nsignals', 'ru_nvcsw', 'ru_nivcsw']
+            for i,a in enumerate(attrs):
+                # Linux versions prior to 2.6.32 didn't report this value, but we
+                # can calculate it from /proc/<pid>/statm
+                v = r[i]
+                if a == 'ru_maxrss' and v == 0:
+                    v = _get_rss() * resource.getpagesize() / 1024
+                MetricCountEvent.log('resource.%s' % a, v, absolute=True)
+            MetricCountEvent.log('resource.pagesize', resource.getpagesize(), absolute=True)
+        # Measure the reactor delay
+        then = util.now(_reactor)
+        dt = 0.1
+        def cb():
+            now = util.now(_reactor)
+            delay = (now - then) - dt
+            MetricTimeEvent.log("reactorDelay", delay)
+        _reactor.callLater(dt, cb)
+    except Exception:
+        log.err(None, "while collecting VM metrics")
 
 class MetricLogObserver(config.ReconfigurableServiceMixin,
                         service.MultiService):
@@ -485,4 +491,4 @@ class MetricLogObserver(config.ReconfigurableServiceMixin,
                 for line in report.split("\n"):
                     log.msg(line)
         except:
-            log.err()
+            log.err(None, "generating metric report")
