@@ -19,10 +19,10 @@ from twisted.application import service
 from twisted.internet import defer
 from buildbot.process.properties import Properties
 from buildbot.util import ComparableMixin
-from buildbot.changes import changes
 from buildbot import config, interfaces
+from buildbot.util.state import StateMixin
 
-class BaseScheduler(service.MultiService, ComparableMixin):
+class BaseScheduler(service.MultiService, ComparableMixin, StateMixin):
     """
     Base class for all schedulers; this provides the equipment to manage
     reconfigurations and to handle basic scheduler state.  It also provides
@@ -34,9 +34,12 @@ class BaseScheduler(service.MultiService, ComparableMixin):
 
     implements(interfaces.IScheduler)
 
-    compare_attrs = ('name', 'builderNames', 'properties')
+    DefaultCodebases = {'':{}}
 
-    def __init__(self, name, builderNames, properties):
+    compare_attrs = ('name', 'builderNames', 'properties', 'codebases')
+
+    def __init__(self, name, builderNames, properties,
+                 codebases = DefaultCodebases):
         """
         Initialize a Scheduler.
 
@@ -50,6 +53,11 @@ class BaseScheduler(service.MultiService, ComparableMixin):
         scheduler
         @type properties: dictionary
 
+        @param codebases: codebases that are necessary to process the changes
+        @type codebases: dict with following struct:
+            key: '<codebase>'
+            value: {'repository':'<repo>', 'branch':'<br>', 'revision:'<rev>'}
+
         @param consumeChanges: true if this scheduler wishes to be informed
         about the addition of new changes.  Defaults to False.  This should
         be passed explicitly from subclasses to indicate their interest in
@@ -84,10 +92,25 @@ class BaseScheduler(service.MultiService, ComparableMixin):
 
         self.master = None
 
+        # Set the codebases that are necessary to process the changes
+        # These codebases will always result in a sourcestamp with or without changes
+        if codebases is not None:
+            if not isinstance(codebases, dict):
+                config.error("Codebases must be a dict of dicts")
+            for codebase, codebase_attrs in codebases.iteritems():
+                if not isinstance(codebase_attrs, dict):
+                    config.error("Codebases must be a dict of dicts")
+                if (codebases != BaseScheduler.DefaultCodebases and
+                   'repository' not in codebase_attrs):
+                    config.error("The key 'repository' is mandatory in codebases")
+        else:
+            config.error("Codebases cannot be None")
+
+        self.codebases = codebases
+        
         # internal variables
         self._change_subscription = None
         self._change_consumption_lock = defer.DeferredLock()
-        self._objectid = None
 
     ## service handling
 
@@ -102,56 +125,6 @@ class BaseScheduler(service.MultiService, ComparableMixin):
         d.addCallback(lambda _ : service.MultiService.stopService(self))
         return d
 
-    ## state management
-
-    @defer.deferredGenerator
-    def getState(self, *args, **kwargs):
-        """
-        For use by subclasses; get a named state value from the scheduler's
-        state, defaulting to DEFAULT.
-
-        @param name: name of the value to retrieve
-        @param default: (optional) value to return if C{name} is not present
-        @returns: state value via a Deferred
-        @raises KeyError: if C{name} is not present and no default is given
-        @raises TypeError: if JSON parsing fails
-        """
-        # get the objectid, if not known
-        if self._objectid is None:
-            wfd = defer.waitForDeferred(
-                self.master.db.state.getObjectId(self.name,
-                                        self.__class__.__name__))
-            yield wfd
-            self._objectid = wfd.getResult()
-
-        wfd = defer.waitForDeferred(
-            self.master.db.state.getState(self._objectid, *args, **kwargs))
-        yield wfd
-        yield wfd.getResult()
-
-    @defer.deferredGenerator
-    def setState(self, key, value):
-        """
-        For use by subclasses; set a named state value in the scheduler's
-        persistent state.  Note that value must be json-able.
-
-        @param name: the name of the value to change
-        @param value: the value to set - must be a JSONable object
-        @param returns: Deferred
-        @raises TypeError: if JSONification fails
-        """
-        # get the objectid, if not known
-        if self._objectid is None:
-            wfd = defer.waitForDeferred(
-                self.master.db.state.getObjectId(self.name,
-                                        self.__class__.__name__))
-            yield wfd
-            self._objectid = wfd.getResult()
-
-        wfd = defer.waitForDeferred(
-            self.master.db.state.setState(self._objectid, key, value))
-        yield wfd
-        wfd.getResult()
 
     ## status queries
 
@@ -199,6 +172,11 @@ class BaseScheduler(service.MultiService, ComparableMixin):
 
             if change_filter and not change_filter.filter_change(change):
                 return
+            if change.codebase not in self.codebases:
+                log.msg(format='change contains codebase %(codebase)s that is'
+                    'not processed by scheduler %(scheduler)s',
+                    codebase=change.codebase, name=self.name)
+                return
             if fileIsImportant:
                 try:
                     important = fileIsImportant(change)
@@ -213,11 +191,7 @@ class BaseScheduler(service.MultiService, ComparableMixin):
 
             # use change_consumption_lock to ensure the service does not stop
             # while this change is being processed
-            d = self._change_consumption_lock.acquire()
-            d.addCallback(lambda _ : self.gotChange(change, important))
-            def release(x):
-                self._change_consumption_lock.release()
-            d.addBoth(release)
+            d = self._change_consumption_lock.run(self.gotChange, change, important)
             d.addErrback(log.err, 'while processing change')
         self._change_subscription = self.master.subscribeToChanges(changeCallback)
 
@@ -228,20 +202,19 @@ class BaseScheduler(service.MultiService, ComparableMixin):
 
         # acquire the lock change consumption lock to ensure that any change
         # consumption is complete before we are done stopping consumption
-        d = self._change_consumption_lock.acquire()
-        def stop(x):
+        def stop():
             if self._change_subscription:
                 self._change_subscription.unsubscribe()
                 self._change_subscription = None
-            self._change_consumption_lock.release()
-        d.addBoth(stop)
-        return d
+        return self._change_consumption_lock.run(stop)
 
     def gotChange(self, change, important):
         """
         Called when a change is received; returns a Deferred.  If the
         C{fileIsImportant} parameter to C{startConsumingChanges} was C{None},
         then all changes are considered important.
+        The C{codebase} of the change has always an entry in the C{codebases}
+        dictionary of the scheduler.
 
         @param change: the new change object
         @type change: L{buildbot.changes.changes.Change} instance
@@ -254,7 +227,7 @@ class BaseScheduler(service.MultiService, ComparableMixin):
 
     ## starting bulids
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def addBuildsetForLatest(self, reason='', external_idstring=None,
                         branch=None, repository='', project='',
                         builderNames=None, properties=None):
@@ -281,40 +254,46 @@ class BaseScheduler(service.MultiService, ComparableMixin):
         @returns: (buildset ID, buildrequest IDs) via Deferred
         """
         # Define setid for this set of changed repositories
-        wfd = defer.waitForDeferred(self.master.db.sourcestampsets.addSourceStampSet())
-        yield wfd
-        setid = wfd.getResult()
+        setid = yield self.master.db.sourcestampsets.addSourceStampSet()
 
-        wfd = defer.waitForDeferred(self.master.db.sourcestamps.addSourceStamp(
-                branch=branch, revision=None, repository=repository,
-                project=project, sourcestampsetid=setid))
-        yield wfd
-        wfd.getResult()
+        # add a sourcestamp for each codebase
+        for codebase, cb_info in self.codebases.iteritems():
+            ss_repository = cb_info.get('repository', repository)
+            ss_branch = cb_info.get('branch', branch)
+            ss_revision = cb_info.get('revision', None)
+            yield self.master.db.sourcestamps.addSourceStamp(
+                        codebase=codebase,
+                        repository=ss_repository,
+                        branch=ss_branch,
+                        revision=ss_revision,
+                        project=project,
+                        changeids=set(),
+                        sourcestampsetid=setid)
 
-        wfd = defer.waitForDeferred(self.addBuildsetForSourceStamp(
+        bsid,brids = yield self.addBuildsetForSourceStamp(
                                 setid=setid, reason=reason,
                                 external_idstring=external_idstring,
                                 builderNames=builderNames,
-                                properties=properties))
-        yield wfd
-        yield wfd.getResult()
+                                properties=properties)
 
-    @defer.deferredGenerator
-    def addBuildsetForChanges(self, reason='', external_idstring=None,
-            changeids=[], builderNames=None, properties=None):
+        defer.returnValue((bsid,brids))
+
+
+    @defer.inlineCallbacks
+    def addBuildsetForSourceStampDetails(self, reason='', external_idstring=None,
+                        branch=None, repository='', project='', revision=None,
+                        builderNames=None, properties=None):
         """
-        Add a buildset for the combination of the given changesets, creating
-        a sourcestamp based on those changes.  The sourcestamp for the buildset
-        will reference all of the indicated changes.
-
-        This method will add any properties provided to the scheduler
-        constructor to the buildset, and will call the master's addBuildset
-        method with the appropriate parameters.
+        Given details about the source code to build, create a source stamp and
+        then add a buildset for it.
 
         @param reason: reason for this buildset
         @type reason: unicode string
         @param external_idstring: external identifier for this buildset, or None
-        @param changeids: nonempty list of changes to include in this buildset
+        @param branch: branch to build (note that None often has a special meaning)
+        @param repository: repository name for sourcestamp
+        @param project: project name for sourcestamp
+        @param revision: revision to build - default is latest
         @param builderNames: builders to name in the buildset (defaults to
             C{self.builderNames})
         @param properties: a properties object containing initial properties for
@@ -322,130 +301,104 @@ class BaseScheduler(service.MultiService, ComparableMixin):
         @type properties: L{buildbot.process.properties.Properties}
         @returns: (buildset ID, buildrequest IDs) via Deferred
         """
-        assert changeids is not []
-
-        # attributes for this sourcestamp will be based on the most recent
-        # change, so fetch the change with the highest id
-        wfd = defer.waitForDeferred(self.master.db.changes.getChange(max(changeids)))
-        yield wfd
-        chdict = wfd.getResult()
-
-        change = None
-        if chdict:
-            wfd = defer.waitForDeferred(changes.Change.fromChdict(self.master, chdict))
-            yield wfd
-            change = wfd.getResult()
-
         # Define setid for this set of changed repositories
-        wfd = defer.waitForDeferred(self.master.db.sourcestampsets.addSourceStampSet())
-        yield wfd
-        setid = wfd.getResult()
+        setid = yield self.master.db.sourcestampsets.addSourceStampSet()
 
-        wfd = defer.waitForDeferred(self.master.db.sourcestamps.addSourceStamp(
-                    branch=change.branch,
-                    revision=change.revision,
-                    repository=change.repository,
-                    project=change.project,
-                    changeids=changeids,
-                    sourcestampsetid=setid))
-        yield wfd
-        wfd.getResult()
+        yield self.master.db.sourcestamps.addSourceStamp(
+                branch=branch, revision=revision, repository=repository,
+                project=project, sourcestampsetid=setid)
 
-        wfd = defer.waitForDeferred(self.addBuildsetForSourceStamp(
+        rv = yield self.addBuildsetForSourceStamp(
                                 setid=setid, reason=reason,
                                 external_idstring=external_idstring,
                                 builderNames=builderNames,
-                                properties=properties))
-        yield wfd
+                                properties=properties)
+        defer.returnValue(rv)
 
-        yield wfd.getResult()
 
-    @defer.deferredGenerator
-    def addBuildsetForChangesMultiRepo(self, reason='', external_idstring=None,
+    @defer.inlineCallbacks
+    def addBuildsetForSourceStampSetDetails(self, reason, sourcestamps,
+                                            properties, builderNames=None):
+        if sourcestamps is None:
+            sourcestamps = {}
+
+        # Define new setid for this set of sourcestamps
+        new_setid = yield self.master.db.sourcestampsets.addSourceStampSet()
+
+        # Merge codebases with the passed list of sourcestamps
+        # This results in a new sourcestamp for each codebase
+        for codebase in self.codebases:
+            ss = self.codebases[codebase].copy()
+             # apply info from passed sourcestamps onto the configured default
+             # sourcestamp attributes for this codebase.
+            ss.update(sourcestamps.get(codebase,{}))
+
+            # add sourcestamp to the new setid
+            yield self.master.db.sourcestamps.addSourceStamp(
+                        codebase=codebase,
+                        repository=ss.get('repository', ''),
+                        branch=ss.get('branch', None),
+                        revision=ss.get('revision', None),
+                        project=ss.get('project', ''),
+                        changeids=[c['number'] for c in ss.get('changes', [])],
+                        patch_body=ss.get('patch_body', None),
+                        patch_level=ss.get('patch_level', None),
+                        patch_author=ss.get('patch_author', None),
+                        patch_comment=ss.get('patch_comment', None),
+                        sourcestampsetid=new_setid)
+
+        rv = yield self.addBuildsetForSourceStamp(
+                                setid=new_setid, reason=reason,
+                                properties=properties,
+                                builderNames=builderNames)
+
+        defer.returnValue(rv)
+
+
+    @defer.inlineCallbacks
+    def addBuildsetForChanges(self, reason='', external_idstring=None,
             changeids=[], builderNames=None, properties=None):
-        assert changeids is not []
-        chDicts = {}
+        changesByCodebase = {}
 
-        def getChange(changeid = None):
-            d = self.master.db.changes.getChange(changeid)
-            def chdict2change(chdict):
-                if not chdict:
-                    return None
-                return changes.Change.fromChdict(self.master, chdict)
-            d.addCallback(chdict2change)
-
-        def groupChange(change):
-            if change.repository not in chDicts:
-                chDicts[change.repository] = []
-            chDicts[change.repository].append(change)
-
-        def get_changeids_from_repo(repository):
-            changeids = []
-            for change in chDicts[repository]:
-                changeids.append(change.number)
-            return changeids
-
-        def create_sourcestamp(changeids, change = None, setid = None):
-            def add_sourcestamp(setid, changeids = None):
-                return self.master.db.sourcestamps.addSourceStamp(
-                        branch=change.branch,
-                        revision=change.revision,
-                        repository=change.repository,
-                        project=change.project,
-                        changeids=changeids,
-                        setid=setid)
-            d.addCallback(add_sourcestamp, setid = setid, changeids = changeids)
-            return d
-
-        def create_sourcestamp_without_changes(setid, repository):
-            return self.master.db.sourcestamps.addSourceStamp(
-                    branch=self.default_branch,
-                    revision=None,
-                    repository=repository,
-                    project=self.default_project,
-                    changeids=changeids,
-                    sourcestampsetid=setid)
-
-        d = defer.Deferred()
-        if self.repositories is None:
-            # attributes for this sourcestamp will be based on the most recent
-            # change, so fetch the change with the highest id (= old single
-            # sourcestamp functionality)
-            d.addCallBack(getChange,changeid=max(changeids))
-            d.addCallBack(groupChange)
-        else:
-            for changeid in changeids:
-                d.addCallBack(getChange,changeid = changeid)
-                d.addCallBack(groupChange)
+        def get_last_change_for_codebase(codebase):
+            return max(changesByCodebase[codebase],key = lambda change: change["changeid"])
 
         # Define setid for this set of changed repositories
-        wfd = defer.waitForDeferred(self.master.db.sourcestampsets.addSourceStampSet)
-        yield wfd
-        setid = wfd.getResult()
+        setid = yield self.master.db.sourcestampsets.addSourceStampSet()
 
-        #process all unchanged repositories
-        if self.repositories is not None:
-            for repo in self.repositories:
-                if repo not in chDicts:
-                    # repository was not changed
-                    # call create_sourcestamp
-                    d.addCallback(create_sourcestamp_without_changes, setid, repo)
+        # Changes are retrieved from database and grouped by their codebase
+        for changeid in changeids:
+            chdict = yield self.master.db.changes.getChange(changeid)
+            # group change by codebase
+            changesByCodebase.setdefault(chdict["codebase"], []).append(chdict)
 
-        # process all changed
-        for repo in chDicts:
-            d.addCallback(get_changeids_from_repo, repository = repo)
-            d.addCallback(create_sourcestamp, setid = setid, change=chDicts[repo][-1])
+        for codebase in self.codebases:
+            args = {'codebase': codebase, 'sourcestampsetid': setid }
+            if codebase not in changesByCodebase:
+                # codebase has no changes
+                # create a sourcestamp that has no changes
+                args['repository'] = self.codebases[codebase]['repository']
+                args['branch'] = self.codebases[codebase].get('branch', None)
+                args['revision'] = self.codebases[codebase].get('revision', None)
+                args['changeids'] = set()
+                args['project'] = ''
+            else:
+                #codebase has changes
+                args['changeids'] = [c["changeid"] for c in changesByCodebase[codebase]]
+                lastChange = get_last_change_for_codebase(codebase)
+                for key in ['repository', 'branch', 'revision', 'project']:
+                    args[key] = lastChange[key]
+
+            yield self.master.db.sourcestamps.addSourceStamp(**args)
 
         # add one buildset, this buildset is connected to the sourcestamps by the setid
-        d.addCallback(self.addBuildsetForSourceStamp, setid=setid, reason=reason,
-                            external_idstring=external_idstring,
-                            builderNames=builderNames,
-                            properties=properties)
+        bsid,brids = yield self.addBuildsetForSourceStamp( setid=setid,
+                            reason=reason, external_idstring=external_idstring,
+                            builderNames=builderNames, properties=properties)
 
-        yield d
+        defer.returnValue((bsid,brids))
 
-
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def addBuildsetForSourceStamp(self, ssid=None, setid=None, reason='', external_idstring=None,
             properties=None, builderNames=None):
         """
@@ -485,19 +438,16 @@ class BaseScheduler(service.MultiService, ComparableMixin):
         properties_dict = properties.asDict()
 
         if setid == None:
-            if ssid != None:
-                wfd = defer.waitForDeferred(self.master.db.sourcestamps.getSourceStamp(ssid))
-                yield wfd
-                ssdict = wfd.getResult()
+            if ssid is not None:
+                ssdict = yield self.master.db.sourcestamps.getSourceStamp(ssid)
                 setid = ssdict['sourcestampsetid']
             else:
                 # no sourcestamp and no sets
                 yield None
 
-        wfd = defer.waitForDeferred(self.master.addBuildset(
-                                        sourcestampsetid=setid, reason=reason,
-                                        properties=properties_dict,
-                                        builderNames=builderNames,
-                                        external_idstring=external_idstring))
-        yield wfd
-        yield wfd.getResult()
+        rv = yield self.master.addBuildset(sourcestampsetid=setid,
+                            reason=reason, properties=properties_dict,
+                            builderNames=builderNames,
+                            external_idstring=external_idstring)
+        defer.returnValue(rv)
+
