@@ -21,6 +21,7 @@ from twisted.protocols import basic
 
 from buildbot import pbutil
 from buildbot.util.maildir import MaildirService
+from buildbot.util import json
 from buildbot.util import netstrings
 from buildbot.process.properties import Properties
 from buildbot.schedulers import base
@@ -68,31 +69,51 @@ class JobdirService(MaildirService):
 
 class Try_Jobdir(TryBase):
 
-    compare_attrs = TryBase.compare_attrs + ( 'jobdir', )
+    compare_attrs = TryBase.compare_attrs + ('jobdir',)
 
     def __init__(self, name, builderNames, jobdir,
                  properties={}):
-        TryBase.__init__(self, name=name, builderNames=builderNames, properties=properties)
+        TryBase.__init__(self, name=name, builderNames=builderNames,
+                         properties=properties)
         self.jobdir = jobdir
         self.watcher = JobdirService()
         self.watcher.setServiceParent(self)
 
     def startService(self):
         # set the watcher's basedir now that we have a master
-        self.watcher.setBasedir(os.path.join(self.master.basedir, self.jobdir))
+        jobdir = os.path.join(self.master.basedir, self.jobdir)
+        self.watcher.setBasedir(jobdir)
+        for subdir in "cur new tmp".split():
+            if not os.path.exists(os.path.join(jobdir, subdir)):
+                os.mkdir(os.path.join(jobdir, subdir))
         TryBase.startService(self)
 
     def parseJob(self, f):
         # jobfiles are serialized build requests. Each is a list of
         # serialized netstrings, in the following order:
-        #  "2", the format version number ("1" does not have project/repo)
-        #  buildsetID, arbitrary string, used to find the buildSet later
-        #  branch name, "" for default-branch
-        #  base revision, "" for HEAD
-        #  patchlevel, usually "1"
-        #  patch
-        #  builderNames...
+        #  format version number:
+        #  "1" the original
+        #  "2" introduces project and repository
+        #  "3" introduces who
+        #  "4" introduces comment
+        #  "5" introduces properties and JSON serialization of values after
+        #      version
+        #  jobid: arbitrary string, used to find the buildSet later
+        #  branch: branch name, "" for default-branch
+        #  baserev: revision, "" for HEAD
+        #  patch_level: usually "1"
+        #  patch_body: patch to be applied for build
+        #  repository
+        #  project
+        #  who: user requesting build
+        #  comment: comment from user about diff and/or build
+        #  builderNames: list of builder names
+        #  properties: dict of build properties
         p = netstrings.NetstringParser()
+        f.seek(0,2)
+        if f.tell() > basic.NetstringReceiver.MAX_LENGTH:
+            raise BadJobfile("The patch size is greater that NetStringReceiver.MAX_LENGTH. Please Set this higher in the master.cfg")
+        f.seek(0,0)
         try:
             p.feed(f.read())
         except basic.NetstringParseError:
@@ -101,59 +122,42 @@ class Try_Jobdir(TryBase):
             raise BadJobfile("could not find any complete netstrings")
         ver = p.strings.pop(0)
 
-        if ver == "1":
-            buildsetID, branch, baserev, patchlevel, diff = p.strings[:5]
-            builderNames = p.strings[5:]
-            if branch == "":
-                branch = None
-            if baserev == "":
-                baserev = None
-            patchlevel = int(patchlevel)
-            repository=''
-            project=''
-            who=''
-            comment=''
-        elif ver == "2": # introduced the repository and project property
-            buildsetID, branch, baserev, patchlevel, diff, repository, project = p.strings[:7]
-            builderNames = p.strings[7:]
-            if branch == "":
-                branch = None
-            if baserev == "":
-                baserev = None
-            patchlevel = int(patchlevel)
-            who=''
-            comment=''
-        elif ver == "3": # introduced who property
-            buildsetID, branch, baserev, patchlevel, diff, repository, project, who = p.strings[:8]
-            builderNames = p.strings[8:]
-            if branch == "":
-                branch = None
-            if baserev == "":
-                baserev = None
-            patchlevel = int(patchlevel)
-            comment=''
-        elif ver == "4": # introduced try comments
-            buildsetID, branch, baserev, patchlevel, diff, repository, project, who, comment = p.strings[:9]
-            builderNames = p.strings[9:]
-            if branch == "":
-                branch = None
-            if baserev == "":
-                baserev = None
-            patchlevel = int(patchlevel)
-            
+        v1_keys = ['jobid', 'branch', 'baserev', 'patch_level', 'patch_body']
+        v2_keys = v1_keys + ['repository', 'project']
+        v3_keys = v2_keys + ['who']
+        v4_keys = v3_keys + ['comment']
+        keys = [v1_keys, v2_keys, v3_keys, v4_keys]
+        # v5 introduces properties and uses JSON serialization
+
+        parsed_job = {}
+
+        def extract_netstrings(p, keys):
+            for i, key in enumerate(keys):
+                parsed_job[key] = p.strings[i]
+
+        def postprocess_parsed_job():
+            # apply defaults and handle type casting
+            parsed_job['branch'] = parsed_job['branch'] or None
+            parsed_job['baserev'] = parsed_job['baserev'] or None
+            parsed_job['patch_level'] = int(parsed_job['patch_level'])
+            for key in 'repository project who comment'.split():
+                parsed_job[key] = parsed_job.get(key, '')
+            parsed_job['properties'] = parsed_job.get('properties', {})
+
+        if ver <= "4":
+            i = int(ver) - 1
+            extract_netstrings(p, keys[i])
+            parsed_job['builderNames'] = p.strings[len(keys[i]):]
+            postprocess_parsed_job()
+        elif ver == "5":
+            try:
+                parsed_job = json.loads(p.strings[0])
+            except ValueError:
+                raise BadJobfile("unable to parse JSON")
+            postprocess_parsed_job()
         else:
             raise BadJobfile("unknown version '%s'" % ver)
-        return dict(
-                builderNames=builderNames,
-                branch=branch,
-                baserev=baserev,
-                patch_body=diff,
-                patch_level=patchlevel,
-                repository=repository,
-                project=project,
-                who=who,
-                comment=comment,
-                jobid=buildsetID)
+        return parsed_job
 
     def handleJobFile(self, filename, f):
         try:
@@ -167,34 +171,47 @@ class Try_Jobdir(TryBase):
         # Validate/fixup the builder names.
         builderNames = self.filterBuilderList(builderNames)
         if not builderNames:
-            log.msg("incoming Try job did not specify any allowed builder names")
+            log.msg(
+                "incoming Try job did not specify any allowed builder names")
             return defer.succeed(None)
-        
+
         who = ""
         if parsed_job['who']:
             who = parsed_job['who']
-        
+
         comment = ""
         if parsed_job['comment']:
             comment = parsed_job['comment']
 
-        d = self.master.db.sourcestamps.addSourceStamp(
+        d = self.master.db.sourcestampsets.addSourceStampSet()
+
+        def addsourcestamp(setid):
+            self.master.db.sourcestamps.addSourceStamp(
+                sourcestampsetid=setid,
                 branch=parsed_job['branch'],
                 revision=parsed_job['baserev'],
                 patch_body=parsed_job['patch_body'],
                 patch_level=parsed_job['patch_level'],
                 patch_author=who,
                 patch_comment=comment,
-                patch_subdir='', # TODO: can't set this remotely - #1769
+                patch_subdir='',  # TODO: can't set this remotely - #1769
                 project=parsed_job['project'],
                 repository=parsed_job['repository'])
-        def create_buildset(ssid):
+            return setid
+
+        d.addCallback(addsourcestamp)
+
+        def create_buildset(setid):
             reason = "'try' job"
             if parsed_job['who']:
                 reason += " by user %s" % parsed_job['who']
-            return self.addBuildsetForSourceStamp(ssid=ssid,
-                    reason=reason, external_idstring=parsed_job['jobid'],
-                    builderNames=builderNames)
+            properties = parsed_job['properties']
+            requested_props = Properties()
+            requested_props.update(properties, "try build")
+            return self.addBuildsetForSourceStamp(
+                ssid=None, setid=setid,
+                reason=reason, external_idstring=parsed_job['jobid'],
+                builderNames=builderNames, properties=requested_props)
         d.addCallback(create_buildset)
         return d
 
@@ -204,9 +221,9 @@ class Try_Userpass_Perspective(pbutil.NewCredPerspective):
         self.scheduler = scheduler
         self.username = username
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def perspective_try(self, branch, revision, patch, repository, project,
-                        builderNames, who="", comment="", properties={} ):
+                        builderNames, who="", comment="", properties={}):
         db = self.scheduler.master.db
         log.msg("user %s requesting build on builders %s" % (self.username,
                                                              builderNames))
@@ -217,41 +234,35 @@ class Try_Userpass_Perspective(pbutil.NewCredPerspective):
             return
 
         reason = "'try' job"
-        
+
         if who:
             reason += " by user %s" % who
-            
+
         if comment:
             reason += " (%s)" % comment
 
-        wfd = defer.waitForDeferred(
-                db.sourcestamps.addSourceStamp(branch=branch, revision=revision,
-                    repository=repository, project=project, patch_level=patch[0],
-                    patch_body=patch[1], patch_subdir='', patch_author=who or '',
-                    patch_comment=comment or ''))
+        sourcestampsetid = yield db.sourcestampsets.addSourceStampSet()
+
+        yield db.sourcestamps.addSourceStamp(
+            branch=branch, revision=revision, repository=repository,
+            project=project, patch_level=patch[0], patch_body=patch[1],
+            patch_subdir='', patch_author=who or '',
+            patch_comment=comment or '',
+            sourcestampsetid=sourcestampsetid)
                     # note: no way to specify patch subdir - #1769
-        yield wfd
-        ssid = wfd.getResult()
 
         requested_props = Properties()
         requested_props.update(properties, "try build")
-        wfd = defer.waitForDeferred(
-                self.scheduler.addBuildsetForSourceStamp(ssid=ssid,
-                        reason=reason, properties=requested_props,
-                        builderNames=builderNames))
-        yield wfd
-        (bsid,brids) = wfd.getResult()
+        (bsid, brids) = yield self.scheduler.addBuildsetForSourceStamp(
+                setid=sourcestampsetid, reason=reason,
+                properties=requested_props, builderNames=builderNames)
 
         # return a remotely-usable BuildSetStatus object
-        wfd = defer.waitForDeferred(
-                db.buildsets.getBuildset(bsid))
-        yield wfd
-        bsdict = wfd.getResult()
+        bsdict = yield db.buildsets.getBuildset(bsid)
 
         bss = BuildSetStatus(bsdict, self.scheduler.master.status)
         from buildbot.status.client import makeRemote
-        r = makeRemote(bss)
-        yield r # return value
+        defer.returnValue(makeRemote(bss))
 
     def perspective_getAvailableBuilderNames(self):
         # Return a list of builder names that are configured
@@ -262,11 +273,12 @@ class Try_Userpass_Perspective(pbutil.NewCredPerspective):
 
 
 class Try_Userpass(TryBase):
-    compare_attrs = ( 'name', 'builderNames', 'port', 'userpass', 'properties' )
+    compare_attrs = ('name', 'builderNames', 'port', 'userpass', 'properties')
 
     def __init__(self, name, builderNames, port, userpass,
                  properties={}):
-        TryBase.__init__(self, name=name, builderNames=builderNames, properties=properties)
+        TryBase.__init__(self, name=name, builderNames=builderNames,
+                         properties=properties)
         self.port = port
         self.userpass = userpass
 
@@ -279,12 +291,14 @@ class Try_Userpass(TryBase):
         self.registrations = []
         for user, passwd in self.userpass:
             self.registrations.append(
-                    self.master.pbmanager.register(self.port, user, passwd, factory))
+                self.master.pbmanager.register(
+                    self.port, user, passwd, factory))
 
     def stopService(self):
         d = defer.maybeDeferred(TryBase.stopService, self)
+
         def unreg(_):
             return defer.gatherResults(
-                [ reg.unregister() for reg in self.registrations ])
+                [reg.unregister() for reg in self.registrations])
         d.addCallback(unreg)
         return d
