@@ -27,8 +27,27 @@ import sys
 
 from buildbot.scripts import base
 
-# Note that the terms 'options' and 'config' are used intechangeably here - in
-# fact, they are intercanged several times.  Caveat legator.
+# Note that the terms 'options' and 'config' are used interchangeably here - in
+# fact, they are interchanged several times.  Caveat legator.
+
+def validateMasterOption(master):
+    """
+    Validate master (-m, --master) command line option.
+
+    Checks that option is a string of the 'hostname:port' form, otherwise
+    raises an UsageError exception.
+
+    @type  master: string
+    @param master: master option
+
+    @raise usage.UsageError: on invalid master option
+    """
+    try:
+        hostname, port = master.split(":")
+        port = int(port)
+    except:
+        raise usage.UsageError("master must have the form 'hostname:port'")
+
 
 class UpgradeMasterOptions(base.BasedirMixin, base.SubcommandOptions):
     subcommandFunction = "buildbot.scripts.upgrade_master.upgradeMaster"
@@ -134,6 +153,7 @@ class StopOptions(base.BasedirMixin, base.SubcommandOptions):
     subcommandFunction = "buildbot.scripts.stop.stop"
     optFlags = [
         ["quiet", "q", "Do not emit the commands being run"],
+        ["clean", "c", "Clean shutdown master"],
         ]
     def getSynopsis(self):
         return "Usage:    buildbot stop [<basedir>]"
@@ -144,6 +164,7 @@ class RestartOptions(base.BasedirMixin, base.SubcommandOptions):
     optFlags = [
         ['quiet', 'q', "Don't display startup log messages"],
         ['nodaemon', None, "Don't daemonize (stay in foreground)"],
+        ["clean", "c", "Clean shutdown master"],
         ]
     def getSynopsis(self):
         return "Usage:    buildbot restart [<basedir>]"
@@ -192,6 +213,10 @@ class DebugClientOptions(base.SubcommandOptions):
         if len(args) > 2:
             raise usage.UsageError("I wasn't expecting so many arguments")
 
+    def postOptions(self):
+        base.SubcommandOptions.postOptions(self)
+        validateMasterOption(self.get('master'))
+
 
 class BaseStatusClientOptions(base.SubcommandOptions):
     optFlags = [
@@ -215,6 +240,11 @@ class BaseStatusClientOptions(base.SubcommandOptions):
         if len(args) > 1:
             raise usage.UsageError("I wasn't expecting so many arguments")
 
+    def postOptions(self):
+        base.SubcommandOptions.postOptions(self)
+        validateMasterOption(self.get('master'))
+
+
 
 class StatusLogOptions(BaseStatusClientOptions):
     subcommandFunction = "buildbot.scripts.statuslog.statuslog"
@@ -319,11 +349,7 @@ class SendChangeOptions(base.SubcommandOptions):
         if self.get('vc') and self.get('vc') not in vcs:
             raise usage.UsageError("vc must be one of %s" % (', '.join(vcs)))
 
-        if not self.get('who'):
-            raise usage.UsageError("you must provide a committer (--who)")
-        if not self.get('master'):
-            raise usage.UsageError("you must provide the master location")
-
+        validateMasterOption(self.get('master'))
 
 class TryOptions(base.SubcommandOptions):
     subcommandFunction = "buildbot.scripts.trycmd.trycmd"
@@ -376,6 +402,9 @@ class TryOptions(base.SubcommandOptions):
          "A set of properties made available in the build environment, "
          "format is --properties=prop1=value1,prop2=value2,.. "
          "option can be specified multiple times."],
+        ["property", None, None,
+         "A property made available in the build environment, "
+         "format:prop=value. Can be used multiple times."],
 
         ["topfile", None, None,
          "Name of a file at the top of the tree, used to find the top. "
@@ -437,6 +466,10 @@ class TryOptions(base.SubcommandOptions):
             splitproperty = propertylist[i].split("=", 1)
             self['properties'][splitproperty[0]] = splitproperty[1]
 
+    def opt_property(self, option):
+        name, _, value = option.partition("=")
+        self['properties'][name] = value
+
     def opt_patchlevel(self, option):
         self['patchlevel'] = int(option)
 
@@ -457,6 +490,12 @@ class TryOptions(base.SubcommandOptions):
         if not self['master']:
             self['master'] = opts.get('masterstatus', None)
 
+        if self['connect'] == 'pb':
+            if not self['master']:
+                raise usage.UsageError("master location must be specified" \
+                                       "for 'pb' connections")
+            validateMasterOption(self['master'])
+
 
 class TryServerOptions(base.SubcommandOptions):
     subcommandFunction = "buildbot.scripts.tryserver.tryserver"
@@ -575,12 +614,7 @@ class UserOptions(base.SubcommandOptions):
     def postOptions(self):
         base.SubcommandOptions.postOptions(self)
 
-        master = self.get('master')
-        try:
-            master, port = master.split(":")
-            port = int(port)
-        except:
-            raise usage.UsageError("master must have the form 'hostname:port'")
+        validateMasterOption(self.get('master'))
 
         op = self.get('op')
         if not op:
