@@ -20,7 +20,7 @@ from twisted.application import service
 from buildbot.interfaces import IStatusReceiver
 from buildbot import util, pbutil
 
-class StatusReceiver:
+class StatusReceiverBase:
     implements(IStatusReceiver)
 
     def requestSubmitted(self, request):
@@ -86,13 +86,18 @@ class StatusReceiver:
     def checkConfig(self, otherStatusReceivers):
         pass
 
-class StatusReceiverMultiService(StatusReceiver, service.MultiService,
+class StatusReceiverMultiService(StatusReceiverBase, service.MultiService,
                                  util.ComparableMixin):
-    implements(IStatusReceiver)
 
     def __init__(self):
         service.MultiService.__init__(self)
 
+class StatusReceiverService(StatusReceiverBase, service.Service,
+                                 util.ComparableMixin):
+    pass
+
+StatusReceiver = StatusReceiverService
+
 
 class StatusReceiverPerspective(StatusReceiver, pbutil.NewCredPerspective):
     implements(IStatusReceiver)
