@@ -96,8 +96,7 @@ class LogFileProducer:
         while data:
             p.dataReceived(data)
             while chunks:
-                c = chunks.pop(0)
-                yield c
+                yield chunks.pop(0)
             f.seek(offset)
             data = f.read(self.BUFFERSIZE)
             offset = f.tell()
@@ -392,13 +391,10 @@ class LogFile:
             else:
                 yield leftover
 
-    def readlines(self, channel=STDOUT):
+    def readlines(self):
         """Return an iterator that produces newline-terminated lines,
         excluding header chunks."""
-        # TODO: make this memory-efficient, by turning it into a generator
-        # that retrieves chunks as necessary, like a pull-driven version of
-        # twisted.protocols.basic.LineReceiver
-        alltext = "".join(self.getChunks([channel], onlyText=True))
+        alltext = "".join(self.getChunks([STDOUT], onlyText=True))
         io = StringIO(alltext)
         return io.readlines()
 
@@ -680,6 +676,8 @@ class HTMLLogFile:
     def __getstate__(self):
         d = self.__dict__.copy()
         del d['step']
+        if d.has_key('master'):
+            del d['master']
         return d
 
 
