@@ -41,10 +41,15 @@ try:
 except ImportError:
     have_ssl = False
 
+# this incantation teaches email to output utf-8 using 7- or 8-bit encoding,
+# although it has no effect before python-2.7.
+from email import Charset
+Charset.add_charset('utf-8', Charset.SHORTEST, None, 'utf-8')
+
 from buildbot import interfaces, util, config
 from buildbot.process.users import users
 from buildbot.status import base
-from buildbot.status.results import FAILURE, SUCCESS, WARNINGS, Results
+from buildbot.status.results import FAILURE, SUCCESS, WARNINGS, EXCEPTION, Results
 
 VALID_EMAIL = re.compile("[a-zA-Z0-9\.\_\%\-\+]+@[a-zA-Z0-9\.\_\%\-]+.[a-zA-Z]{2,6}")
 
@@ -69,7 +74,8 @@ class Domain(util.ComparableMixin):
 def defaultMessage(mode, name, build, results, master_status):
     """Generate a buildbot mail message and return a tuple of message text
         and type."""
-    ss = build.getSourceStamp()
+    ss_list = build.getSourceStamps()
+    
     prev = build.getPreviousBuild()
 
     text = ""
@@ -86,12 +92,17 @@ def defaultMessage(mode, name, build, results, master_status):
             text += "The Buildbot has detected a restored build"
         else:
             text += "The Buildbot has detected a passing build"
+    elif results == EXCEPTION:
+        text += "The Buildbot has detected a build exception"
 
-    if ss and ss.project:
-        project = ss.project
-    else:
-        project = master_status.getTitle()
-    text += " on builder %s while building %s.\n" % (name, project)
+    projects = []
+    if ss_list:
+        for ss in ss_list:
+            if ss.project and ss.project not in projects:
+                projects.append(ss.project)
+    if not projects:
+        projects = [master_status.getTitle()]
+    text += " on builder %s while building %s.\n" % (name, ', '.join(projects))
 
     if master_status.getURLForThing(build):
         text += "Full details are available at:\n %s\n" % master_status.getURLForThing(build)
@@ -103,17 +114,21 @@ def defaultMessage(mode, name, build, results, master_status):
     text += "Buildslave for this Build: %s\n\n" % build.getSlavename()
     text += "Build Reason: %s\n" % build.getReason()
 
-    source = ""
-    if ss and ss.branch:
-        source += "[branch %s] " % ss.branch
-    if ss and ss.revision:
-        source += str(ss.revision)
-    else:
-        source += "HEAD"
-    if ss and ss.patch:
-        source += " (plus patch)"
+    for ss in ss_list:
+        source = ""
+        if ss and ss.branch:
+            source += "[branch %s] " % ss.branch
+        if ss and ss.revision:
+            source += str(ss.revision)
+        else:
+            source += "HEAD"
+        if ss and ss.patch:
+            source += " (plus patch)"
 
-    text += "Build Source Stamp: %s\n" % source
+        discriminator = ""
+        if ss.codebase:
+            discriminator = " '%s'" % ss.codebase
+        text += "Build Source Stamp%s: %s\n" % (discriminator, source)
 
     text += "Blamelist: %s\n" % ",".join(build.getResponsibleUsers())
 
@@ -138,6 +153,9 @@ def defaultMessage(mode, name, build, results, master_status):
     text += "\n"
     return { 'body' : text, 'type' : 'plain' }
 
+def defaultGetPreviousBuild(current_build):
+        return current_build.getPreviousBuild()
+
 class MailNotifier(base.StatusReceiverMultiService):
     """This is a status notifier which sends email to a list of recipients
     upon the completion of each build. It can be configured to only send out
@@ -164,7 +182,7 @@ class MailNotifier(base.StatusReceiverMultiService):
                      "subject", "sendToInterestedUsers", "customMesg",
                      "messageFormatter", "extraHeaders"]
 
-    possible_modes = ("change", "failing", "passing", "problem", "warnings")
+    possible_modes = ("change", "failing", "passing", "problem", "warnings", "exception")
 
     def __init__(self, fromaddr, mode=("failing", "passing", "warnings"),
                  categories=None, builders=None, addLogs=False,
@@ -173,8 +191,9 @@ class MailNotifier(base.StatusReceiverMultiService):
                  lookup=None, extraRecipients=[],
                  sendToInterestedUsers=True, customMesg=None,
                  messageFormatter=defaultMessage, extraHeaders=None,
-                 addPatch=True, useTls=False, 
-                 smtpUser=None, smtpPassword=None, smtpPort=25):
+                 addPatch=True, useTls=False,
+                 smtpUser=None, smtpPassword=None, smtpPort=25,
+                 previousBuildGetter=defaultGetPreviousBuild):
         """
         @type  fromaddr: string
         @param fromaddr: the email address to be used in the 'From' header.
@@ -207,6 +226,8 @@ class MailNotifier(base.StatusReceiverMultiService):
                      - "problem": send mail about a build which failed
                                   when the previous build passed
                      - "warnings": send mail if a build contain warnings
+                     - "exception": send mail if a build fails due to an exception
+                     - "all": always send mail
                      Defaults to ("failing", "passing", "warnings").
 
         @type  builders: list of strings
@@ -288,6 +309,14 @@ class MailNotifier(base.StatusReceiverMultiService):
         @type smtpPort: int
         @param smtpPort: The port that will be used when connecting to the
                          relayhost. Defaults to 25.
+
+        @type previousBuildGetter: func
+        @param previousBuildGetter: function taking a BuildStatus instance
+                                    returning a BuildStatus of the build
+                                    previous to the one passed in. This allows
+                                    to implement a relative ordering between
+                                    builds other than the default one, which is
+                                    chronological.
         """
         base.StatusReceiverMultiService.__init__(self)
 
@@ -303,7 +332,7 @@ class MailNotifier(base.StatusReceiverMultiService):
         self.fromaddr = fromaddr
         if isinstance(mode, basestring):
             if mode == "all":
-                mode = ("failing", "passing", "warnings")
+                mode = ("failing", "passing", "warnings", "exception")
             elif mode == "warnings":
                 mode = ("failing", "warnings")
             else:
@@ -339,6 +368,7 @@ class MailNotifier(base.StatusReceiverMultiService):
         self.smtpPort = smtpPort
         self.buildSetSummary = buildSetSummary
         self.buildSetSubscription = None
+        self.getPreviousBuild = previousBuildGetter
         self.watched = []
         self.master_status = None
 
@@ -359,6 +389,7 @@ class MailNotifier(base.StatusReceiverMultiService):
         base.StatusReceiverMultiService.setServiceParent(self, parent)
         self.master_status = self.parent
         self.master_status.subscribe(self)
+        self.master = self.master_status.master
 
     def startService(self):
         if self.buildSetSummary:
@@ -407,7 +438,7 @@ class MailNotifier(base.StatusReceiverMultiService):
                builder.category not in self.categories:
             return False # ignore this build
 
-        prev = build.getPreviousBuild()
+        prev = self.getPreviousBuild(build)
         if "change" in self.mode:
             if prev and prev.getResults() != results:
                 return True
@@ -420,6 +451,8 @@ class MailNotifier(base.StatusReceiverMultiService):
                 return True
         if "warnings" in self.mode and results == WARNINGS:
             return True
+        if "exception" in self.mode and results == EXCEPTION:
+            return True
 
         return False
 
@@ -435,36 +468,36 @@ class MailNotifier(base.StatusReceiverMultiService):
             return self.buildMessage(name, [build], results)
         return None
     
-    def _gotBuilds(self, res, builddicts, buildset, builders):
+    def _gotBuilds(self, res, buildset):
         builds = []
-        for (builddictlist, builder) in zip(builddicts, builders):
-                for builddict in builddictlist:
-                    build = builder.getBuild(builddict['number'])
-                    if build is not None and self.isMailNeeded(build, build.results):
-                        builds.append(build)
+        for (builddictlist, builder) in res:
+            for builddict in builddictlist:
+                build = builder.getBuild(builddict['number'])
+                if build is not None and self.isMailNeeded(build, build.results):
+                    builds.append(build)
 
-        self.buildMessage("Buildset Complete: " + buildset['reason'], builds,
-                          buildset['results'])
+        if builds:
+            self.buildMessage("Buildset Complete: " + buildset['reason'], builds,
+                              buildset['results'])
         
     def _gotBuildRequests(self, breqs, buildset):
-        builddicts = []
-        builders =[]
         dl = []
         for breq in breqs:
             buildername = breq['buildername']
-            builders.append(self.master_status.getBuilder(buildername))
-            d = self.parent.db.builds.getBuildsForRequest(breq['brid'])
-            d.addCallback(builddicts.append)
+            builder = self.master_status.getBuilder(buildername)
+            d = self.master.db.builds.getBuildsForRequest(breq['brid'])
+            d.addCallback(lambda builddictlist, builder=builder:
+                          (builddictlist, builder))
             dl.append(d)
-        d = defer.DeferredList(dl)
-        d.addCallback(self._gotBuilds, builddicts, buildset, builders)
+        d = defer.gatherResults(dl)
+        d.addCallback(self._gotBuilds, buildset)
 
     def _gotBuildSet(self, buildset, bsid):
-        d = self.parent.db.buildrequests.getBuildRequests(bsid=bsid)
+        d = self.master.db.buildrequests.getBuildRequests(bsid=bsid)
         d.addCallback(self._gotBuildRequests, buildset)
         
     def buildsetFinished(self, bsid, result):
-        d = self.parent.db.buildsets.getBuildset(bsid=bsid)
+        d = self.master.db.buildsets.getBuildset(bsid=bsid)
         d.addCallback(self._gotBuildSet, bsid)
             
         return d
@@ -496,7 +529,7 @@ class MailNotifier(base.StatusReceiverMultiService):
                  'buildText': build.getText(),
                  'buildProperties': build.getProperties(),
                  'slavename': build.getSlavename(),
-                 'reason':  build.getReason(),
+                 'reason':  build.getReason().replace('\n', ''),
                  'responsibleUsers': build.getResponsibleUsers(),
                  'branch': "",
                  'revision': "",
@@ -505,13 +538,27 @@ class MailNotifier(base.StatusReceiverMultiService):
                  'changes': [],
                  'logs': logs}
 
-        ss = build.getSourceStamp()
-        if ss:
-            attrs['branch'] = ss.branch
-            attrs['revision'] = ss.revision
-            attrs['patch'] = ss.patch
-            attrs['patch_info'] = ss.patch_info
-            attrs['changes'] = ss.changes[:]
+        ss = None
+        ss_list = build.getSourceStamps()
+
+        if ss_list:
+            if len(ss_list) == 1:
+                ss = ss_list[0]
+                if ss:
+                    attrs['branch'] = ss.branch
+                    attrs['revision'] = ss.revision
+                    attrs['patch'] = ss.patch
+                    attrs['patch_info'] = ss.patch_info
+                    attrs['changes'] = ss.changes[:]
+            else:
+                for key in ['branch', 'revision', 'patch', 'patch_info', 'changes']:
+                    attrs[key] = {}
+                for ss in ss_list:
+                    attrs['branch'][ss.codebase] = ss.branch
+                    attrs['revision'][ss.codebase] = ss.revision
+                    attrs['patch'][ss.codebase] = ss.patch
+                    attrs['patch_info'][ss.codebase] = ss.patch_info
+                    attrs['changes'][ss.codebase] = ss.changes[:]
 
         return attrs
 
@@ -582,7 +629,9 @@ class MailNotifier(base.StatusReceiverMultiService):
                      self._shouldAttachLog(name) ):
                     text = log.getText()
                     if not isinstance(text, unicode):
-                        text = text.decode(LOG_ENCODING)
+                        # guess at the encoding, and use replacement symbols
+                        # for anything that's not in that encoding
+                        text = text.decode(LOG_ENCODING, 'replace')
                     a = MIMEText(text.encode(ENCODING),
                                  _charset=ENCODING)
                     a.add_header('Content-Disposition', "attachment",
@@ -594,18 +643,22 @@ class MailNotifier(base.StatusReceiverMultiService):
         # interpolation if only one build was given
         if self.extraHeaders:
             if len(builds) == 1:
-                extraHeaders = builds[0].render(self.extraHeaders)
+                d = builds[0].render(self.extraHeaders)
             else:
-                extraHeaders = self.extraHeaders
-            for k,v in extraHeaders.items():
-                if k in m:
-                    twlog.msg("Warning: Got header " + k +
-                      " in self.extraHeaders "
-                      "but it already exists in the Message - "
-                      "not adding it.")
-                m[k] = v
+                d = defer.succeed(self.extraHeaders)
+            @d.addCallback
+            def addExtraHeaders(extraHeaders):
+                for k,v in extraHeaders.items():
+                    if k in m:
+                        twlog.msg("Warning: Got header " + k +
+                          " in self.extraHeaders "
+                          "but it already exists in the Message - "
+                          "not adding it.")
+                    m[k] = v
+            d.addCallback(lambda _: m)
+            return d
     
-        return m
+        return defer.succeed(m)
     
     def buildMessageDict(self, name, build, results):
         if self.customMesg:
@@ -627,9 +680,11 @@ class MailNotifier(base.StatusReceiverMultiService):
         msgdict = {"body":""}
         
         for build in builds:
-            ss = build.getSourceStamp()
-            if ss and ss.patch and self.addPatch:
-                patches.append(ss.patch)
+            ss_list = build.getSourceStamps()
+            if self.addPatch:
+                for ss in ss_list:
+                    if ss.patch:
+                        patches.append(ss.patch)
             if self.addLogs:
                 logs.extend(build.getLogs())
             
@@ -641,69 +696,35 @@ class MailNotifier(base.StatusReceiverMultiService):
             if "subject" in tmp:
                 msgdict['subject'] = tmp['subject']
 
-        m = self.createEmail(msgdict, name, self.master_status.getTitle(),
+        d = self.createEmail(msgdict, name, self.master_status.getTitle(),
                              results, builds, patches, logs)
 
-        # now, who is this message going to?
-        if self.sendToInterestedUsers:
-            dl = []
-            for build in builds:
-                if self.lookup:
-                    d = self.useLookup(build)
-                else:
-                    d = self.useUsers(build)
-                dl.append(d)
-            d = defer.gatherResults(dl)
-        else:
-            d = defer.succeed([])
-        d.addCallback(self._gotRecipients, m)
+        @d.addCallback
+        def getRecipients(m):
+            # now, who is this message going to?
+            if self.sendToInterestedUsers:
+                dl = []
+                for build in builds:
+                    if self.lookup:
+                        d = self.useLookup(build)
+                    else:
+                        d = self.useUsers(build)
+                    dl.append(d)
+                d = defer.gatherResults(dl)
+            else:
+                d = defer.succeed([])
+            d.addCallback(self._gotRecipients, m)
         return d
 
     def useLookup(self, build):
         dl = []
-        for u in build.getInterestedUsers():
+        for u in build.getResponsibleUsers() + build.getInterestedUsers():
             d = defer.maybeDeferred(self.lookup.getAddress, u)
             dl.append(d)
         return defer.gatherResults(dl)
 
     def useUsers(self, build):
-        dl = []
-        ss = build.getSourceStamp()
-        for change in ss.changes:
-            d = self.parent.db.changes.getChangeUids(change.number)
-            def getContacts(uids):
-                def uidContactPair(contact, uid):
-                    return (contact, uid)
-                contacts = []
-                for uid in uids:
-                    d = users.getUserContact(self.parent,
-                            contact_type='email',
-                            uid=uid)
-                    d.addCallback(lambda contact: uidContactPair(contact, uid))
-                    contacts.append(d)
-                return defer.gatherResults(contacts)
-            d.addCallback(getContacts)
-            def logNoMatch(contacts):
-                for pair in contacts:
-                    contact, uid = pair
-                    if contact is None:
-                        twlog.msg("Unable to find email for uid: %r" % uid)
-                return [pair[0] for pair in contacts]
-            d.addCallback(logNoMatch)
-            def addOwners(recipients):
-                owners = [e for e in build.getInterestedUsers()
-                          if e not in build.getResponsibleUsers()]
-                recipients.extend(owners)
-                return recipients
-            d.addCallback(addOwners)
-            dl.append(d)
-        d = defer.gatherResults(dl)
-        @d.addCallback
-        def gatherRecipients(res):
-            recipients = []
-            map(recipients.extend, res)
-            return recipients
-        return d
+        return users.getBuildContacts(self.master, build, ['email'])
 
     def _shouldAttachLog(self, logname):
         if type(self.addLogs) is bool:
