@@ -22,43 +22,74 @@ from buildbot import interfaces
 from buildbot.status.web.base import HtmlResource, BuildLineMixin, \
     path_to_build, path_to_slave, path_to_builder, path_to_change, \
     path_to_root, ICurrentBox, build_get_class, \
-    map_branches, path_to_authzfail, ActionResource
-from buildbot.schedulers.forcesched import ForceScheduler, InheritBuildParameter
+    map_branches, path_to_authzfail, ActionResource, \
+    getRequestCharset
+from buildbot.schedulers.forcesched import ForceScheduler
+from buildbot.schedulers.forcesched import ValidationError
 from buildbot.status.web.build import BuildsResource, StatusResourceBuild
 from buildbot import util
+import collections
 
-class ForceAllBuildsActionResource(ActionResource):
+class ForceAction(ActionResource):
+    @defer.inlineCallbacks
+    def force(self, req, builderNames):
+        master = self.getBuildmaster(req)
+        owner = self.getAuthz(req).getUsernameFull(req)
+        schedulername = req.args.get("forcescheduler", ["<unknown>"])[0]
+        if schedulername == "<unknown>":
+            defer.returnValue((path_to_builder(req, self.builder_status),
+                               "forcescheduler arg not found"))
+            return
+
+        args = {}
+        # decode all of the args
+        encoding = getRequestCharset(req)
+        for name, argl in req.args.iteritems():
+           if name == "checkbox":
+               # damn html's ungeneric checkbox implementation...
+               for cb in argl:
+                   args[cb.decode(encoding)] = True
+           else:
+               args[name] = [ arg.decode(encoding) for arg in argl ]
+
+        for sch in master.allSchedulers():
+            if schedulername == sch.name:
+                try:
+                    yield sch.force(owner, builderNames, **args)
+                    msg = ""
+                except ValidationError, e:
+                    msg = html.escape(e.message.encode('ascii','ignore'))
+                break
+
+        # send the user back to the builder page
+        defer.returnValue(msg)
+
+
+class ForceAllBuildsActionResource(ForceAction):
 
     def __init__(self, status, selectedOrAll):
         self.status = status
         self.selectedOrAll = selectedOrAll
         self.action = "forceAllBuilds"
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def performAction(self, req):
         authz = self.getAuthz(req)
-        d = authz.actionAllowed('forceAllBuilds', req)
-        wfd = defer.waitForDeferred(d)
-        yield wfd
-        res = wfd.getResult()
+        res = yield authz.actionAllowed('forceAllBuilds', req)
 
         if not res:
-            yield path_to_authzfail(req)
+            defer.returnValue(path_to_authzfail(req))
             return
 
-        builders = None
         if self.selectedOrAll == 'all':
-            builders = self.status.getBuilderNames()
+            builderNames = None
         elif self.selectedOrAll == 'selected':
-            builders = [b for b in req.args.get("selected", []) if b]
+            builderNames = [b for b in req.args.get("selected", []) if b]
+
+        msg = yield self.force(req, builderNames)
 
-        for bname in builders:
-            builder_status = self.status.getBuilder(bname)
-            ar = ForceBuildActionResource(builder_status)
-            d = ar.performAction(req)
-            d.addErrback(log.err, "(ignored) while trying to force build")
         # back to the welcome page
-        yield path_to_root(req)
+        defer.returnValue((path_to_root(req) + "builders", msg))
 
 class StopAllBuildsActionResource(ActionResource):
 
@@ -67,15 +98,12 @@ class StopAllBuildsActionResource(ActionResource):
         self.selectedOrAll = selectedOrAll
         self.action = "stopAllBuilds"
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def performAction(self, req):
         authz = self.getAuthz(req)
-        d = authz.actionAllowed('stopAllBuilds', req)
-        wfd = defer.waitForDeferred(d)
-        yield wfd
-        res = wfd.getResult()
+        res = yield authz.actionAllowed('stopAllBuilds', req)
         if not res:
-            yield path_to_authzfail(req)
+            defer.returnValue(path_to_authzfail(req))
             return
 
         builders = None
@@ -95,8 +123,9 @@ class StopAllBuildsActionResource(ActionResource):
                     continue
                 build = StatusResourceBuild(build_status)
                 build.stop(req, auth_ok=True)
+
         # go back to the welcome page
-        yield path_to_root(req)
+        defer.returnValue(path_to_root(req))
 
 class PingBuilderActionResource(ActionResource):
 
@@ -104,86 +133,78 @@ class PingBuilderActionResource(ActionResource):
         self.builder_status = builder_status
         self.action = "pingBuilder"
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def performAction(self, req):
         log.msg("web ping of builder '%s'" % self.builder_status.getName())
-        d = self.getAuthz(req).actionAllowed('pingBuilder', req, self.builder_status)
-        wfd = defer.waitForDeferred(d)
-        yield wfd
-        res = wfd.getResult()
+        res = yield self.getAuthz(req).actionAllowed('pingBuilder', req,
+                                                    self.builder_status)
         if not res:
             log.msg("..but not authorized")
-            yield path_to_authzfail(req)
+            defer.returnValue(path_to_authzfail(req))
             return
 
         c = interfaces.IControl(self.getBuildmaster(req))
         bc = c.getBuilder(self.builder_status.getName())
         bc.ping()
         # send the user back to the builder page
-        yield path_to_builder(req, self.builder_status)
+        defer.returnValue(path_to_builder(req, self.builder_status))
 
-class ForceBuildActionResource(ActionResource):
+class ForceBuildActionResource(ForceAction):
 
     def __init__(self, builder_status):
         self.builder_status = builder_status
         self.action = "forceBuild"
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def performAction(self, req):
         # check if this is allowed
-        d = self.getAuthz(req).actionAllowed(self.action, req,
+        res = yield self.getAuthz(req).actionAllowed(self.action, req,
                                              self.builder_status)
-        wfd = defer.waitForDeferred(d)
-        yield wfd
-        res = wfd.getResult()
         if not res:
             log.msg("..but not authorized")
-            yield path_to_authzfail(req)
+            defer.returnValue(path_to_authzfail(req))
             return
 
-        master = self.getBuildmaster(req)
-        owner = self.getAuthz(req).getUsernameFull(req)
-        schedulername = req.args.get("forcescheduler", ["<unknown>"])[0]
-        if schedulername == "<unknown>":
-            yield path_to_builder(req, self.builder_status), "forcescheduler arg not found"
-            return
-        for sch in master.allSchedulers():
-            if schedulername == sch.name:
-                try:
-                    d = sch.forceWithWebRequest(owner,self.builder_status.getName(),req)
-                    msg = ""
-                except Exception, e:
-                    msg = html.escape(e.message.encode('ascii','ignore'))
-                break
-        wfd = defer.waitForDeferred(d)
-        yield wfd
-        res = wfd.getResult()
+        builderName = self.builder_status.getName()
+
+        msg = yield self.force(req, [builderName])
+
         # send the user back to the builder page
-        yield path_to_builder(req, self.builder_status), msg
+        defer.returnValue((path_to_builder(req, self.builder_status), msg))
+
+def buildForceContextForField(req, default_props, sch, field, master, buildername):
+    pname = "%s.%s"%(sch.name, field.fullName)
+    
+    default = field.default
+
+    if "list" in field.type:
+        choices = field.getChoices(master, sch, buildername)
+        if choices:
+            default = choices[0]
+        default_props[pname+".choices"] = choices
+            
+    default = req.args.get(pname, [default])[0]
+    if "bool" in field.type:
+        default = "checked" if default else ""
+    elif isinstance(default, unicode):
+        # filter out unicode chars, and html stuff
+        default = html.escape(default.encode('utf-8','ignore'))
+
+    default_props[pname] = default
+        
+    if "nested" in field.type:
+        for subfield in field.fields:
+            buildForceContextForField(req, default_props, sch, subfield, master, buildername)
 
 def buildForceContext(cxt, req, master, buildername=None):
     force_schedulers = {}
-    default_props = {}
+    default_props = collections.defaultdict(str)
     for sch in master.allSchedulers():
         if isinstance(sch, ForceScheduler) and (buildername is None or(buildername in sch.builderNames)):
             force_schedulers[sch.name] = sch
-            for p in sch.all_fields:
-                pname = "%s.%s"%(sch.name, p.name)
-                default = p.default
-                if isinstance(p, InheritBuildParameter):
-                    # yes, I know, its bad to overwrite the parameter attribute,
-                    # but I dont have any other simple way of doing this atm.
-                    p.choices = p.compatible_builds(master.status, buildername)
-                    if p.choices:
-                        default = p.choices[0]
-                default = req.args.get(pname, [default])[0]
-                if p.type=="bool":
-                    default_props[pname] = default and "checked" or ""
-                else:
-                    # filter out unicode chars, and html stuff
-                    if type(default)==unicode:
-                        default = html.escape(default.encode('ascii','ignore'))
-                    default_props[pname] = default
+            for field in sch.all_fields:
+                buildForceContextForField(req, default_props, sch, field, master, buildername)
+                
     cxt['force_schedulers'] = force_schedulers
     cxt['default_props'] = default_props
 
@@ -191,9 +212,10 @@ def buildForceContext(cxt, req, master, buildername=None):
 class StatusResourceBuilder(HtmlResource, BuildLineMixin):
     addSlash = True
 
-    def __init__(self, builder_status):
+    def __init__(self, builder_status, numbuilds=20):
         HtmlResource.__init__(self)
         self.builder_status = builder_status
+        self.numbuilds = numbuilds
 
     def getPageTitle(self, request):
         return "Buildbot: %s" % self.builder_status.getName()
@@ -224,11 +246,12 @@ class StatusResourceBuilder(HtmlResource, BuildLineMixin):
 
         return b
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def content(self, req, cxt):
         b = self.builder_status
 
         cxt['name'] = b.getName()
+        cxt['description'] = b.getDescription()
         req.setHeader('Cache-Control', 'no-cache')
         slaves = b.getSlaves()
         connected_slaves = [s for s in slaves if s.isConnected()]
@@ -236,32 +259,16 @@ class StatusResourceBuilder(HtmlResource, BuildLineMixin):
         cxt['current'] = [self.builder(x, req) for x in b.getCurrentBuilds()]
 
         cxt['pending'] = []
-        wfd = defer.waitForDeferred(
-            b.getPendingBuildRequestStatuses())
-        yield wfd
-        statuses = wfd.getResult()
+        statuses = yield b.getPendingBuildRequestStatuses()
         for pb in statuses:
             changes = []
 
-            wfd = defer.waitForDeferred(
-                    pb.getSourceStamp())
-            yield wfd
-            source = wfd.getResult()
+            source = yield pb.getSourceStamp()
+            submitTime = yield pb.getSubmitTime()
+            bsid = yield pb.getBsid()
 
-            wfd = defer.waitForDeferred(
-                    pb.getSubmitTime())
-            yield wfd
-            submitTime = wfd.getResult()
-
-            wfd = defer.waitForDeferred(
-                    pb.getBsid())
-            yield wfd
-            bsid = wfd.getResult()
-
-            wfd = defer.waitForDeferred(
-                pb.master.db.buildsets.getBuildsetProperties(bsid))
-            yield wfd
-            properties = wfd.getResult()
+            properties = yield \
+                    pb.master.db.buildsets.getBuildsetProperties(bsid)
 
             if source.changes:
                 for c in source.changes:
@@ -280,7 +287,7 @@ class StatusResourceBuilder(HtmlResource, BuildLineMixin):
                 'properties' : properties,
                 })
 
-        numbuilds = int(req.args.get('numbuilds', ['5'])[0])
+        numbuilds = cxt['numbuilds'] = int(req.args.get('numbuilds', [self.numbuilds])[0])
         recent = cxt['recent'] = []
         for build in b.generateFinishedBuilds(num_builds=int(numbuilds)):
             recent.append(self.get_line_values(req, build, False))
@@ -293,8 +300,9 @@ class StatusResourceBuilder(HtmlResource, BuildLineMixin):
             s['link'] = path_to_slave(req, slave)
             s['name'] = slave.getName()
             c = s['connected'] = slave.isConnected()
+            s['paused'] = slave.isPaused()
+            s['admin'] = unicode(slave.getAdmin() or '', 'utf-8')
             if c:
-                s['admin'] = unicode(slave.getAdmin() or '', 'utf-8')
                 connected_slaves += 1
         cxt['connected_slaves'] = connected_slaves
 
@@ -302,7 +310,7 @@ class StatusResourceBuilder(HtmlResource, BuildLineMixin):
         cxt['builder_url'] = path_to_builder(req, b)
         buildForceContext(cxt, req, self.getBuildmaster(req), b.getName())
         template = req.site.buildbot_service.templates.get_template("builder.html")
-        yield template.render(**cxt)
+        defer.returnValue(template.render(**cxt))
 
     def ping(self, req):
         return PingBuilderActionResource(self.builder_status)
@@ -327,7 +335,7 @@ class CancelChangeResource(ActionResource):
         ActionResource.__init__(self)
         self.builder_status = builder_status
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def performAction(self, req):
         try:
             request_id = req.args.get("id", [None])[0]
@@ -344,32 +352,26 @@ class CancelChangeResource(ActionResource):
             c = interfaces.IControl(self.getBuildmaster(req))
             builder_control = c.getBuilder(self.builder_status.getName())
 
-            wfd = defer.waitForDeferred(
-                    builder_control.getPendingBuildRequestControls())
-            yield wfd
-            brcontrols = wfd.getResult()
+            brcontrols = yield builder_control.getPendingBuildRequestControls()
 
             for build_req in brcontrols:
                 if cancel_all or (build_req.brid == request_id):
                     log.msg("Cancelling %s" % build_req)
-                    d = authz.actionAllowed('cancelPendingBuild',
-                                            req, build_req)
-                    wfd = defer.waitForDeferred(d)
-                    yield wfd
-                    res = wfd.getResult()
+                    res = yield authz.actionAllowed('cancelPendingBuild', req,
+                                                                build_req)
                     if res:
                         build_req.cancel()
                     else:
-                        yield path_to_authzfail(req)
+                        defer.returnValue(path_to_authzfail(req))
                         return
                     if not cancel_all:
                         break
 
-        yield path_to_builder(req, self.builder_status)
+        defer.returnValue(path_to_builder(req, self.builder_status))
 
 class StopChangeMixin(object):
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def stopChangeForBuilder(self, req, builder_status, auth_ok=False):
         try:
             request_change = req.args.get("change", [None])[0]
@@ -382,23 +384,14 @@ class StopChangeMixin(object):
             c = interfaces.IControl(self.getBuildmaster(req))
             builder_control = c.getBuilder(builder_status.getName())
 
-            wfd = defer.waitForDeferred(
-                    builder_control.getPendingBuildRequestControls())
-            yield wfd
-            brcontrols = wfd.getResult()
-
+            brcontrols = yield builder_control.getPendingBuildRequestControls()
             build_controls = dict((x.brid, x) for x in brcontrols)
 
-            wfd = defer.waitForDeferred(
-                    builder_status.getPendingBuildRequestStatuses())
-            yield wfd
-            build_req_statuses = wfd.getResult()
+            build_req_statuses = yield \
+                    builder_status.getPendingBuildRequestStatuses()
 
             for build_req in build_req_statuses:
-                wfd = defer.waitForDeferred(
-                        build_req.getSourceStamp())
-                yield wfd
-                ss = wfd.getResult()
+                ss = yield build_req.getSourceStamp()
 
                 if not ss.changes:
                     continue
@@ -407,17 +400,14 @@ class StopChangeMixin(object):
                     if change.number == request_change:
                         control = build_controls[build_req.brid]
                         log.msg("Cancelling %s" % control)
-                        d = authz.actionAllowed('stopChange', req, control)
-                        wfd = defer.waitForDeferred(d)
-                        yield wfd
-                        res = wfd.getResult()
+                        res = yield authz.actionAllowed('stopChange', req, control)
                         if (auth_ok or res):
                             control.cancel()
                         else:
-                            yield False
+                            defer.returnValue(False)
                             return
 
-        yield True
+        defer.returnValue(True)
 
 
 class StopChangeResource(StopChangeMixin, ActionResource):
@@ -426,18 +416,15 @@ class StopChangeResource(StopChangeMixin, ActionResource):
         ActionResource.__init__(self)
         self.builder_status = builder_status
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def performAction(self, req):
         """Cancel all pending builds that include a given numbered change."""
-        wfd = defer.waitForDeferred(
-            self.stopChangeForBuilder(req, self.builder_status))
-        yield wfd
-        success = wfd.getResult()
+        success = yield self.stopChangeForBuilder(req, self.builder_status)
 
         if not success:
-            yield path_to_authzfail(req)
+            defer.returnValue(path_to_authzfail(req))
         else:
-            yield path_to_builder(req, self.builder_status)
+            defer.returnValue(path_to_builder(req, self.builder_status))
 
 
 class StopChangeAllResource(StopChangeMixin, ActionResource):
@@ -446,27 +433,23 @@ class StopChangeAllResource(StopChangeMixin, ActionResource):
         ActionResource.__init__(self)
         self.status = status
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def performAction(self, req):
         """Cancel all pending builds that include a given numbered change."""
         authz = self.getAuthz(req)
-        wfd = defer.waitForDeferred(authz.actionAllowed('stopChange', req))
-        yield wfd
-        res = wfd.getResult()
+        res = yield authz.actionAllowed('stopChange', req)
         if not res:
-            yield path_to_authzfail(req)
+            defer.returnValue(path_to_authzfail(req))
             return
 
         for bname in self.status.getBuilderNames():
             builder_status = self.status.getBuilder(bname)
-            wfd = defer.waitForDeferred(
-                self.stopChangeForBuilder(req, builder_status, auth_ok=True))
-            yield wfd
-            if not wfd.getResult():
-                yield path_to_authzfail(req)
+            res = yield self.stopChangeForBuilder(req, builder_status, auth_ok=True)
+            if not res:
+                defer.returnValue(path_to_authzfail(req))
                 return
 
-        yield path_to_root(req)
+        defer.returnValue(path_to_root(req))
 
 
 # /builders/_all
@@ -518,12 +501,19 @@ class BuildersResource(HtmlResource):
     pageTitle = "Builders"
     addSlash = True
 
-    @defer.deferredGenerator
+    def __init__(self, numbuilds=20):
+        HtmlResource.__init__(self)
+        self.numbuilds = numbuilds
+
+    @defer.inlineCallbacks
     def content(self, req, cxt):
         status = self.getStatus(req)
+        encoding = getRequestCharset(req)
 
         builders = req.args.get("builder", status.getBuilderNames())
-        branches = [b for b in req.args.get("branch", []) if b]
+        branches = [ b.decode(encoding)
+                for b in req.args.get("branch", [])
+                if b ]
 
         # get counts of pending builds for each builder
         brstatus_ds = []
@@ -535,10 +525,7 @@ class BuildersResource(HtmlResource):
             d = builder_status.getPendingBuildRequestStatuses()
             d.addCallback(keep_count, builderName)
             brstatus_ds.append(d)
-        wfd = defer.waitForDeferred(
-            defer.gatherResults(brstatus_ds))
-        yield wfd
-        wfd.getResult()
+        yield defer.gatherResults(brstatus_ds)
 
         cxt['branches'] = branches
         bs = cxt['builders'] = []
@@ -557,7 +544,11 @@ class BuildersResource(HtmlResource):
             if builds:
                 b = builds[0]
                 bld['build_url'] = (bld['link'] + "/builds/%d" % b.getNumber())
-                label = b.getProperty("got_revision")
+                label = None
+                all_got_revisions = b.getAllGotRevisions()
+                # If len = 1 then try if revision can be used as label.
+                if len(all_got_revisions) == 1:
+                    label = all_got_revisions[all_got_revisions.keys()[0]]
                 if not label or len(str(label)) > 20:
                     label = "#%d" % b.getNumber()
 
@@ -580,13 +571,13 @@ class BuildersResource(HtmlResource):
         cxt['num_online'] = online
         buildForceContext(cxt, req, self.getBuildmaster(req))
         template = req.site.buildbot_service.templates.get_template("builders.html")
-        yield template.render(**cxt)
+        defer.returnValue(template.render(**cxt))
 
     def getChild(self, path, req):
         s = self.getStatus(req)
         if path in s.getBuilderNames():
             builder_status = s.getBuilder(path)
-            return StatusResourceBuilder(builder_status)
+            return StatusResourceBuilder(builder_status, self.numbuilds)
         if path == "_all":
             return StatusResourceAllBuilders(self.getStatus(req))
         if path == "_selected":
