@@ -1,13 +1,27 @@
-from __future__ import generators
+# This file is part of Buildbot.  Buildbot is free software: you can
+# redistribute it and/or modify it under the terms of the GNU General Public
+# License as published by the Free Software Foundation, version 2.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
+#
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, write to the Free Software Foundation, Inc., 51
+# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# Copyright Buildbot Team Members
 
 import time
 import operator
 import re
 import urllib
-
+from twisted.internet import defer
 from buildbot import util
 from buildbot.status import builder
 from buildbot.status.web.base import HtmlResource
+from buildbot.changes import changes
 
 class DoesNotPassFilter(Exception): pass # Used for filtering revs
 
@@ -38,7 +52,7 @@ def getResultsClass(results, prevResults, inProgress):
             return "failure"
         else:
             # The previous build also failed.
-            return "warnings"
+            return "failure-again"
   
     # Any other results? Like EXCEPTION?
     return "exception"
@@ -71,7 +85,8 @@ class DevBuild:
         self.eta = build.getETA()
         self.details = details
         self.when = build.getTimes()[0]
-        self.source = build.getSourceStamp()
+        #TODO: support multiple sourcestamps
+        self.source = build.getSourceStamps()[0]
 
 
 class ConsoleStatusResource(HtmlResource):
@@ -89,11 +104,11 @@ class ConsoleStatusResource(HtmlResource):
         else:
             self.comparator = IntegerRevisionComparator()
 
-    def getTitle(self, request):
+    def getPageTitle(self, request):
         status = self.getStatus(request)
-        projectName = status.getProjectName()
-        if projectName:
-            return "BuildBot: %s" % projectName
+        title = status.getTitle()
+        if title:
+            return "BuildBot: %s" % title
         else:
             return "BuildBot"
 
@@ -141,26 +156,23 @@ class ConsoleStatusResource(HtmlResource):
             while build and depth < max_depth and build_count < max_builds:
                 depth += 1
                 build_count += 1
-                sourcestamp = build.getSourceStamp()
+                sourcestamp = build.getSourceStamps()[0]
                 allChanges.extend(sourcestamp.changes[:])
                 build = build.getPreviousBuild()
 
         debugInfo["source_fetch_len"] = len(allChanges)
         return allChanges                
 
-    def getAllChanges(self, source, status, debugInfo):
-        """Return all the changes we can find at this time. If |source| does not
-        not have enough (less than 25), we try to fetch more from the builders
-        history."""
+    @defer.inlineCallbacks
+    def getAllChanges(self, request, status, debugInfo):
+        master = request.site.buildbot_service.master
 
-        g = source.eventGenerator()
-        allChanges = []
-        while len(allChanges) < 25:
-            try:
-                c = g.next()
-            except StopIteration:
-                break
-            allChanges.append(c)
+        chdicts = yield master.db.changes.getRecentChanges(25)
+
+        # convert those to Change instances
+        allChanges = yield defer.gatherResults([
+                changes.Change.fromChdict(master, chdict)
+                for chdict in chdicts ])
 
         allChanges.sort(key=self.comparator.getSortingKey())
 
@@ -174,7 +186,7 @@ class ConsoleStatusResource(HtmlResource):
             prevChange = change
         allChanges = newChanges
 
-        return allChanges
+        defer.returnValue(allChanges)
 
     def getBuildDetails(self, request, builderName, build):
         """Returns an HTML list of failures for a given build."""
@@ -208,8 +220,8 @@ class ConsoleStatusResource(HtmlResource):
                         logs.append(dict(url=logurl, name=logname))
         return details
 
-    def getBuildsForRevision(self, request, builder, builderName, lastRevision,
-                             numBuilds, debugInfo):
+    def getBuildsForRevision(self, request, builder, builderName, codebase,
+                             lastRevision, numBuilds, debugInfo):
         """Return the list of all the builds for a given builder that we will
         need to be able to display the console page. We start by the most recent
         build, and we go down until we find a build that was built prior to the
@@ -222,32 +234,28 @@ class ConsoleStatusResource(HtmlResource):
         number = 0
         while build and number < numBuilds:
             debugInfo["builds_scanned"] += 1
-            number += 1
 
-            # Get the last revision in this build.
-            # We first try "got_revision", but if it does not work, then
-            # we try "revision".
-            got_rev = -1
-            try:
-                got_rev = build.getProperty("got_revision")
-                if not self.comparator.isValidRevision(got_rev):
-                    got_rev = -1
-            except KeyError:
-                pass
-
-            try:
-                if got_rev == -1:
-                    got_rev = build.getProperty("revision")
-                if not self.comparator.isValidRevision(got_rev):
-                    got_rev = -1
-            except:
-                pass
+            got_rev = None
+            sourceStamps = build.getSourceStamps(absolute=True)
 
+            # The console page cannot handle builds that have more than 1 revision
+            if codebase is not None:
+                # Get the last revision in this build for this codebase.
+                for ss in sourceStamps:
+                    if ss.codebase == codebase:
+                        got_rev = ss.revision
+                        break
+            elif len(sourceStamps) == 1:
+                ss = sourceStamps[0]
+                # Get the last revision in this build.
+                got_rev = ss.revision
+                    
             # We ignore all builds that don't have last revisions.
             # TODO(nsylvain): If the build is over, maybe it was a problem
             # with the update source step. We need to find a way to tell the
             # user that his change might have broken the source update.
-            if got_rev and got_rev != -1:
+            if got_rev is not None:
+                number += 1
                 details = self.getBuildDetails(request, builderName, build)
                 devBuild = DevBuild(got_rev, build, details)
                 builds.append(devBuild)
@@ -276,13 +284,14 @@ class ConsoleStatusResource(HtmlResource):
         changes.sort(key=self.comparator.getSortingKey())
         return changes[-1]
     
-    def getAllBuildsForRevision(self, status, request, lastRevision, numBuilds,
-                                categories, builders, debugInfo):
+    def getAllBuildsForRevision(self, status, request, codebase, lastRevision,
+                                numBuilds, categories, builders, debugInfo):
         """Returns a dictionary of builds we need to inspect to be able to
         display the console page. The key is the builder name, and the value is
         an array of build we care about. We also returns a dictionary of
         builders we care about. The key is it's category.
  
+        codebase is the codebase to get revisions from
         lastRevision is the last revision we want to display in the page.
         categories is a list of categories to display. It is coming from the
             HTTP GET parameters.
@@ -324,6 +333,7 @@ class ConsoleStatusResource(HtmlResource):
             allBuilds[builderName] = self.getBuildsForRevision(request,
                                                                builder,
                                                                builderName,
+                                                               codebase,
                                                                lastRevision,
                                                                numBuilds,
                                                                debugInfo)
@@ -349,11 +359,8 @@ class ConsoleStatusResource(HtmlResource):
         
         for category in categories:            
             c = {}
-            # TODO(nsylvain): Another hack to display the category in a pretty
-            # way.  If the master owner wants to display the categories in a
-            # given order, he/she can prepend a number to it. This number won't
-            # be shown.
-            c["name"] = category.lstrip('0123456789')
+
+            c["name"] = category
 
             # To be able to align the table correctly, we need to know
             # what percentage of space this category will be taking. This is
@@ -387,7 +394,7 @@ class ConsoleStatusResource(HtmlResource):
             for builder in builderList[category]:
                 s = {}
                 s["color"] = "notstarted"
-                s["title"] = builder
+                s["pageTitle"] = builder
                 s["url"] = "./builders/%s" % urllib.quote(builder)
                 state, builds = status.getBuilder(builder).getState()
                 # Check if it's offline, if so, the box is purple.
@@ -456,15 +463,15 @@ class ConsoleStatusResource(HtmlResource):
                     isRunning = True
 
                 url = "./waterfall"
-                title = builder
+                pageTitle = builder
                 tag = ""
                 current_details = {}
                 if introducedIn:
                     current_details = introducedIn.details or ""
                     url = "./buildstatus?builder=%s&number=%s" % (urllib.quote(builder),
                                                                   introducedIn.number)
-                    title += " "
-                    title += urllib.quote(' '.join(introducedIn.text), ' \n\\/:')
+                    pageTitle += " "
+                    pageTitle += urllib.quote(' '.join(introducedIn.text), ' \n\\/:')
 
                     builderStrip = builder.replace(' ', '')
                     builderStrip = builderStrip.replace('(', '')
@@ -473,13 +480,13 @@ class ConsoleStatusResource(HtmlResource):
                     tag = "Tag%s%s" % (builderStrip, introducedIn.number)
 
                 if isRunning:
-                    title += ' ETA: %ds' % (introducedIn.eta or 0)
+                    pageTitle += ' ETA: %ds' % (introducedIn.eta or 0)
                     
                 resultsClass = getResultsClass(results, previousResults, isRunning)
 
                 b = {}                
                 b["url"] = url
-                b["title"] = title
+                b["pageTitle"] = pageTitle
                 b["color"] = resultsClass
                 b["tag"] = tag
 
@@ -523,13 +530,16 @@ class ConsoleStatusResource(HtmlResource):
                 except DoesNotPassFilter:
                     pass
 
-    def displayPage(self, request, status, builderList, allBuilds, revisions,
-                    categories, repository, branch, debugInfo):
+    def displayPage(self, request, status, builderList, allBuilds, codebase,
+                    revisions, categories, repository, project, branch,
+                    debugInfo):
         """Display the console page."""
         # Build the main template directory with all the informations we have.
         subs = dict()
         subs["branch"] = branch or 'trunk'
         subs["repository"] = repository
+        subs["project"] = project
+        subs["codebase"] = codebase
         if categories:
             subs["categories"] = ' '.join(categories)
         subs["time"] = time.strftime("%a %d %b %Y %H:%M:%S",
@@ -613,20 +623,18 @@ class ConsoleStatusResource(HtmlResource):
         builders = request.args.get("builder", [])
         # Repo used to filter the changes shown.
         repository = request.args.get("repository", [None])[0]
+        # Project used to filter the changes shown.
+        project = request.args.get("project", [None])[0]
         # Branch used to filter the changes shown.
         branch = request.args.get("branch", [ANYBRANCH])[0]
+        # Codebase used to filter the changes shown.
+        codebase = request.args.get("codebase", [None])[0]
         # List of all the committers name to display on the page.
         devName = request.args.get("name", [])
 
         # and the data we want to render
         status = self.getStatus(request)
 
-        # Get all revisions we can find.
-        source = self.getChangeManager(request)
-        allChanges = self.getAllChanges(source, status, debugInfo)
-
-        debugInfo["source_all"] = len(allChanges)
-
         # Keep only the revisions we care about.
         # By default we process the last 40 revisions.
         # If a dev name is passed, we look for the changes by this person in the
@@ -636,40 +644,57 @@ class ConsoleStatusResource(HtmlResource):
             numRevs *= 2
         numBuilds = numRevs
 
-        revFilter = {}
-        if branch != ANYBRANCH:
-            revFilter['branch'] = branch
-        if devName:
-            revFilter['who'] = devName
-        if repository:
-            revFilter['repository'] = repository
-        revisions = list(self.filterRevisions(allChanges, max_revs=numRevs, filter=revFilter))
-        debugInfo["revision_final"] = len(revisions)
+        # Get all changes we can find.  This is a DB operation, so it must use
+        # a deferred.
+        d = self.getAllChanges(request, status, debugInfo)
+        def got_changes(allChanges):
+            debugInfo["source_all"] = len(allChanges)
 
-        # Fetch all the builds for all builders until we get the next build
-        # after lastRevision.
-        builderList = None
-        allBuilds = None
-        if revisions:
-            lastRevision = revisions[len(revisions) - 1].revision
-            debugInfo["last_revision"] = lastRevision
+            revFilter = {}
+            if branch != ANYBRANCH:
+                revFilter['branch'] = branch
+            if devName:
+                revFilter['who'] = devName
+            if repository:
+                revFilter['repository'] = repository
+            if project:
+                revFilter['project'] = project
+            if codebase is not None:
+                revFilter['codebase'] = codebase
+            revisions = list(self.filterRevisions(allChanges, max_revs=numRevs,
+                                                            filter=revFilter))
+            debugInfo["revision_final"] = len(revisions)
 
-            (builderList, allBuilds) = self.getAllBuildsForRevision(status,
-                                                request,
-                                                lastRevision,
-                                                numBuilds,
-                                                categories,
-                                                builders,
-                                                debugInfo)
+            # Fetch all the builds for all builders until we get the next build
+            # after lastRevision.
+            builderList = None
+            allBuilds = None
+            if revisions:
+                lastRevision = revisions[len(revisions) - 1].revision
+                debugInfo["last_revision"] = lastRevision
 
-        debugInfo["added_blocks"] = 0
+                (builderList, allBuilds) = self.getAllBuildsForRevision(status,
+                                                    request,
+                                                    codebase,
+                                                    lastRevision,
+                                                    numBuilds,
+                                                    categories,
+                                                    builders,
+                                                    debugInfo)
 
-        cxt.update(self.displayPage(request, status, builderList, allBuilds,
-                                    revisions, categories, repository, branch, debugInfo))
+            debugInfo["added_blocks"] = 0
 
-        template = request.site.buildbot_service.templates.get_template("console.html")
-        data = template.render(cxt)
-        return data
+            cxt.update(self.displayPage(request, status, builderList,
+                                        allBuilds, codebase, revisions,
+                                        categories, repository, project,
+                                        branch, debugInfo))
+
+            templates = request.site.buildbot_service.templates
+            template = templates.get_template("console.html")
+            data = template.render(cxt)
+            return data
+        d.addCallback(got_changes)
+        return d
 
 class RevisionComparator(object):
     """Used for comparing between revisions, as some
@@ -704,7 +729,7 @@ class IntegerRevisionComparator(RevisionComparator):
     def isRevisionEarlier(self, first, second):
         try:
             return int(first.revision) < int(second.revision)
-        except TypeError:
+        except (TypeError, ValueError):
             return False
 
     def isValidRevision(self, revision):
