@@ -1,3 +1,18 @@
+# This file is part of Buildbot.  Buildbot is free software: you can
+# redistribute it and/or modify it under the terms of the GNU General Public
+# License as published by the Free Software Foundation, version 2.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
+#
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, write to the Free Software Foundation, Inc., 51
+# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# Copyright Buildbot Team Members
+
 # This module enables ATOM and RSS feeds from webstatus.
 #
 # It is based on "feeder.py" which was part of the Buildbot
@@ -27,7 +42,7 @@ import os
 import re
 import time
 from twisted.web import resource
-from buildbot.status.builder import FAILURE
+from buildbot.status import results
 
 class XmlResource(resource.Resource):
     contentType = "text/xml; charset=UTF-8"
@@ -44,18 +59,28 @@ class XmlResource(resource.Resource):
             return ''
         return data
 
+_abbr_day = [ 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun']
+_abbr_mon = ['', 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug',
+            'Sep', 'Oct', 'Nov', 'Dec']
+
+def rfc822_time(tstamp):
+    res = time.strftime("%%s, %d %%s %Y %H:%M:%S GMT",
+                                       tstamp)
+    res = res % (_abbr_day[tstamp.tm_wday], _abbr_mon[tstamp.tm_mon])
+    return res
+
 class FeedResource(XmlResource):
-    title = None
+    pageTitle = None
     link = 'http://dummylink'
     language = 'en-us'
     description = 'Dummy rss'
     status = None
 
-    def __init__(self, status, categories=None, title=None):
+    def __init__(self, status, categories=None, pageTitle=None):
         self.status = status
         self.categories = categories
-        self.title = title
-        self.projectName = self.status.getProjectName()
+        self.pageTitle = pageTitle
+        self.title = self.status.getTitle()
         self.link = self.status.getBuildbotURL()
         self.description = 'List of builds'
         self.pubdate = time.gmtime(int(time.time()))
@@ -97,7 +122,8 @@ class FeedResource(XmlResource):
         if showCategories:
             builders = [b for b in builders if b.category in showCategories]
 
-        failures_only = request.args.get("failures_only", "false")
+        failures_only = request.args.get("failures_only", ["false"])
+        failures_only = failures_only[0] not in ('false', '0', 'no', 'off')
 
         maxFeeds = 25
 
@@ -105,29 +131,11 @@ class FeedResource(XmlResource):
         # This could clearly be implemented much better if we had
         # access to a global list of builds.
         for b in builders:
-            lastbuild = b.getLastFinishedBuild()
-            if lastbuild is None:
-                continue
-
-            lastnr = lastbuild.getNumber()
-
-            totalbuilds = 0
-            i = lastnr
-            while i >= 0:
-                build = b.getBuild(i)
-                i -= 1
-                if not build:
-                    continue
-
-                results = build.getResults()
-
-                if failures_only == "false" or results == FAILURE:
-                    totalbuilds += 1
-                    builds.append(build)
-
-                # stop for this builder when our total nr. of feeds is reached
-                if totalbuilds >= maxFeeds:
-                    break
+            if failures_only:
+                res = (results.FAILURE,)
+            else:
+                res = None
+            builds.extend(b.generateFinishedBuilds(results=res, max_search=maxFeeds))
 
         # Sort build list by date, youngest first.
         # To keep compatibility with python < 2.4, use this for sorting instead:
@@ -153,39 +161,41 @@ class FeedResource(XmlResource):
 
             # title: trunk r22191 (plus patch) failed on
             # 'i686-debian-sarge1 shared gcc-3.3.5'
-            ss = build.getSourceStamp()
-            source = ""
-            if ss.branch:
-                source += "Branch %s " % ss.branch
-            if ss.revision:
-                source += "Revision %s " % str(ss.revision)
-            if ss.patch:
-                source += " (plus patch)"
-            if ss.changes:
-                pass
-            if (ss.branch is None and ss.revision is None and ss.patch is None
-                and not ss.changes):
-                source += "Latest revision "
-            got_revision = None
-            try:
-                got_revision = build.getProperty("got_revision")
-            except KeyError:
-                pass
-            if got_revision:
-                got_revision = str(got_revision)
-                if len(got_revision) > 40:
-                    got_revision = "[revision string too long]"
-                source += "(Got Revision: %s)" % got_revision
-            failflag = (build.getResults() != FAILURE)
-            title = ('%s %s on "%s"' %
-                     (source, ["failed","succeeded"][failflag],
-                      build.getBuilder().getName()))
+            ss_list = build.getSourceStamps()
+            all_got_revisions = build.getAllGotRevisions()
+            src_cxts = []
+            for ss in ss_list:
+                sc = {}
+                sc['codebase'] = ss.codebase
+                if (ss.branch is None and ss.revision is None and ss.patch is None
+                    and not ss.changes):
+                    sc['repository'] = None
+                    sc['branch'] = None
+                    sc['revision'] = "Latest revision"
+                else:
+                    sc['repository'] = ss.repository
+                    sc['branch'] = ss.branch
+                    got_revision = all_got_revisions.get(ss.codebase, None)
+                    if got_revision:
+                        sc['revision'] = got_revision
+                    else:
+                        sc['revision'] = str(ss.revision)
+                if ss.patch:
+                    sc['revision'] += " (plus patch)"
+                if ss.changes:
+                    pass
+                src_cxts.append(sc)
+            res = build.getResults()
+            pageTitle = ('Builder "%s": %s' %
+                (build.getBuilder().getName(), results.Results[res]))
 
             # Add information about the failing steps.
             failed_steps = []
             log_lines = []
             for s in build.getSteps():
-                if s.getResults()[0] == FAILURE:
+                res = s.getResults()[0]
+                if res not in (results.SUCCESS, results.WARNINGS,
+                               results.SKIPPED):
                     failed_steps.append(s.getName())
 
                     # Add the last 30 lines of each log.
@@ -198,12 +208,14 @@ class FeedResource(XmlResource):
                         except IOError:
                             # Probably the log file has been removed
                             logdata ='** log file not available **'
-
-                        log_lines.extend(logdata.split('\n')[-30:])
+                        unilist = list()
+                        for line in logdata.split('\n')[-30:]:
+                            unilist.append(unicode(line,'utf-8'))
+                        log_lines.extend(unilist)
 
             bc = {}
-            bc['date'] = time.strftime("%a, %d %b %Y %H:%M:%S GMT",
-                                       finishedTime)
+            bc['sources'] = src_cxts
+            bc['date'] = rfc822_time(finishedTime)
             bc['summary_link'] = ('%sbuilders/%s' %
                                   (self.link,
                                    build.getBuilder().getName()))            
@@ -211,13 +223,12 @@ class FeedResource(XmlResource):
             bc['number'] = build.getNumber()
             bc['responsible_users'] = build.getResponsibleUsers()
             bc['failed_steps'] = failed_steps
-            bc['title'] = title
+            bc['pageTitle'] = pageTitle
             bc['link'] = link
             bc['log_lines'] = log_lines
 
             if finishedTime is not None:
-                bc['rfc822_pubdate'] = time.strftime("%a, %d %b %Y %H:%M:%S GMT",
-                                              finishedTime)
+                bc['rfc822_pubdate'] = rfc822_time(finishedTime)
                 bc['rfc3339_pubdate'] = time.strftime("%Y-%m-%dT%H:%M:%SZ",
                                                finishedTime)
 
@@ -230,19 +241,18 @@ class FeedResource(XmlResource):
 
             build_cxts.append(bc)
 
-        title = self.title
-        if not title:
-            title = 'Build status of %s' % self.projectName
+        pageTitle = self.pageTitle
+        if not pageTitle:
+            pageTitle = 'Build status of %s' % self.title
 
         cxt = {}
-        cxt['title'] = title
-        cxt['project_url'] = self.link
-        cxt['project_name'] = self.projectName
+        cxt['pageTitle'] = pageTitle
+        cxt['title_url'] = self.link
+        cxt['title'] = self.title
         cxt['language'] = self.language
         cxt['description'] = self.description
         if self.pubdate is not None:
-            cxt['rfc822_pubdate'] = time.strftime("%a, %d %b %Y %H:%M:%S GMT",
-                                                  self.pubdate)
+            cxt['rfc822_pubdate'] = rfc822_time( self.pubdate)
             cxt['rfc3339_pubdate'] = time.strftime("%Y-%m-%dT%H:%M:%SZ",
                                                    self.pubdate)
 
@@ -254,12 +264,12 @@ class Rss20StatusResource(FeedResource):
     # contentType = 'application/rss+xml' (browser dependent)
     template_file = 'feed_rss20.xml'
 
-    def __init__(self, status, categories=None, title=None):
-        FeedResource.__init__(self, status, categories, title)
+    def __init__(self, status, categories=None, pageTitle=None):
+        FeedResource.__init__(self, status, categories, pageTitle)
 
 class Atom10StatusResource(FeedResource):
     # contentType = 'application/atom+xml' (browser dependent)
     template_file = 'feed_atom10.xml'
 
-    def __init__(self, status, categories=None, title=None):
-        FeedResource.__init__(self, status, categories, title)
+    def __init__(self, status, categories=None, pageTitle=None):
+        FeedResource.__init__(self, status, categories, pageTitle)
