@@ -15,8 +15,11 @@
 
 
 import re
+import inspect
 from twisted.python import log, failure
 from twisted.spread import pb
+from twisted.python.deprecate import deprecatedModuleAttribute
+from twisted.python.versions import Version
 from buildbot.process import buildstep
 from buildbot.status.results import SUCCESS, WARNINGS, FAILURE
 from buildbot.status.logfile import STDOUT, STDERR
@@ -31,7 +34,8 @@ del _hush_pyflakes
 class ShellCommand(buildstep.LoggingBuildStep):
     """I run a single shell command on the buildslave. I return FAILURE if
     the exit code of that command is non-zero, SUCCESS otherwise. To change
-    this behavior, override my .evaluateCommand method.
+    this behavior, override my .evaluateCommand method, or customize
+    decodeRC argument
 
     By default, a failure of this step will mark the whole build as FAILURE.
     To override this, give me an argument of flunkOnFailure=False .
@@ -61,13 +65,17 @@ class ShellCommand(buildstep.LoggingBuildStep):
                         they are written to. Empty or nonexistent logfiles
                         will be omitted. (Also handled by class
                         LoggingBuildStep.)
-
     """
 
     name = "shell"
-    renderables = [ 'description', 'descriptionDone', 'slaveEnvironment', 'remote_kwargs', 'command', 'logfiles' ]
+    renderables = buildstep.LoggingBuildStep.renderables + [
+                   'slaveEnvironment', 'remote_kwargs', 'command',
+                   'description', 'descriptionDone', 'descriptionSuffix']
+
     description = None # set this to a list of short strings to override
     descriptionDone = None # alternate description when the step is complete
+    descriptionSuffix = None # extra information to append to suffix
+
     command = None # set this to a command, or set in kwargs
     # logfiles={} # you can also set 'logfiles' to a dictionary, and it
     #               will be merged with any logfiles= argument passed in
@@ -78,7 +86,7 @@ class ShellCommand(buildstep.LoggingBuildStep):
     flunkOnFailure = True
 
     def __init__(self, workdir=None,
-                 description=None, descriptionDone=None,
+                 description=None, descriptionDone=None, descriptionSuffix=None,
                  command=None,
                  usePTY="slave-config",
                  **kwargs):
@@ -95,6 +103,12 @@ class ShellCommand(buildstep.LoggingBuildStep):
             self.descriptionDone = descriptionDone
         if isinstance(self.descriptionDone, str):
             self.descriptionDone = [self.descriptionDone]
+
+        if descriptionSuffix:
+            self.descriptionSuffix = descriptionSuffix
+        if isinstance(self.descriptionSuffix, str):
+            self.descriptionSuffix = [self.descriptionSuffix]
+
         if command:
             self.setCommand(command)
 
@@ -105,17 +119,22 @@ class ShellCommand(buildstep.LoggingBuildStep):
                 buildstep_kwargs[k] = kwargs[k]
                 del kwargs[k]
         buildstep.LoggingBuildStep.__init__(self, **buildstep_kwargs)
-        self.addFactoryArguments(workdir=workdir,
-                                 description=description,
-                                 descriptionDone=descriptionDone,
-                                 command=command)
+
+        # check validity of arguments being passed to RemoteShellCommand
+        invalid_args = []
+        valid_rsc_args = inspect.getargspec(buildstep.RemoteShellCommand.__init__)[0]
+        for arg in kwargs.keys():
+            if arg not in valid_rsc_args:
+                invalid_args.append(arg)
+        # Raise Configuration error in case invalid arguments are present
+        if invalid_args:
+            config.error("Invalid argument(s) passed to RemoteShellCommand: "
+                         + ', '.join(invalid_args))
 
         # everything left over goes to the RemoteShellCommand
         kwargs['workdir'] = workdir # including a copy of 'workdir'
         kwargs['usePTY'] = usePTY
         self.remote_kwargs = kwargs
-        # we need to stash the RemoteShellCommand's args too
-        self.addFactoryArguments(**kwargs)
 
     def setBuild(self, build):
         buildstep.LoggingBuildStep.setBuild(self, build)
@@ -142,12 +161,20 @@ class ShellCommand(buildstep.LoggingBuildStep):
 
     def _flattenList(self, mainlist, commands):
         for x in commands:
-          if isinstance(x, (str, unicode)):
-             mainlist.append(x)
-          elif x != []:
-             self._flattenList(mainlist, x)
+            if isinstance(x, (list, tuple)):
+                if x != []:
+                    self._flattenList(mainlist, x)
+            else:
+                mainlist.append(x)
 
     def describe(self, done=False):
+        desc = self._describe(done)
+        if self.descriptionSuffix:
+            desc = desc[:]
+            desc.extend(self.descriptionSuffix)
+        return desc
+
+    def _describe(self, done=False):
         """Return a list of short strings to describe this step, for the
         status display. This uses the first few words of the shell command.
         You can replace this by setting .description in your subclass, or by
@@ -185,20 +212,22 @@ class ShellCommand(buildstep.LoggingBuildStep):
                 # WithProperties and Property don't have __len__
                 return ["???"]
 
+            # flatten any nested lists
             tmp = []
-            for x in words:
-                if isinstance(x, (str, unicode)):
-                   tmp.append(x)
-                else:
-                   self._flattenList(tmp, x)
+            self._flattenList(tmp, words)
+            words = tmp
 
-            if len(tmp) < 1:
+            # strip instances and other detritus (which can happen if a
+            # description is requested before rendering)
+            words = [ w for w in words if isinstance(w, (str, unicode)) ]
+
+            if len(words) < 1:
                 return ["???"]
-            if len(tmp) == 1:
-                return ["'%s'" % tmp[0]]
-            if len(tmp) == 2:
-                return ["'%s" % tmp[0], "%s'" % tmp[1]]
-            return ["'%s" % tmp[0], "%s" % tmp[1], "...'"]
+            if len(words) == 1:
+                return ["'%s'" % words[0]]
+            if len(words) == 2:
+                return ["'%s" % words[0], "%s'" % words[1]]
+            return ["'%s" % words[0], "%s" % words[1], "...'"]
 
         except:
             log.err(failure.Failure(), "Error describing step")
@@ -276,7 +305,7 @@ class TreeSize(ShellCommand):
             self.setProperty("tree-size-KiB", self.kib, "treesize")
 
     def evaluateCommand(self, cmd):
-        if cmd.rc != 0:
+        if cmd.didFail():
             return FAILURE
         if self.kib is None:
             return WARNINGS # not sure how 'du' could fail, but whatever
@@ -287,8 +316,7 @@ class TreeSize(ShellCommand):
             return ["treesize", "%d KiB" % self.kib]
         return ["treesize", "unknown"]
 
-
-class SetProperty(ShellCommand):
+class SetPropertyFromCommand(ShellCommand):
     name = "setproperty"
     renderables = [ 'property' ]
 
@@ -303,15 +331,11 @@ class SetProperty(ShellCommand):
 
         ShellCommand.__init__(self, **kwargs)
 
-        self.addFactoryArguments(property=self.property)
-        self.addFactoryArguments(extract_fn=self.extract_fn)
-        self.addFactoryArguments(strip=self.strip)
-
         self.property_changes = {}
 
     def commandComplete(self, cmd):
         if self.property:
-            if cmd.rc != 0:
+            if cmd.didFail():
                 return
             result = cmd.logs['stdio'].getText()
             if self.strip: result = result.strip()
@@ -342,6 +366,13 @@ class SetProperty(ShellCommand):
             # let ShellCommand describe
             return ShellCommand.getText(self, cmd, results)
 
+
+SetProperty = SetPropertyFromCommand
+deprecatedModuleAttribute(Version("Buildbot", 0, 8, 8),
+        "It has been renamed to SetPropertyFromCommand",
+        "buildbot.steps.shell", "SetProperty")
+
+
 class Configure(ShellCommand):
 
     name = "configure"
@@ -405,12 +436,6 @@ class WarningCountingShellCommand(ShellCommand):
         # And upcall to let the base class do its work
         ShellCommand.__init__(self, **kwargs)
 
-        self.addFactoryArguments(warningPattern=warningPattern,
-                                 directoryEnterPattern=directoryEnterPattern,
-                                 directoryLeavePattern=directoryLeavePattern,
-                                 warningExtractor=warningExtractor,
-                                 maxWarnCount=maxWarnCount,
-                                 suppressionFile=suppressionFile)
         self.suppressions = []
         self.directoryStack = []
 
@@ -584,7 +609,7 @@ class WarningCountingShellCommand(ShellCommand):
 
 
     def evaluateCommand(self, cmd):
-        if ( cmd.rc != 0 or
+        if ( cmd.didFail() or
            ( self.maxWarnCount != None and self.warnCount > self.maxWarnCount ) ):
             return FAILURE
         if self.warnCount:
@@ -660,7 +685,7 @@ class PerlModuleTest(Test):
         passed = 0
         failed = 0
         rc = SUCCESS
-        if cmd.rc > 0:
+        if cmd.didFail():
             rc = FAILURE
 
         # New version of Test::Harness?
