@@ -30,3 +30,6 @@ class FakeBotMaster(service.MultiService):
         # that this requires that MasterLock and SlaveLock (marker) instances
         # be hashable and that they should compare properly.
         return self.locks[lockid]
+
+    def getLockFromLockAccess(self, access):
+        return self.getLockByID(access.lockid)
