@@ -13,7 +13,6 @@
 #
 # Copyright Buildbot Team Members
 
-import re
 import os
 import mock
 import signal
@@ -23,7 +22,7 @@ from twisted.python import log
 from buildbot import master, monkeypatches, config
 from buildbot.util import subscription
 from buildbot.db import connector
-from buildbot.test.util import dirs, compat, misc
+from buildbot.test.util import dirs, compat, misc, logging
 from buildbot.test.fake import fakedb
 from buildbot.util import epoch2datetime
 from buildbot.changes import changes
@@ -42,7 +41,7 @@ class Subscriptions(dirs.DirsMixin, unittest.TestCase):
 
     def tearDown(self):
         return self.tearDownDirs()
-
+        
     def test_change_subscription(self):
         changeid = 918
         chdict = {
@@ -82,9 +81,8 @@ class Subscriptions(dirs.DirsMixin, unittest.TestCase):
             # appropriate default values
             self.master.db.changes.addChange.assert_called_with(author=None,
                     files=None, comments=None, is_dir=0,
-                    revision=None, when_timestamp=None, branch=None,
-                    category=None, revlink='', properties={}, repository='',
-                    project='', uid=None)
+                    revision=None, when_timestamp=None, branch=None, codebase='',
+                    category=None, revlink='', properties={}, repository='', project='', uid=None)
 
             self.master.db.changes.getChange.assert_called_with(changeid)
             # addChange returned the right value
@@ -99,7 +97,7 @@ class Subscriptions(dirs.DirsMixin, unittest.TestCase):
         default_db_kwargs = dict(files=None, comments=None, author=None,
                 is_dir=0, revision=None, when_timestamp=None,
                 branch=None, category=None, revlink='', properties={},
-                repository='', project='', uid=None)
+                repository='', codebase='', project='', uid=None)
         k = default_db_kwargs
         k.update(exp_db_kwargs)
         exp_db_kwargs = k
@@ -158,7 +156,7 @@ class Subscriptions(dirs.DirsMixin, unittest.TestCase):
         return self.do_test_addChange_args(
                 args=('me', ['a'], 'com'),
                 exp_db_kwargs=dict(author='me', files=['a'], comments='com'))
-
+                
     def do_test_createUserObjects_args(self, args=(), kwargs={}, exp_args=()):
         got = []
         def fake_createUserObject(*args, **kwargs):
@@ -181,7 +179,7 @@ class Subscriptions(dirs.DirsMixin, unittest.TestCase):
         return self.do_test_createUserObjects_args(
                 kwargs=dict(who='me', src='git'),
                 exp_args=(self.master, 'me', 'git'))
-
+               
     def test_buildset_subscription(self):
         self.master.db = mock.Mock()
         self.master.db.buildsets.addBuildset.return_value = \
@@ -213,9 +211,10 @@ class Subscriptions(dirs.DirsMixin, unittest.TestCase):
         # assert the notification sub was called correctly
         cb.assert_called_with(938593, 999)
 
-class StartupAndReconfig(dirs.DirsMixin, unittest.TestCase):
+class StartupAndReconfig(dirs.DirsMixin, logging.LoggingMixin, unittest.TestCase):
 
     def setUp(self):
+        self.setUpLogging()
         self.basedir = os.path.abspath('basedir')
         d = self.setUpDirs(self.basedir)
         @d.addCallback
@@ -224,7 +223,7 @@ class StartupAndReconfig(dirs.DirsMixin, unittest.TestCase):
             self.patch(master.BuildMaster, 'create_child_services',
                     lambda self : None)
 
-            # patch out a few other annoying things the msater likes to do
+            # patch out a few other annoying things the master likes to do
             self.patch(monkeypatches, 'patch_all', lambda : None)
             self.patch(signal, 'signal', lambda sig, hdlr : None)
             self.patch(master, 'Status', lambda master : mock.Mock()) # XXX temporary
@@ -234,11 +233,6 @@ class StartupAndReconfig(dirs.DirsMixin, unittest.TestCase):
             self.master = master.BuildMaster(self.basedir)
             self.db = self.master.db = fakedb.FakeDBConnector(self)
 
-        @d.addCallback
-        def patch_log_msg(_):
-            msg = mock.Mock(side_effect=log.msg)
-            self.patch(log, 'msg', msg)
-
         return d
 
     def tearDown(self):
@@ -249,13 +243,6 @@ class StartupAndReconfig(dirs.DirsMixin, unittest.TestCase):
         r.callWhenRunning = reactor.callWhenRunning
         return r
 
-    def assertLogged(self, regexp):
-        r = re.compile(regexp)
-        for args, kwargs in log.msg.call_args_list:
-            if args and r.search(args[0]):
-                return
-        self.fail("%r not matched in log output" % regexp)
-
     def patch_loadConfig_fail(self):
         @classmethod
         def loadConfig(cls, b, f):
@@ -270,7 +257,6 @@ class StartupAndReconfig(dirs.DirsMixin, unittest.TestCase):
         self.patch_loadConfig_fail()
 
         d = self.master.startService(_reactor=reactor)
-        d.addCallback(lambda _ : self.master.stopService())
 
         @d.addCallback
         def check(_):
@@ -286,7 +272,6 @@ class StartupAndReconfig(dirs.DirsMixin, unittest.TestCase):
         self.db.setup = db_setup
 
         d = self.master.startService(_reactor=reactor)
-        d.addCallback(lambda _ : self.master.stopService())
 
         @d.addCallback
         def check(_):
@@ -302,7 +287,6 @@ class StartupAndReconfig(dirs.DirsMixin, unittest.TestCase):
         self.db.setup = db_setup
 
         d = self.master.startService(_reactor=reactor)
-        d.addCallback(lambda _ : self.master.stopService())
 
         @d.addCallback
         def check(_):
@@ -336,41 +320,37 @@ class StartupAndReconfig(dirs.DirsMixin, unittest.TestCase):
             self.master.reconfigService.assert_called()
         return d
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def test_reconfig_bad_config(self):
         reactor = self.make_reactor()
         self.master.reconfigService = mock.Mock(
                 side_effect=lambda n : defer.succeed(None))
 
-        wfd = defer.waitForDeferred(
-            self.master.startService(_reactor=reactor))
-        yield wfd
-        wfd.getResult()
-
+        yield self.master.startService(_reactor=reactor)
 
         # reset, since startService called reconfigService
         self.master.reconfigService.reset_mock()
 
         # reconfig, with a failure
         self.patch_loadConfig_fail()
-        wfd = defer.waitForDeferred(
-            self.master.reconfig())
-        yield wfd
-        wfd.getResult()
+        yield self.master.reconfig()
 
         self.master.stopService()
 
         self.assertLogged("reconfig aborted without")
         self.failIf(self.master.reconfigService.called)
 
+    @defer.inlineCallbacks
     def test_reconfigService_db_url_changed(self):
         old = self.master.config = config.MasterConfig()
         old.db['db_url'] = 'aaaa'
+        yield self.master.reconfigService(old)
+
         new = config.MasterConfig()
         new.db['db_url'] = 'bbbb'
 
         self.assertRaises(config.ConfigErrors, lambda :
-            self.master.reconfigService(new))
+                self.master.reconfigService(new))
 
     def test_reconfigService_start_polling(self):
         loopingcall = mock.Mock()
@@ -386,20 +366,20 @@ class StartupAndReconfig(dirs.DirsMixin, unittest.TestCase):
             loopingcall.start.assert_called_with(120, now=False)
         return d
 
+    @defer.inlineCallbacks
     def test_reconfigService_stop_polling(self):
         db_loop = self.master.db_loop = mock.Mock()
 
         old = self.master.config = config.MasterConfig()
         old.db['db_poll_interval'] = 120
+        yield self.master.reconfigService(old)
+
         new = config.MasterConfig()
         new.db['db_poll_interval'] = None
+        yield self.master.reconfigService(new)
 
-        d = self.master.reconfigService(new)
-        @d.addCallback
-        def check(_):
-            db_loop.stop.assert_called()
-            self.assertEqual(self.master.db_loop, None)
-        return d
+        db_loop.stop.assert_called()
+        self.assertEqual(self.master.db_loop, None)
 
 
 class Polling(dirs.DirsMixin, misc.PatcherMixin, unittest.TestCase):
