@@ -107,6 +107,13 @@ class BaseBasicScheduler(CommonStuffMixin,
         d.addCallback(lambda _ : sched.stopService())
         return d
 
+    def test_subclass_fileIsImportant(self):
+        class Subclass(self.Subclass):
+            def fileIsImportant(self, change):
+                return False
+        sched = self.makeScheduler(Subclass, onlyImportant=True)
+        self.failUnlessEqual(Subclass.fileIsImportant.__get__(sched), sched.fileIsImportant)
+
     def test_startService_treeStableTimer(self):
         cf = mock.Mock()
         sched = self.makeScheduler(self.Subclass, treeStableTimer=10, change_filter=cf)
@@ -129,6 +136,9 @@ class BaseBasicScheduler(CommonStuffMixin,
                                         onlyImportant=False)
             self.db.schedulers.assertClassifications(self.OBJECTID, { 20 : True })
             self.assertTrue(sched.timer_started)
+            self.assertEqual(sched.getPendingBuildTimes(), [ 10 ])
+            self.clock.advance(10)
+            self.assertEqual(sched.getPendingBuildTimes(), [])
         d.addCallback(check)
         d.addCallback(lambda _ : sched.stopService())
         return d
@@ -184,7 +194,7 @@ class BaseBasicScheduler(CommonStuffMixin,
 
         d.addCallback(lambda _ : sched.stopService())
 
-    @defer.deferredGenerator
+    @defer.inlineCallbacks
     def test_gotChange_treeStableTimer_sequence(self):
         sched = self.makeScheduler(self.Subclass, treeStableTimer=9, branch='master')
         self.master.db.insertTestData([
@@ -203,24 +213,22 @@ class BaseBasicScheduler(CommonStuffMixin,
 
         # this important change arrives at 2220, so the stable timer will last
         # until 2229
-        wfd = defer.waitForDeferred(
-            sched.gotChange(self.makeFakeChange(branch='master', number=1, when=2220),
-                            True))
-        yield wfd
-        wfd.getResult()
+        yield sched.gotChange(
+                self.makeFakeChange(branch='master', number=1, when=2220),
+                True)
         self.assertEqual(self.events, [])
+        self.assertEqual(sched.getPendingBuildTimes(), [2229])
         self.db.schedulers.assertClassifications(self.OBJECTID, { 1 : True })
 
         # but another (unimportant) change arrives before then
         self.clock.advance(6) # to 2226
         self.assertEqual(self.events, [])
 
-        wfd = defer.waitForDeferred(
-            sched.gotChange(self.makeFakeChange(branch='master', number=2, when=2226),
-                            False))
-        yield wfd
-        wfd.getResult()
+        yield sched.gotChange(
+                self.makeFakeChange(branch='master', number=2, when=2226),
+                False)
         self.assertEqual(self.events, [])
+        self.assertEqual(sched.getPendingBuildTimes(), [2235])
         self.db.schedulers.assertClassifications(self.OBJECTID, { 1 : True, 2 : False })
 
         self.clock.advance(3) # to 2229
@@ -230,12 +238,11 @@ class BaseBasicScheduler(CommonStuffMixin,
         self.assertEqual(self.events, [])
 
         # another important change arrives at 2232
-        wfd = defer.waitForDeferred(
-            sched.gotChange(self.makeFakeChange(branch='master', number=3, when=2232),
-                            True))
-        yield wfd
-        wfd.getResult()
+        yield sched.gotChange(
+                self.makeFakeChange(branch='master', number=3, when=2232),
+                True)
         self.assertEqual(self.events, [])
+        self.assertEqual(sched.getPendingBuildTimes(), [2241])
         self.db.schedulers.assertClassifications(self.OBJECTID, { 1 : True, 2 : False, 3 : True })
 
         self.clock.advance(3) # to 2235
@@ -244,11 +251,10 @@ class BaseBasicScheduler(CommonStuffMixin,
         # finally, time to start the build!
         self.clock.advance(6) # to 2241
         self.assertEqual(self.events, [ 'B[1,2,3]@2241' ])
+        self.assertEqual(sched.getPendingBuildTimes(), [])
         self.db.schedulers.assertClassifications(self.OBJECTID, { })
 
-        wfd = defer.waitForDeferred(sched.stopService())
-        yield wfd
-        wfd.getResult()
+        yield sched.stopService()
 
 
 class SingleBranchScheduler(CommonStuffMixin,
@@ -308,7 +314,7 @@ class AnyBranchScheduler(CommonStuffMixin,
                 lambda : basic.SingleBranchScheduler(name="tsched", treeStableTimer=60, branch='x'))
 
     def test_gotChange_treeStableTimer_multiple_branches(self):
-        # check that two changes with different branches have different treeStableTimers
+        """Two changes with different branches get different treeStableTimers"""
         sched = self.makeScheduler(basic.AnyBranchScheduler,
                             treeStableTimer=10, branches=['master', 'devel', 'boring'])
 
@@ -322,16 +328,22 @@ class AnyBranchScheduler(CommonStuffMixin,
         d = defer.succeed(None)
         d.addCallback(lambda _ :
                 sched.gotChange(mkch(branch='master', number=13), True))
+        d.addCallback(lambda _ :
+                self.assertEqual(sched.getPendingBuildTimes(), [10]))
         d.addCallback(lambda _ :
                 self.clock.advance(1)) # time is now 1
         d.addCallback(lambda _ :
                 sched.gotChange(mkch(branch='master', number=14), False))
+        d.addCallback(lambda _ :
+                self.assertEqual(sched.getPendingBuildTimes(), [11]))
         d.addCallback(lambda _ :
                 sched.gotChange(mkch(branch='boring', number=15), False))
         d.addCallback(lambda _ :
                 self.clock.pump([1]*4)) # time is now 5
         d.addCallback(lambda _ :
                 sched.gotChange(mkch(branch='devel', number=16), True))
+        d.addCallback(lambda _ :
+                self.assertEqual(sorted(sched.getPendingBuildTimes()), [11,15]))
         d.addCallback(lambda _ :
                 self.clock.pump([1]*10)) # time is now 15
         def check(_):
@@ -339,3 +351,102 @@ class AnyBranchScheduler(CommonStuffMixin,
         d.addCallback(check)
 
         d.addCallback(lambda _ : sched.stopService())
+
+    def test_gotChange_treeStableTimer_multiple_repositories(self):
+        """Two repositories, even with the same branch name, have different treeStableTimers"""
+        sched = self.makeScheduler(basic.AnyBranchScheduler,
+                            treeStableTimer=10, branches=['master'])
+
+        sched.startService()
+
+        def mkch(**kwargs):
+            ch = self.makeFakeChange(**kwargs)
+            self.db.changes.fakeAddChangeInstance(ch)
+            return ch
+
+        d = defer.succeed(None)
+        d.addCallback(lambda _ :
+                sched.gotChange(mkch(branch='master', repository="repo", number=13), True))
+        d.addCallback(lambda _ :
+                self.clock.advance(1)) # time is now 1
+        d.addCallback(lambda _ :
+                sched.gotChange(mkch(branch='master', repository="repo", number=14), False))
+        d.addCallback(lambda _ :
+                sched.gotChange(mkch(branch='master', repository="other_repo", number=15), False))
+        d.addCallback(lambda _ :
+                self.clock.pump([1]*4)) # time is now 5
+        d.addCallback(lambda _ :
+                sched.gotChange(mkch(branch='master', repository="other_repo", number=17), True))
+        d.addCallback(lambda _ :
+                self.clock.pump([1]*10)) # time is now 15
+        def check(_):
+            self.assertEqual(self.events, [ 'B[13,14]@11', 'B[15,17]@15' ])
+        d.addCallback(check)
+
+        d.addCallback(lambda _ : sched.stopService())
+
+    def test_gotChange_treeStableTimer_multiple_projects(self):
+        """Two projects, even with the same branch name, have different treeStableTimers"""
+        sched = self.makeScheduler(basic.AnyBranchScheduler,
+                            treeStableTimer=10, branches=['master'])
+
+        sched.startService()
+
+        def mkch(**kwargs):
+            ch = self.makeFakeChange(**kwargs)
+            self.db.changes.fakeAddChangeInstance(ch)
+            return ch
+
+        d = defer.succeed(None)
+        d.addCallback(lambda _ :
+                sched.gotChange(mkch(branch='master', project="proj", number=13), True))
+        d.addCallback(lambda _ :
+                self.clock.advance(1)) # time is now 1
+        d.addCallback(lambda _ :
+                sched.gotChange(mkch(branch='master', project="proj", number=14), False))
+        d.addCallback(lambda _ :
+                sched.gotChange(mkch(branch='master', project="other_proj", number=15), False))
+        d.addCallback(lambda _ :
+                self.clock.pump([1]*4)) # time is now 5
+        d.addCallback(lambda _ :
+                sched.gotChange(mkch(branch='master', project="other_proj", number=17), True))
+        d.addCallback(lambda _ :
+                self.clock.pump([1]*10)) # time is now 15
+        def check(_):
+            self.assertEqual(self.events, [ 'B[13,14]@11', 'B[15,17]@15' ])
+        d.addCallback(check)
+
+        d.addCallback(lambda _ : sched.stopService())
+
+    def test_gotChange_treeStableTimer_multiple_codebases(self):
+        """Two codebases, even with the same branch name, have different treeStableTimers"""
+        sched = self.makeScheduler(basic.AnyBranchScheduler,
+                            treeStableTimer=10, branches=['master'])
+
+        sched.startService()
+
+        def mkch(**kwargs):
+            ch = self.makeFakeChange(**kwargs)
+            self.db.changes.fakeAddChangeInstance(ch)
+            return ch
+
+        d = defer.succeed(None)
+        d.addCallback(lambda _ :
+                sched.gotChange(mkch(branch='master', codebase="base", number=13), True))
+        d.addCallback(lambda _ :
+                self.clock.advance(1)) # time is now 1
+        d.addCallback(lambda _ :
+                sched.gotChange(mkch(branch='master', codebase="base", number=14), False))
+        d.addCallback(lambda _ :
+                sched.gotChange(mkch(branch='master', codebase="other_base", number=15), False))
+        d.addCallback(lambda _ :
+                self.clock.pump([1]*4)) # time is now 5
+        d.addCallback(lambda _ :
+                sched.gotChange(mkch(branch='master', codebase="other_base", number=17), True))
+        d.addCallback(lambda _ :
+                self.clock.pump([1]*10)) # time is now 15
+        def check(_):
+            self.assertEqual(self.events, [ 'B[13,14]@11', 'B[15,17]@15' ])
+        d.addCallback(check)
+
+        d.addCallback(lambda _ : sched.stopService())
