@@ -20,12 +20,13 @@ from buildbot.steps import shell
 from buildbot.status.results import SKIPPED, SUCCESS, WARNINGS, FAILURE
 from buildbot.status.results import EXCEPTION
 from buildbot.test.util import steps, compat
+from buildbot.test.util import config as configmixin
 from buildbot.test.fake.remotecommand import ExpectShell, Expect
 from buildbot.test.fake.remotecommand import ExpectRemoteRef
 from buildbot import config
 from buildbot.process import properties
 
-class TestShellCommandExecution(steps.BuildStepMixin, unittest.TestCase):
+class TestShellCommandExecution(steps.BuildStepMixin, unittest.TestCase, configmixin.ConfigErrorsMixin):
 
     def setUp(self):
         return self.setUpBuildStep()
@@ -58,9 +59,17 @@ class TestShellCommandExecution(steps.BuildStepMixin, unittest.TestCase):
         # this is an ugly way to define an API, but for now check that
         # the RemoteCommand arguments are properly passed on
         step = shell.ShellCommand(workdir='build', command="echo hello",
-                abc=1, xyz=2)
-        self.assertEqual(step.remote_kwargs, dict(abc=1, xyz=2,
-                                workdir='build', usePTY='slave-config'))
+                want_stdout=0, logEnviron=False)
+        self.assertEqual(step.remote_kwargs, dict(want_stdout=0,
+                         logEnviron=False, workdir='build',
+                         usePTY='slave-config'))
+
+    def test_constructor_args_validity(self):
+        # this checks that an exception is raised for invalid arguments
+        self.assertRaisesConfigError(
+                    "Invalid argument(s) passed to RemoteShellCommand: ",
+                    lambda: shell.ShellCommand('build', "echo Hello World",
+                            wrongArg1=1, wrongArg2='two'))
 
     def test_describe_no_command(self):
         step = shell.ShellCommand(workdir='build')
@@ -129,11 +138,34 @@ class TestShellCommandExecution(steps.BuildStepMixin, unittest.TestCase):
         self.assertEqual((step.describe(), step.describe(done=True)),
                          (['echoing'], ['echoed']))
 
+    def test_describe_with_suffix(self):
+        step = shell.ShellCommand(command="echo hello", descriptionSuffix="suffix")
+        self.assertEqual((step.describe(), step.describe(done=True)),
+                        (["'echo", "hello'", 'suffix'],)*2)
+
+    def test_describe_custom_with_suffix(self):
+        step = shell.ShellCommand(command="echo hello",
+                                  description=["echoing"], descriptionDone=["echoed"],
+                                  descriptionSuffix="suffix")
+        self.assertEqual((step.describe(), step.describe(done=True)),
+                         (['echoing', 'suffix'], ['echoed', 'suffix']))
+
+    def test_describe_no_command_with_suffix(self):
+        step = shell.ShellCommand(workdir='build', descriptionSuffix="suffix")
+        self.assertEqual((step.describe(), step.describe(done=True)),
+                         (['???', 'suffix'],)*2)
+
     def test_describe_unrendered_WithProperties(self):
         step = shell.ShellCommand(command=properties.WithProperties(''))
         self.assertEqual((step.describe(), step.describe(done=True)),
                          (['???'],)*2)
 
+    def test_describe_unrendered_WithProperties_list(self):
+        step = shell.ShellCommand(
+                command=[ 'x', properties.WithProperties(''), 'y' ])
+        self.assertEqual((step.describe(), step.describe(done=True)),
+                         (["'x", "y'"],)*2)
+
     @compat.usesFlushLoggedErrors
     def test_describe_fail(self):
         step = shell.ShellCommand(command=object())
@@ -259,7 +291,24 @@ class TestShellCommandExecution(steps.BuildStepMixin, unittest.TestCase):
         )
         self.expectOutcome(result=SUCCESS, status_text=["'echo", "hello'"])
         return self.runStep()
-    
+
+    def test_run_decodeRC(self, rc=1, results=WARNINGS, extra_text = ["warnings"]):
+        self.setupStep(
+                shell.ShellCommand(workdir='build', command="echo hello",
+                decodeRC={1:WARNINGS}))
+        self.expectCommands(
+            ExpectShell(workdir='build', command='echo hello',
+                         usePTY="slave-config")
+            + rc
+        )
+        self.expectOutcome(result=results, status_text=["'echo", "hello'"]+extra_text)
+        return self.runStep()
+
+    def test_run_decodeRC_defaults(self):
+        return  self.test_run_decodeRC(2, FAILURE,extra_text=["failed"])
+
+    def test_run_decodeRC_defaults_0_is_failure(self):
+        return  self.test_run_decodeRC(0, FAILURE,extra_text=["failed"])
 
 
 
@@ -308,7 +357,7 @@ class TreeSize(steps.BuildStepMixin, unittest.TestCase):
                 status_text=["treesize", "unknown"])
         return self.runStep()
 
-class SetProperty(steps.BuildStepMixin, unittest.TestCase):
+class SetPropertyFromCommand(steps.BuildStepMixin, unittest.TestCase):
 
     def setUp(self):
         return self.setUpBuildStep()
@@ -318,10 +367,10 @@ class SetProperty(steps.BuildStepMixin, unittest.TestCase):
 
     def test_constructor_conflict(self):
         self.assertRaises(config.ConfigErrors, lambda :
-                shell.SetProperty(property='foo', extract_fn=lambda : None))
+                shell.SetPropertyFromCommand(property='foo', extract_fn=lambda : None))
 
     def test_run_property(self):
-        self.setupStep(shell.SetProperty(property="res", command="cmd"))
+        self.setupStep(shell.SetPropertyFromCommand(property="res", command="cmd"))
         self.expectCommands(
             ExpectShell(workdir='wkdir', usePTY='slave-config',
                         command="cmd")
@@ -335,7 +384,7 @@ class SetProperty(steps.BuildStepMixin, unittest.TestCase):
         return self.runStep()
 
     def test_run_property_no_strip(self):
-        self.setupStep(shell.SetProperty(property="res", command="cmd",
+        self.setupStep(shell.SetPropertyFromCommand(property="res", command="cmd",
                                          strip=False))
         self.expectCommands(
             ExpectShell(workdir='wkdir', usePTY='slave-config',
@@ -350,7 +399,7 @@ class SetProperty(steps.BuildStepMixin, unittest.TestCase):
         return self.runStep()
 
     def test_run_failure(self):
-        self.setupStep(shell.SetProperty(property="res", command="blarg"))
+        self.setupStep(shell.SetPropertyFromCommand(property="res", command="blarg"))
         self.expectCommands(
             ExpectShell(workdir='wkdir', usePTY='slave-config',
                         command="blarg")
@@ -366,7 +415,7 @@ class SetProperty(steps.BuildStepMixin, unittest.TestCase):
         def extract_fn(rc, stdout, stderr):
             self.assertEqual((rc, stdout, stderr), (0, 'startend', 'STARTEND'))
             return dict(a=1, b=2)
-        self.setupStep(shell.SetProperty(extract_fn=extract_fn, command="cmd"))
+        self.setupStep(shell.SetPropertyFromCommand(extract_fn=extract_fn, command="cmd"))
         self.expectCommands(
             ExpectShell(workdir='wkdir', usePTY='slave-config',
                         command="cmd")
@@ -386,7 +435,7 @@ class SetProperty(steps.BuildStepMixin, unittest.TestCase):
         def extract_fn(rc, stdout, stderr):
             self.assertEqual((rc, stdout, stderr), (3, '', ''))
             return dict(a=1, b=2)
-        self.setupStep(shell.SetProperty(extract_fn=extract_fn, command="cmd"))
+        self.setupStep(shell.SetPropertyFromCommand(extract_fn=extract_fn, command="cmd"))
         self.expectCommands(
             ExpectShell(workdir='wkdir', usePTY='slave-config',
                         command="cmd")
@@ -402,7 +451,7 @@ class SetProperty(steps.BuildStepMixin, unittest.TestCase):
         def extract_fn(rc, stdout, stderr):
             self.assertEqual((rc, stdout, stderr), (3, '', ''))
             return dict()
-        self.setupStep(shell.SetProperty(extract_fn=extract_fn, command="cmd"))
+        self.setupStep(shell.SetPropertyFromCommand(extract_fn=extract_fn, command="cmd"))
         self.expectCommands(
             ExpectShell(workdir='wkdir', usePTY='slave-config',
                         command="cmd")
@@ -417,7 +466,7 @@ class SetProperty(steps.BuildStepMixin, unittest.TestCase):
     def test_run_extract_fn_exception(self):
         def extract_fn(rc, stdout, stderr):
             raise RuntimeError("oh noes")
-        self.setupStep(shell.SetProperty(extract_fn=extract_fn, command="cmd"))
+        self.setupStep(shell.SetPropertyFromCommand(extract_fn=extract_fn, command="cmd"))
         self.expectCommands(
             ExpectShell(workdir='wkdir', usePTY='slave-config',
                         command="cmd")
@@ -431,6 +480,25 @@ class SetProperty(steps.BuildStepMixin, unittest.TestCase):
             self.assertEqual(len(self.flushLoggedErrors(RuntimeError)), 1))
         return d
 
+class SetPropertyDeprecation(unittest.TestCase):
+    """
+    Tests for L{shell.SetProperty}
+    """
+
+    def test_deprecated(self):
+        """
+        Accessing L{shell.SetProperty} reports a deprecation error.
+        """
+        shell.SetProperty
+        warnings = self.flushWarnings([self.test_deprecated])
+        self.assertEqual(len(warnings), 1)
+        self.assertIdentical(warnings[0]['category'], DeprecationWarning)
+        self.assertEqual(warnings[0]['message'],
+                "buildbot.steps.shell.SetProperty was deprecated in Buildbot 0.8.8: "
+                "It has been renamed to SetPropertyFromCommand"
+                )
+
+
 class Configure(unittest.TestCase):
 
     def test_class_attrs(self):
@@ -521,7 +589,7 @@ class WarningCountingShellCommand(steps.BuildStepMixin, unittest.TestCase):
 
         # Invoke the expected callbacks for the suppression file upload.  Note
         # that this assumes all of the remote_* are synchronous, but can be
-        # easily adapted to suit if that changes (using deferredGenerator)
+        # easily adapted to suit if that changes (using inlineCallbacks)
         def upload_behavior(command):
             writer = command.args['writer']
             writer.remote_write(supps_file)
