@@ -14,10 +14,12 @@
 # Copyright Buildbot Team Members
 
 from twisted.trial import unittest
+from twisted.python.reflect import namedModule
 from buildbot.steps.source import bzr
 from buildbot.status.results import SUCCESS, FAILURE
 from buildbot.test.util import sourcesteps
 from buildbot.test.fake.remotecommand import ExpectShell, Expect
+import os.path
 
 class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
 
@@ -51,6 +53,65 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             + 0,
                     )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', '100', 'Bzr')
+        return self.runStep()
+
+    def test_mode_full_win32path(self):
+        self.setupStep(
+            bzr.Bzr(repourl='http://bzr.squid-cache.org/bzr/squid3/trunk',
+                    mode='full', method='fresh'))
+        self.build.path_module = namedModule('ntpath')
+        self.expectCommands(
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', '--version'])
+            + 0,
+            Expect('stat', dict(file=r'wkdir\.bzr',
+                                logEnviron=True))
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'clean-tree', '--force'])
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'update'])
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'version-info', '--custom', "--template='{revno}"])
+            + ExpectShell.log('stdio',
+                stdout='100')
+            + 0,
+                    )
+        self.expectOutcome(result=SUCCESS, status_text=["update"])
+        return self.runStep()
+
+    def test_mode_full_timeout(self):
+        self.setupStep(
+            bzr.Bzr(repourl='http://bzr.squid-cache.org/bzr/squid3/trunk',
+                    mode='full', method='fresh', timeout=1))
+        self.expectCommands(
+            ExpectShell(workdir='wkdir',
+                        timeout=1,
+                        command=['bzr', '--version'])
+            + 0,
+            Expect('stat', dict(file='wkdir/.bzr',
+                                logEnviron=True))
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        timeout=1,
+                        command=['bzr', 'clean-tree', '--force'])
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        timeout=1,
+                        command=['bzr', 'update'])
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        timeout=1,
+                        command=['bzr', 'version-info', '--custom', "--template='{revno}"])
+            + ExpectShell.log('stdio',
+                stdout='100')
+            + 0,
+                    )
+        self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
 
     def test_mode_full_revision(self):
@@ -78,6 +139,7 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             + 0,
                     )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
 
     def test_mode_full_clean(self):
@@ -104,6 +166,7 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             + 0,
                     )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
 
     def test_mode_full_clean_revision(self):
@@ -131,6 +194,7 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             + 0,
                     )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
 
     def test_mode_full_fresh(self):
@@ -157,6 +221,7 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             + 0,
                     )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
 
     def test_mode_full_clobber(self):
@@ -181,6 +246,7 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             + 0,
                     )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
 
 
@@ -208,6 +274,60 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             + 0,
                     )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', '100', 'Bzr')
+        return self.runStep()
+
+
+    def test_mode_full_clobber_baseurl(self):
+        self.setupStep(
+            bzr.Bzr(baseURL='http://bzr.squid-cache.org/bzr/squid3',
+                    defaultBranch='trunk', mode='full', method='clobber'))
+        self.expectCommands(
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', '--version'])
+            + 0,
+            Expect('rmdir', dict(dir='wkdir',
+                                 logEnviron=True))
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'checkout',
+                                os.path.join('http://bzr.squid-cache.org/bzr/squid3', 'trunk'), '.'])
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'version-info', '--custom', "--template='{revno}"])
+            + ExpectShell.log('stdio',
+                stdout='100')
+            + 0,
+                    )
+        self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', '100', 'Bzr')
+        return self.runStep()
+
+
+    def test_mode_full_clobber_baseurl_nodefault(self):
+        self.setupStep(
+            bzr.Bzr(baseURL='http://bzr.squid-cache.org/bzr/squid3',
+                    defaultBranch='trunk', mode='full', method='clobber'),
+            args=dict(branch='branches/SQUID_3_0'))
+        self.expectCommands(
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', '--version'])
+            + 0,
+            Expect('rmdir', dict(dir='wkdir',
+                                 logEnviron=True))
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'checkout',
+                                os.path.join('http://bzr.squid-cache.org/bzr/squid3', 'branches/SQUID_3_0'), '.'])
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['bzr', 'version-info', '--custom', "--template='{revno}"])
+            + ExpectShell.log('stdio',
+                stdout='100')
+            + 0,
+                    )
+        self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
 
 
@@ -239,6 +359,7 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             + 0,
             )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
 
     def test_mode_incremental(self):
@@ -262,6 +383,7 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             + 0,
             )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
 
     def test_mode_incremental_revision(self):
@@ -286,6 +408,7 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             + 0,
             )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', '100', 'Bzr')
         return self.runStep()
 
     def test_mode_incremental_no_existing_repo(self):
@@ -310,6 +433,7 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             + 0,
             )
         self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', '100\n', 'Bzr')
         return self.runStep()
 
     def test_bad_revparse(self):
@@ -356,4 +480,3 @@ class TestBzr(sourcesteps.SourceStepMixin, unittest.TestCase):
             )
         self.expectOutcome(result=FAILURE, status_text=["updating"])
         return self.runStep()
-
