@@ -14,12 +14,13 @@
 # Copyright Buildbot Team Members
 
 from twisted.trial import unittest
+from twisted.python.reflect import namedModule
 from buildbot.steps.source import git
 from buildbot.status.results import SUCCESS, FAILURE
-from buildbot.test.util import sourcesteps
+from buildbot.test.util import config, sourcesteps
 from buildbot.test.fake.remotecommand import ExpectShell, Expect
 
-class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
+class TestGit(sourcesteps.SourceStepMixin, config.ConfigErrorsMixin, unittest.TestCase):
 
     def setUp(self):
         return self.setUpSourceStep()
@@ -47,7 +48,40 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'HEAD'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['git', 'rev-parse', 'HEAD'])
+            + ExpectShell.log('stdio',
+                stdout='f6ad368298bd941e934a41f3babc827b2aa95a1d')
+            + 0,
+        )
+        self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', 'f6ad368298bd941e934a41f3babc827b2aa95a1d', 'Git')
+        return self.runStep()
+
+    def test_mode_full_clean_win32path(self):
+        self.setupStep(
+                git.Git(repourl='http://github.com/buildbot/buildbot.git',
+                                    mode='full', method='clean'))
+        self.build.path_module = namedModule('ntpath')
+        self.expectCommands(
+            ExpectShell(workdir='wkdir',
+                        command=['git', '--version'])
+            + 0,
+            Expect('stat', dict(file=r'wkdir\.git',
+                                logEnviron=True))
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['git', 'clean', '-f', '-d'])
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['git', 'fetch', '-t',
+                                 'http://github.com/buildbot/buildbot.git',
+                                 'HEAD'])
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'rev-parse', 'HEAD'])
@@ -84,7 +118,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
             + 0,
             ExpectShell(workdir='wkdir',
                         timeout=1,
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 0,
             ExpectShell(workdir='wkdir',
                         timeout=1,
@@ -118,7 +152,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'HEAD'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'apply', '--index', '-p', '1'],
@@ -155,7 +189,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'HEAD'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'apply', '--index', '-p', '1'],
@@ -186,7 +220,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'test-branch'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'branch', '-M', 'test-branch'])
@@ -221,7 +255,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'HEAD'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + ExpectShell.log('stdio',
                 stderr="fatal: Could not parse object " 
                     "'b08076bc71c7813038f2cefedff9c5b678d225a8'.\n")
@@ -245,7 +279,6 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
             + 1,
             ExpectShell(workdir='wkdir',
                         command=['git', 'clone',
-                                 '--branch', 'HEAD',
                                  'http://github.com/buildbot/buildbot.git', '.'])
             + 0,
             ExpectShell(workdir='wkdir',
@@ -293,11 +326,11 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                         command=['git', '--version'])
             + 0,
             Expect('rmdir', dict(dir='wkdir',
-                                 logEnviron=True))
+                                 logEnviron=True,
+                                 timeout=1200))
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'clone',
-                                 '--branch', 'HEAD',
                                  'http://github.com/buildbot/buildbot.git',
                                  '.', '--progress'])
             + 0,
@@ -321,7 +354,8 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                         command=['git', '--version'])
             + 0,
             Expect('rmdir', dict(dir='wkdir',
-                                       logEnviron=True))
+                                       logEnviron=True,
+                                       timeout=1200))
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'clone',
@@ -356,7 +390,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'HEAD'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'rev-parse', 'HEAD'])
@@ -386,7 +420,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'test-branch'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'branch', '-M', 'test-branch'])
@@ -422,7 +456,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'HEAD'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'rev-parse', 'HEAD'])
@@ -451,7 +485,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                         command=['git', 'cat-file', '-e', 'abcdef01'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'abcdef01'])
+                        command=['git', 'reset', '--hard', 'abcdef01', '--'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'rev-parse', 'HEAD'])
@@ -483,7 +517,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'HEAD'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'submodule', 'update', '--recursive'])
@@ -512,11 +546,39 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                         command=['git', '--version'])
             + 0,
             Expect('rmdir', dict(dir='wkdir',
-                                 logEnviron=True))
+                                 logEnviron=True,
+                                 timeout=1200))
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'clone', '--depth', '1',
-                                 '--branch', 'HEAD',
+                                 'http://github.com/buildbot/buildbot.git',
+                                 '.'])
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['git', 'rev-parse', 'HEAD'])
+            + ExpectShell.log('stdio',
+                stdout='f6ad368298bd941e934a41f3babc827b2aa95a1d')
+            + 0,
+        )
+        self.expectOutcome(result=SUCCESS, status_text=["update"])
+        self.expectProperty('got_revision', 'f6ad368298bd941e934a41f3babc827b2aa95a1d', 'Git')
+        return self.runStep()
+
+    def test_mode_full_clobber_no_shallow(self):
+        self.setupStep(
+                git.Git(repourl='http://github.com/buildbot/buildbot.git',
+                        mode='full', method='clobber'))
+
+        self.expectCommands(
+            ExpectShell(workdir='wkdir',
+                        command=['git', '--version'])
+            + 0,
+            Expect('rmdir', dict(dir='wkdir',
+                                 logEnviron=True,
+                                 timeout=1200))
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=['git', 'clone',
                                  'http://github.com/buildbot/buildbot.git',
                                  '.'])
             + 0,
@@ -548,7 +610,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'HEAD'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 1,
             ExpectShell(workdir='wkdir',
                         command=['git', 'fetch', '-t',
@@ -556,7 +618,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'HEAD'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'rev-parse', 'HEAD'])
@@ -586,7 +648,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'test-branch'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 1,
             ExpectShell(workdir='wkdir',
                         command=['git', 'fetch', '-t',
@@ -594,7 +656,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'test-branch'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'branch', '-M', 'test-branch'])
@@ -627,14 +689,14 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'HEAD'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 1,
             Expect('rmdir', dict(dir='wkdir',
-                                       logEnviron=True))
+                                       logEnviron=True,
+                                       timeout=1200))
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'clone',
-                                 '--branch', 'HEAD',
                                  'http://github.com/buildbot/buildbot.git',
                                  '.'])
             + 0,
@@ -666,10 +728,11 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'test-branch'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 1,
             Expect('rmdir', dict(dir='wkdir',
-                                 logEnviron=True))
+                                 logEnviron=True,
+                                 timeout=1200))
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'clone',
@@ -690,14 +753,15 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
     def test_mode_full_copy(self):
         self.setupStep(
                 git.Git(repourl='http://github.com/buildbot/buildbot.git',
-                        mode='full', method='copy', shallow=True))
+                        mode='full', method='copy'))
 
         self.expectCommands(
             ExpectShell(workdir='wkdir',
                         command=['git', '--version'])
             + 0,
             Expect('rmdir', dict(dir='wkdir',
-                                 logEnviron=True)),
+                                 logEnviron=True,
+                                 timeout=1200)),
             Expect('stat', dict(file='source/.git',
                                 logEnviron=True))
             + 0,
@@ -707,10 +771,10 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'HEAD'])
             + 0,
             ExpectShell(workdir='source',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 0,
             Expect('cpdir', {'fromdir': 'source', 'todir': 'build',
-                             'logEnviron': True})
+                             'logEnviron': True, 'timeout': 1200})
             + 0,
             ExpectShell(workdir='build',
                         command=['git', 'rev-parse', 'HEAD'])
@@ -722,6 +786,10 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
         self.expectProperty('got_revision', 'f6ad368298bd941e934a41f3babc827b2aa95a1d', 'Git')
         return self.runStep()
 
+    def test_mode_full_copy_shallow(self):
+        self.assertRaisesConfigError("shallow only possible with mode 'full' and method 'clobber'", lambda :
+                git.Git(repourl='http://github.com/buildbot/buildbot.git',
+                        mode='full', method='copy', shallow=True))
 
     def test_mode_incremental_no_existing_repo(self):
         self.setupStep(
@@ -736,7 +804,6 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
             + 1,
             ExpectShell(workdir='wkdir',
                         command=['git', 'clone',
-                                 '--branch', 'HEAD',
                                  'http://github.com/buildbot/buildbot.git', '.'])
             + 0,
             ExpectShell(workdir='wkdir',
@@ -761,16 +828,16 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                         command=['git', '--version'])
             + 0,
             Expect('rmdir', dict(dir='wkdir',
-                                 logEnviron=True))
+                                 logEnviron=True,
+                                 timeout=1200))
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'clone',
-                                 '--branch', 'HEAD',
                                  'http://github.com/buildbot/buildbot.git',
                                  '.', '--progress'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'abcdef01'])
+                        command=['git', 'reset', '--hard', 'abcdef01', '--'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'rev-parse', 'HEAD'])
@@ -793,16 +860,16 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                         command=['git', '--version'])
             + 0,
             Expect('rmdir', dict(dir='wkdir',
-                                 logEnviron=True))
+                                 logEnviron=True,
+                                 timeout=1200))
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'clone',
-                                 '--branch', 'HEAD',
                                  'http://github.com/buildbot/buildbot.git',
                                  '.', '--progress'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'abcdef01'])
+                        command=['git', 'reset', '--hard', 'abcdef01', '--'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'rev-parse', 'HEAD'])
@@ -824,11 +891,11 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                         command=['git', '--version'])
             + 0,
             Expect('rmdir', dict(dir='wkdir',
-                                 logEnviron=True))
+                                 logEnviron=True,
+                                 timeout=1200))
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'clone',
-                                 '--branch', 'HEAD',
                                  'http://github.com/buildbot/buildbot.git', '.'])
             + 0,
             ExpectShell(workdir='wkdir',
@@ -846,7 +913,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
         return self.runStep()
 
     def test_repourl(self):
-        self.assertRaises(AssertionError, lambda :
+        self.assertRaisesConfigError("must provide repourl", lambda :
                 git.Git(mode="full"))
 
     def test_mode_full_fresh_revision(self):
@@ -864,12 +931,11 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
             + 1,
             ExpectShell(workdir='wkdir',
                         command=['git', 'clone',
-                                 '--branch', 'HEAD',
                                  'http://github.com/buildbot/buildbot.git',
                                  '.', '--progress'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'abcdef01'])
+                        command=['git', 'reset', '--hard', 'abcdef01', '--'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'rev-parse', 'HEAD'])
@@ -895,15 +961,14 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
             + 1,
             ExpectShell(workdir='wkdir',
                         command=['git', 'clone',
-                                 '--branch', 'HEAD',
                                  'http://github.com/buildbot/buildbot.git', '.'])
             + 1,
             Expect('rmdir', dict(dir='wkdir',
-                                 logEnviron=True))
+                                 logEnviron=True,
+                                 timeout=1200))
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'clone', 
-                                 '--branch', 'HEAD',
                                  'http://github.com/buildbot/buildbot.git',
                                  '.'])
             + 0,
@@ -937,7 +1002,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'HEAD'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'rev-parse', 'HEAD'])
@@ -972,7 +1037,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                         env={'abc': '123'})
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'],
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'],
                         env={'abc': '123'})
             + 0,
             ExpectShell(workdir='wkdir',
@@ -1009,7 +1074,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                         logEnviron=False)
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'],
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'],
                         logEnviron=False)
             + 0,
             ExpectShell(workdir='wkdir',
@@ -1045,7 +1110,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'HEAD'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'rev-parse', 'HEAD'])
@@ -1090,7 +1155,7 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                                  'HEAD'])
             + 0,
             ExpectShell(workdir='wkdir',
-                        command=['git', 'reset', '--hard', 'FETCH_HEAD'])
+                        command=['git', 'reset', '--hard', 'FETCH_HEAD', '--'])
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'rev-parse', 'HEAD'])
@@ -1130,11 +1195,11 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
                         command=['git', '--version'])
             + 0,
             Expect('rmdir', dict(dir='wkdir',
-                                 logEnviron=True))
+                                 logEnviron=True,
+                                 timeout=1200))
             + 0,
             ExpectShell(workdir='wkdir',
                         command=['git', 'clone',
-                                 '--branch', 'HEAD',
                                  'http://github.com/buildbot/buildbot.git',
                                  '.', '--progress'])
             + 0,
@@ -1354,3 +1419,39 @@ class TestGit(sourcesteps.SourceStepMixin, unittest.TestCase):
             codebase='baz'
         )
         return self.runStep()
+
+    def test_config_option(self):
+        name = 'url.http://github.com.insteadOf'
+        value = 'blahblah'
+        self.setupStep(
+                git.Git(repourl='%s/buildbot/buildbot.git' % (value,),
+                                    mode='full', method='clean',
+                                    config={name: value}))
+        prefix = ['git', '-c', '%s=%s' % (name, value)]
+        self.expectCommands(
+            ExpectShell(workdir='wkdir',
+                        command=prefix + ['--version'])
+            + 0,
+            Expect('stat', dict(file='wkdir/.git',
+                                logEnviron=True))
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=prefix + ['clean', '-f', '-d'])
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=prefix + ['fetch', '-t',
+                                 '%s/buildbot/buildbot.git' % (value,),
+                                 'HEAD'])
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=prefix + ['reset', '--hard',
+                                 'FETCH_HEAD', '--'])
+            + 0,
+            ExpectShell(workdir='wkdir',
+                        command=prefix + ['rev-parse', 'HEAD'])
+            + ExpectShell.log('stdio',
+                stdout='f6ad368298bd941e934a41f3babc827b2aa95a1d')
+            + 0,
+        )
+        self.expectOutcome(result=SUCCESS, status_text=["update"])
+        return self.runStep()
