@@ -38,6 +38,16 @@ class TestMaildirService(dirs.DirsMixin, unittest.TestCase):
 
     # tests
 
+    @defer.inlineCallbacks
+    def test_start_stop_repeatedly(self):
+        self.svc = maildir.MaildirService(self.maildir)
+        self.svc.startService()
+        yield self.svc.stopService()
+        self.svc.startService()
+        yield self.svc.stopService()
+        self.assertEqual(len(list(self.svc)), 0)
+
+
     def test_messageReceived(self):
         self.svc = maildir.MaildirService(self.maildir)
 
