@@ -16,6 +16,7 @@
 import mock
 from buildbot.test.util import steps
 
+import os
 
 class SourceStepMixin(steps.BuildStepMixin):
     """
@@ -54,7 +55,7 @@ class SourceStepMixin(steps.BuildStepMixin):
         ss.patch = patch
         ss.patch_info = None
         ss.changes = []
-
+        self.build.path_module = os.path
         self.build.getSourceStamp = lambda x=None: ss
         return step
 
