@@ -16,7 +16,7 @@
 import mock
 from buildbot import interfaces
 from buildbot.process import buildstep
-from buildbot.test.fake import remotecommand, fakebuild
+from buildbot.test.fake import remotecommand, fakebuild, slave
 
 
 class BuildStepMixin(object):
@@ -71,10 +71,8 @@ class BuildStepMixin(object):
 
         @param slave_env: environment from the slave at slave startup
         """
-        # yes, Virginia, "factory" refers both to the tuple and its first
-        # element TODO: fix that up
-        factory, args = step.getStepFactory()
-        step = self.step = factory(**args)
+        factory = interfaces.IBuildStepFactory(step)
+        step = self.step = factory.buildStep()
 
         # step.build
 
@@ -98,7 +96,7 @@ class BuildStepMixin(object):
 
         # step.buildslave
 
-        self.buildslave = step.buildslave = mock.Mock(name="buildslave")
+        self.buildslave = step.buildslave = slave.FakeSlave()
 
         # step.step_status
 
@@ -214,17 +212,17 @@ class BuildStepMixin(object):
                              "assert all expected commands were run")
             got_outcome = dict(result=result,
                         status_text=self.step_status.status_text)
-            self.assertEqual(got_outcome, self.exp_outcome)
+            self.assertEqual(got_outcome, self.exp_outcome, "expected step outcome")
             for pn, (pv, ps) in self.exp_properties.iteritems():
-                self.failUnless(self.properties.hasProperty(pn),
-                        "missing %s" % pn)
-                self.assertEqual(self.properties.getProperty(pn), pv)
+                self.assertTrue(self.properties.hasProperty(pn),
+                        "missing property '%s'" % pn)
+                self.assertEqual(self.properties.getProperty(pn), pv, "property '%s'" % pn)
                 if ps is not None:
-                    self.assertEqual(self.properties.getPropertySource(pn), ps)
+                    self.assertEqual(self.properties.getPropertySource(pn), ps, "property '%s' source" % pn)
             for pn in self.exp_missing_properties:
-                self.failIf(self.properties.hasProperty(pn))
+                self.assertFalse(self.properties.hasProperty(pn), "unexpected property '%s'" % pn)
             for log, contents in self.exp_logfiles.iteritems():
-                self.assertEqual(self.step_status.logs[log].stdout, contents)
+                self.assertEqual(self.step_status.logs[log].stdout, contents, "log '%s' contents" % log)
             self.step_status.setHidden.assert_called_once_with(self.exp_hidden)
         d.addCallback(check)
         return d
@@ -240,10 +238,19 @@ class BuildStepMixin(object):
             self.fail("got command %r when no further commands were expected"
                     % (got,))
 
-        # first check any ExpectedRemoteReference instances
         exp = self.expected_remote_commands.pop(0)
+
+        # handle any incomparable args
+        for arg in exp.incomparable_args:
+            self.failUnless(arg in got[1],
+                    "incomparable arg '%s' not received" % (arg,))
+            del got[1][arg]
+
+        # first check any ExpectedRemoteReference instances
         self.assertEqual((exp.remote_command, exp.args), got)
 
         # let the Expect object show any behaviors that are required
-        return exp.runBehaviors(command)
+        d = exp.runBehaviors(command)
+        d.addCallback(lambda _: command)
+        return d
 
