@@ -143,6 +143,9 @@ class BBService(win32serviceutil.ServiceFramework):
             if not os.path.isfile(python_exe):
                 # for ppl who build Python itself from source.
                 python_exe = os.path.join(sys.prefix, "PCBuild", "python.exe")
+            if not os.path.isfile(python_exe):
+                # virtualenv support
+                python_exe = os.path.join(sys.prefix, "Scripts", "python.exe")
             if not os.path.isfile(python_exe):
                 self.error("Can not find python.exe to spawn subprocess")
                 return False
@@ -226,7 +229,7 @@ class BBService(win32serviceutil.ServiceFramework):
             self.info("Starting BuildBot in directory '%s'" % (bbdir, ))
             hstop = self.hWaitStop
 
-            cmd = '%s --spawn %d start %s' % (self.runner_prefix, hstop, bbdir)
+            cmd = '%s --spawn %d start --nodaemon %s' % (self.runner_prefix, hstop, bbdir)
             #print "cmd is", cmd
             h, t, output = self.createProcess(cmd)
             child_infos.append((bbdir, h, t, output))
@@ -342,7 +345,7 @@ class BBService(win32serviceutil.ServiceFramework):
         self._dolog(servicemanager.LogErrorMsg, s)
 
     # Functions that spawn a child process, redirecting any output.
-    # Although builtbot itself does this, it is very handy to debug issues
+    # Although buildbot itself does this, it is very handy to debug issues
     # such as ImportErrors that happen before buildbot has redirected.
 
     def createProcess(self, cmd):
@@ -517,24 +520,24 @@ def _WaitForShutdown(h):
     reactor.callLater(0, reactor.stop)
 
 def DetermineRunner(bbdir):
-   '''Checks if the given directory is a buildslave or a master and returns the
-   appropriate run function.'''
-   try:
-      import buildslave.scripts.runner
-      tacfile = os.path.join(bbdir, 'buildbot.tac')
+    '''Checks if the given directory is a buildslave or a master and returns the
+    appropriate run function.'''
+    try:
+        import buildslave.scripts.runner
+        tacfile = os.path.join(bbdir, 'buildbot.tac')
 
-      if os.path.exists(tacfile):
-         with open(tacfile, 'r') as f:
-            contents = f.read()
-            if 'import BuildSlave' in contents:
-               return buildslave.scripts.runner.run
+        if os.path.exists(tacfile):
+            with open(tacfile, 'r') as f:
+                contents = f.read()
+                if 'import BuildSlave' in contents:
+                    return buildslave.scripts.runner.run
 
-   except ImportError:
-      # Use the default
-      pass
+    except ImportError:
+        # Use the default
+        pass
 
-   import buildbot.scripts.runner
-   return buildbot.scripts.runner.run
+    import buildbot.scripts.runner
+    return buildbot.scripts.runner.run
 
 # This function is also called by the py2exe startup code.
 
@@ -545,7 +548,7 @@ def HandleCommandLine():
         # child-process.
         # First arg is the handle to wait on
         # Fourth arg is the config directory to use for the buildbot/slave
-        _RunChild(DetermineRunner(sys.argv[4]))
+        _RunChild(DetermineRunner(sys.argv[5]))
     else:
         win32serviceutil.HandleCommandLine(BBService,
                                            customOptionHandler=CustomInstall)
